package inescID.crypto;

import java.io.*;
import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import org.apache.soap.encoding.soapenc.Base64;
import java.math.BigInteger;


/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

public class EncryptionTool {

  public static final int DEFAULT_RSA_KEY_SIZE = 1024;
  public static final String RANDOM_ALGORITHM = "SHA1PRNG";
  public static final String RSA_ALGORITHM = "RSA";
  public static final String TRIPLE_DES = "DESede";


  KeyPairGenerator RSAKeyPairGenerator;
  KeyFactory RSAKeyFactory;
  SecureRandom secureRandom;


  public EncryptionTool() throws Exception{
    this.RSAKeyPairGenerator = KeyPairGenerator.getInstance(RSA_ALGORITHM);
    this.RSAKeyFactory = KeyFactory.getInstance(RSA_ALGORITHM);
    this.secureRandom = SecureRandom.getInstance(RANDOM_ALGORITHM);
  }

  String ByteToString(byte[] b) {
    return Base64.encode(b);
  }

  byte[] StringToByte(String s) throws Exception {
    return Base64.decode(s);
  }



  public KeyPair genRSAKeyPair() throws Exception{
    return this.genRSAKeyPair(DEFAULT_RSA_KEY_SIZE);
  }

  public KeyPair genRSAKeyPair(int size) throws Exception{
    this.RSAKeyPairGenerator.initialize(size, this.secureRandom);
    KeyPair kp = this.RSAKeyPairGenerator.generateKeyPair();
    return kp;
  }


  public RSAPrivateKey getRSAPrivateKey(byte[] chave) throws
      InvalidKeySpecException {
    if (chave == null)
      return null;
    PKCS8EncodedKeySpec kprispec = new PKCS8EncodedKeySpec(chave);
    RSAPrivateKey kpri = (RSAPrivateKey) this.RSAKeyFactory.generatePrivate(kprispec);
    return kpri;
  }

  public RSAPublicKey getRSAPublicKey(byte[] chave) throws
      InvalidKeySpecException {
    if (chave == null)
      return null;
    X509EncodedKeySpec kpubspec = new X509EncodedKeySpec(chave);
    RSAPublicKey kpub = (RSAPublicKey) this.RSAKeyFactory.generatePublic(kpubspec);
    return kpub;
  }

  public byte[] ObjectToByteArray(Object obj) throws
       Exception {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    ObjectOutputStream objOut = new ObjectOutputStream(out);
    objOut.writeObject(obj);
    objOut.flush();
    objOut.close();
    return out.toByteArray();
  }


  public Object ByteArrayToObject(byte[] object_bytes) throws
      Exception {
    ByteArrayInputStream in = new ByteArrayInputStream(object_bytes);
    ObjectInputStream objIn = new ObjectInputStream(in);
    Object obj = objIn.readObject();
    return obj;
  }


/*
 *Simetric cipher
 *
 */

  public String getTripleDESkeyString() throws Exception {
    KeyGenerator kgen = KeyGenerator.getInstance(TRIPLE_DES);
    SecretKey skey = kgen.generateKey();
    return this.ByteToString(skey.getEncoded());
  }


  public SecretKey getTripleDESkey() throws Exception {
    KeyGenerator kgen = KeyGenerator.getInstance(TRIPLE_DES);
    SecretKey skey = kgen.generateKey();
    return skey;
  }

  public SecretKey getTripleDESkey(String key) throws Exception {
   byte[] raw = this.StringToByte(key);
   SecretKeySpec skeySpec = new SecretKeySpec(raw, TRIPLE_DES);
   return skeySpec;
 }

 public SecretKey getTripleDESkey(byte[] key) throws Exception {
   //DESedeKeySpec skeySpec = new DESedeKeySpec(key);
   SecretKeySpec skeySpec = new SecretKeySpec(key, 0, 24, TRIPLE_DES);
   return  skeySpec;
 }


 public byte[] EncryptTripleDES(Object obj, SecretKey key) throws
     Exception {
  byte[] obj_bytes = ObjectToByteArray(obj);
  return this.EncryptTripleDES(obj_bytes, key);
}


  public byte[] EncryptTripleDES(byte[] data, SecretKey key) throws
      Exception {
   Cipher cipher = Cipher.getInstance(TRIPLE_DES);
   cipher.init(Cipher.ENCRYPT_MODE, key);
   byte[] encrypted =
       cipher.doFinal(data);
   return encrypted;
 }



 public byte[] DecryptTripleDES(byte[] object_encrypted, SecretKey key) throws
     Exception {
   Cipher cipher = Cipher.getInstance(TRIPLE_DES);
   cipher.init(Cipher.DECRYPT_MODE, key);
   byte[] decrypted =
       cipher.doFinal(object_encrypted);
   return decrypted;
 }


 public MixCipherEnvelope EncryptMixCipherMode(Object obj, RSAKey key) throws
      Exception {
  return this.EncryptMixCipherMode(this.ObjectToByteArray(obj), key);
}


 public MixCipherEnvelope EncryptMixCipherMode(byte[] data, RSAKey key) throws
      Exception {
  SecretKey secret_key = this.getTripleDESkey();
  byte[] cipher = this.EncryptTripleDES(data, secret_key);
  byte[] cipher_key = this.EncryptRSA_NoPad(secret_key.getEncoded(), key);
  return new MixCipherEnvelope(cipher_key, cipher);
}

public byte[] DecryptMixCipherMode(MixCipherEnvelope envelope, RSAKey key) throws
     Exception {

   SecretKey secret_key = this.getTripleDESkey(
      this.DecryptRSA_NoPad(envelope.getkey(), key));
   byte[] data = this.DecryptTripleDES(envelope.getData(), secret_key);
   return data;
}



public BigInteger EncryptRSA_NoPad(BigInteger data, RSAKey key) throws
      Exception {
  return this.RSAEncryption(data, key);
}

public byte[] EncryptRSA_NoPad(byte[] data, RSAKey key) throws Exception {
  BigInteger aux = new BigInteger(1, data);
  BigInteger cipher = this.EncryptRSA_NoPad(aux, key);
  return cipher.toByteArray();
}


public BigInteger DecryptRSA_NoPad(BigInteger data, RSAKey key) throws
    Exception {
  return this.RSAEncryption(data, key);
}

public byte[] DecryptRSA_NoPad(byte[] data, RSAKey key) throws Exception {
  BigInteger aux = new BigInteger(1, data);
  BigInteger cipher = this.DecryptRSA_NoPad(aux, key);
  byte[] msg = cipher.toByteArray();
  if (msg[0] != 0)
    return msg;
  byte[] clear_msg = new byte[msg.length-1];
  System.arraycopy(msg, 1, clear_msg, 0, clear_msg.length);
  return clear_msg;
}


 public BigInteger RSAEncryption(BigInteger data, RSAKey key) throws Exception {
 //  System.err.println("A cifrar " + data);

   BigInteger exponent;
   try {
     RSAPrivateKey kpri = (RSAPrivateKey) key;
     exponent = kpri.getPrivateExponent();
   }
   catch (Exception ex) {
     RSAPublicKey kpub = (RSAPublicKey) key;
     exponent = kpub.getPublicExponent();
   }

   if (key.getModulus().compareTo(data)!=1)
     throw new Exception("Data to large for key");

   BigInteger cipher = data.modPow(exponent, key.getModulus());
//   System.err.println("Cifrado " + cipher);
   return cipher;
}
}
