package inescID.crypto;

import java.io.*;
import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import java.security.cert.*;

/**
 * <p>Title: revs</p>
 * <p>Description: Auxiliar na interaccao com um KeyStore</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

public class KeyStoreTool {
  public static final int DEFAULT_KEY_SIZE = 1024;
  public static final String RSA_ALGORITHM = "RSA";

  KeyStore ks;
  KeyPairGenerator RSAKeyPairGenerator;
  KeyFactory RSAKeyFactory;

  public KeyStoreTool() throws NoSuchAlgorithmException {
    this.init();
  }

  public KeyStoreTool(String keyStorePath, char[] passwd) throws
      FileNotFoundException, KeyStoreException, CertificateException,
      NoSuchAlgorithmException, IOException {

    FileInputStream f = new FileInputStream(keyStorePath);
    this.ks = KeyStore.getInstance(KeyStore.getDefaultType());
    this.ks.load(f, passwd);
    f.close();

    this.init();

  }

  protected void init() throws NoSuchAlgorithmException {
    this.RSAKeyPairGenerator = KeyPairGenerator.getInstance(RSA_ALGORITHM);
    this.RSAKeyFactory = KeyFactory.getInstance(RSA_ALGORITHM);
  }

  /**@todo
   * retirar geracao de chaves
   */

public KeyPair genRSAKeyPair() {
    return this.genRSAKeyPair(DEFAULT_KEY_SIZE);
  }

  public KeyPair genRSAKeyPair(int size) {
    this.RSAKeyPairGenerator.initialize(size);
    KeyPair kp = this.RSAKeyPairGenerator.generateKeyPair();
    return kp;
  }


  public PublicKey getPublicKey(String alias) throws
      UnrecoverableKeyException, NoSuchAlgorithmException, KeyStoreException {
  PublicKey pub =  ks.getCertificate(alias).getPublicKey();
  return pub;
}

  public PrivateKey getPrivateKey(String alias, char[] passwd) throws
      UnrecoverableKeyException, NoSuchAlgorithmException, KeyStoreException {
    PrivateKey pri = (PrivateKey) ks.getKey(alias, passwd);
    return pri;
  }


  public RSAPrivateKey getRSAPrivateKey(byte[] chave) throws
      InvalidKeySpecException {
   if (chave == null)
      return null;

    PKCS8EncodedKeySpec kprispec = new PKCS8EncodedKeySpec(chave);
    RSAPrivateKey kpri = (RSAPrivateKey) this.RSAKeyFactory.generatePrivate(kprispec);
    return kpri;
  }

  public RSAPublicKey getRSAPublicKey(byte[] chave) throws
      InvalidKeySpecException {
    if (chave == null)
      return null;

    X509EncodedKeySpec kpubspec = new X509EncodedKeySpec(chave);
    RSAPublicKey kpub = (RSAPublicKey) this.RSAKeyFactory.generatePublic(kpubspec);
    return kpub;
  }

  public PublicKey getPublicKeyFromCertFile(String file){
    try {
      InputStream in = new FileInputStream(file);
      CertificateFactory cf = CertificateFactory.getInstance("X.509");
      X509Certificate cert = (X509Certificate) cf.generateCertificate(in);
      return cert.getPublicKey();
    }
    catch (Exception ex) {
          return null;
    }
  }
}
