package inescID.crypto;

/**
 * <p>Title: REVS</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */


import java.security.*;
import org.apache.soap.encoding.soapenc.Base64;
import java.math.BigInteger;

public class PasswordGenerator {

  public static final String DEFAULT_HASH_ALGORITHM = "SHA-1";
  private static final int DEFAULT_PASSWORD_SIZE = 12;
  private static final int DEFAULT_PIN_SIZE = 6;
  private static final int MINIMAL_SIZE = 4;

  MessageDigest md;
  SecureRandom randomGenerator = new SecureRandom();

  public PasswordGenerator() throws NoSuchAlgorithmException {
    md = MessageDigest.getInstance(DEFAULT_HASH_ALGORITHM);
  }

  public PasswordGenerator(String digest) throws NoSuchAlgorithmException {
    md = MessageDigest.getInstance(digest);
  }

  public byte[] getHashValue(byte[] data){
      md.reset();
      md.update(data);
      return md.digest();
  }

  public byte[] getCommittedHashValue(byte[] data, byte[] commitment){
    md.reset();
    md.update(data);
    md.update(commitment);
    return md.digest();
}


  public String CreateRandomStringPin() throws Exception {
      String aux = Base64.encode(CreateRandomPin());
      return aux.substring(0, DEFAULT_PIN_SIZE);
     }

     public String CreateRandomStringPin(int size) throws Exception{
       String aux = Base64.encode(CreateRandomPin(size));
       return aux.substring(0, size);
     }


    public String CreateRandomStringPassword() throws Exception{
      String aux = Base64.encode(CreateRandomPassword());
      return aux.substring(0, DEFAULT_PASSWORD_SIZE);
    }

    public String CreateRandomStringPassword(int size) throws Exception{
      String aux = Base64.encode(CreateRandomPassword(size));
      return aux.substring(0, size);
    }


  public byte[] CreateRandomPassword() throws Exception{
    return CreateRandomPassword(DEFAULT_PASSWORD_SIZE);
  }

  public byte[] CreateRandomPin() throws Exception{
    return CreateRandomPassword(DEFAULT_PIN_SIZE);
  }


  public byte[] CreateRandomPassword(int size) throws Exception {
    if (size <= MINIMAL_SIZE) throw new Exception("Password must have at least " +
        MINIMAL_SIZE +
        " characters.");
        return CreateRandomBytes(size);
  }

  public  byte[] CreateRandomPin(int size) throws Exception{
    if (size <= MINIMAL_SIZE) throw new Exception("Pin must have at least " +
        MINIMAL_SIZE +
        " characters.");
    return CreateRandomBytes(size);
  }

  public byte[] CreateRandomBytes(int size) {

     byte [] bytes = new byte [size];
     randomGenerator.nextBytes(bytes);
     return bytes;
   }


  public int GetRandomNumber(int min, int max){
    int interval = max-min;
    if (interval<=0) return 0;

    byte[] aux = this.CreateRandomBytes(8);
    BigInteger random = new BigInteger(aux);
    random = random.abs();
    BigInteger intervalBI = new BigInteger(Integer.toString(interval + 1));

    intervalBI = random.mod(intervalBI);
    int value = (min + intervalBI.intValue());
    return value;
  }


  private byte[] GenerateFinalPassword(byte[] password, byte[] pin) {
    if (password==null || pin==null) return null;
    byte[] finalPassword = null;
    try{
      md.reset();
      md.update(password);
      md.update(pin);
      finalPassword = md.digest();
    } catch (Exception e) {
      System.err.println(e.getMessage());
    }
    return finalPassword;
  }


  public byte[][] GenerateFinalPasswordArray(int passwordNumber, char[] password) throws Exception{

    char[][] password_pin = CreatePasswordPinFromCharArray(password, MINIMAL_SIZE);
    return GenerateFinalPasswordArray(passwordNumber, new String(password_pin[0]).getBytes(),
                                      new String(password_pin[1]).getBytes());
   }


  public byte[][] GenerateFinalPasswordArray(int passwordNumber, String password,
                                      String pin) throws Exception{
      return GenerateFinalPasswordArray(passwordNumber, password.getBytes(),
                                      pin.getBytes());
  }


  public byte[][] GenerateFinalPasswordArray(int passwordNumber, char[] password,
                                    char[] pin) throws Exception{
    return GenerateFinalPasswordArray(passwordNumber, new String(password).getBytes(),
                                      new String(pin).getBytes());
  }


  public byte[][] GenerateFinalPasswordArray(int number_of_passwords, byte[] password,
                                      byte[] pin) throws Exception {
    if ((number_of_passwords <= 0)||(password.length < MINIMAL_SIZE)
        ||(pin.length < MINIMAL_SIZE))
      throw new Exception("Password and Pin must have at least " +
                          MINIMAL_SIZE +
                          " characters.");
    byte[][] passwordArray = new byte[number_of_passwords][];
    byte[] step;
    byte[] mi;
    for(int i = 0; i < number_of_passwords; i++)
    {
      mi = GenerateFinalPassword(password,pin);
      step = GenerateFinalPassword(pin,mi);
      passwordArray[i] = step;
      password = mi;
    }
    return passwordArray;
  }


  public char[][] CreatePasswordPinFromCharArray (char[] password_phrase, int password_size)
      throws Exception {
    if ((password_size > password_phrase.length) || (password_size < MINIMAL_SIZE)
        || (password_phrase.length - password_size < MINIMAL_SIZE))
      throw new Exception("Password and Pin must have at least " +
                              MINIMAL_SIZE +
                              " characters.");

    char[][] res = new char[2][];
    char[] password = new char[password_size];
    char[] pin = new char[password_phrase.length - password_size];
    for(int i = 0; i < password_size; i++){
      password[i] = password_phrase[i];
    }
    for(int i = 0; i < password_phrase.length - password_size; i++){
      pin[i] = password_phrase[password_size + i];
    }
    res[0] = password;
    res[1] = pin;
    return res;
  }



  public String ByteToString(byte[] b){
    return Base64.encode(b);
  }

  public byte[] StringToByte(String s) throws Exception{
    return Base64.decode(s);
  }

}
