package inescID.crypto.blindSignatures;

import java.security.Signature;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.MessageDigest;
import java.security.SecureRandom;
import java.math.BigInteger;
import java.security.NoSuchAlgorithmException;
import java.security.SignatureException;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */


public class RSABlindSignature extends inescID.crypto.SigningTool{


  public static final String DEFAULT_HASH_ALGORITHM = "SHA-1";
  public static final String DEFAULT_RANDOM_ALGORITHM = "SHA1PRNG";
  private final static int KEY_SIZE_BITS = 1024;
  private final static int KEY_SIZE_BYTES = KEY_SIZE_BITS / 8;


  BigInteger modulus;
  BigInteger privateExponent;
  BigInteger publicExponent;
  EMSA_PKCS1_V1_5 pkcs1;
  MessageDigest md;
  int emLen;
  SecureRandom srandom;


  public RSABlindSignature() throws NoSuchAlgorithmException {
     super(DEFAULT_SIGN_ALGORITHM);
   }


  public RSABlindSignature(String algorithm) throws NoSuchAlgorithmException {
    super(algorithm);
  }

  public void initBlindSign(RSAPrivateKey key) {
    this.modulus = key.getModulus();
    this.privateExponent = key.getPrivateExponent();
  }

  private BigInteger blindSign(BigInteger msg) {
    BigInteger signature = msg.modPow(privateExponent, modulus);
    return signature;
  }

  public byte[] blindSign(byte[] msg) {
    BigInteger signature = this.blindSign(new BigInteger(msg));
    byte[] msgbytes = signature.toByteArray();
   /*
    if (msgbytes.length != KEY_SIZE_BYTES) {
      byte[] clearmsg = new byte[KEY_SIZE_BYTES];
      int n = (msgbytes.length < KEY_SIZE_BYTES) ? msgbytes.length : KEY_SIZE_BYTES;
      for (int i = 1; i <= n; i++)
        clearmsg[clearmsg.length - i] = msgbytes[msgbytes.length - i];
      msgbytes = clearmsg;
    }*/
    return msgbytes;
  }

  public void initBlindEngine(RSAPublicKey key) throws java.
      security.NoSuchAlgorithmException {
    this.initBlindEngine(key, DEFAULT_HASH_ALGORITHM);
  }


  public void initBlindEngine(RSAPublicKey key, String hash) throws java.
      security.NoSuchAlgorithmException {
    this.modulus = key.getModulus();
    this.publicExponent = key.getPublicExponent();
    this.emLen = (key.getModulus().bitLength() + 7) / 8;
    this.pkcs1 = EMSA_PKCS1_V1_5.getInstance(hash);
    this.md = MessageDigest.getInstance(hash);
    this.srandom = SecureRandom.getInstance(DEFAULT_RANDOM_ALGORITHM);

  }

  private byte[] getMessagePadPKCS1(byte[] msg) {
    md.update(msg);
    byte[] digest = md.digest();
    byte[] pad = pkcs1.encode(digest, emLen);
    return pad;
  }

  private BigInteger getMessagePadPKCS1(BigInteger msg) {
    return new BigInteger(this.getMessagePadPKCS1(msg.toByteArray()));
  }

  private BigInteger blindMessage(BigInteger msg, BigInteger bf) {
    BigInteger blindmsg = bf;
    blindmsg = blindmsg.modPow(publicExponent, modulus);
    blindmsg = blindmsg.multiply(this.getMessagePadPKCS1(msg));
    blindmsg = blindmsg.mod(modulus);
    return blindmsg;
  }

  public byte[] blindMessage(byte[] msg, byte[] bf) {
    BigInteger blindmsg = this.blindMessage(new BigInteger(msg),
                                             new BigInteger(bf));
    return blindmsg.toByteArray();
  }

  private BigInteger unblindSignature(BigInteger blindmsg, BigInteger bf) {
    BigInteger msg = new BigInteger(blindmsg.toByteArray());
    BigInteger bfI = bf.modInverse(modulus);
    msg = msg.multiply(bfI);
    msg = msg.mod(modulus);
    return msg;
    /*
    byte[] msgbytes = msg.toByteArray();
    if (msgbytes.length != KEY_SIZE_BYTES) {
      byte[] clearmsg = new byte[KEY_SIZE_BYTES];
      int n = (msgbytes.length < KEY_SIZE_BYTES) ? msgbytes.length : KEY_SIZE_BYTES;
      for (int i = 1; i <= n; i++)
        clearmsg[clearmsg.length - i] = msgbytes[msgbytes.length - i];
      msgbytes = clearmsg;
    }
    return msgbytes;
 */
  }

  public byte[] unblindSignature(byte[] blindmsg, byte[] bf) {
    byte[] msgbytes = unblindSignature(new BigInteger(blindmsg),
                                       new BigInteger(bf)).toByteArray();

    if (msgbytes.length != KEY_SIZE_BYTES) {
      byte[] clearmsg = new byte[KEY_SIZE_BYTES];
      int n = (msgbytes.length < KEY_SIZE_BYTES) ? msgbytes.length : KEY_SIZE_BYTES;
      for (int i = 1; i <= n; i++)
        clearmsg[clearmsg.length - i] = msgbytes[msgbytes.length - i];
      msgbytes = clearmsg;
    }

    return msgbytes;
  }


  private BigInteger generateBlindFactor() {
    BigInteger bf;
    BigInteger gcd;
    BigInteger one = new BigInteger("1");
    do {
      bf = new BigInteger(this.modulus.bitLength() - 1, this.srandom);
      bf = bf.abs();
      gcd = bf.gcd(this.modulus);
    }
    while (gcd.compareTo(one) != 0);
    return bf;
  }

  public byte[] generateBlindFactorBytes() {
    return this.generateBlindFactor().toByteArray();
  }

  public boolean verifyBlindSignature(byte[] msg, byte[] signature) throws
      SignatureException {
    BigInteger aux = new BigInteger(msg);
    return this.verify(aux.toByteArray(), signature);
  }

}
