package inescID.revs.commissioner;

import java.io.*;
import java.security.*;
import java.security.interfaces.*;
import java.net.PasswordAuthentication;
import java.sql.*;
import java.util.*;
import javax.crypto.*;

import java.awt.event.*;

import inescID.crypto.*;
import inescID.revs.*;
import inescID.revs.database.*;
import inescID.revs.util.*;
import inescID.util.*;
import inescID.revs.REVSConfiguration;
import inescID.revs.servers.ReadConfigurationFile;
import java.security.spec.*;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

public class Commissioner {


  FrameCommissioner frame_commissioner;
  DB_Commissioner dbcommi;
  PasswordGenerator password_generator;
  char[] keystore_pass;
  char[] key_pass;
  boolean export_elections = false;


  public Commissioner() {
    start();
  }

  public void setFrameCommissioner(FrameCommissioner fcommi){
    this.frame_commissioner = fcommi;
  }


  public void start(){
    try {
      InputTool input = new InputTool();
      PasswordAuthentication database_authentication = input.readUserPassword(
          "Database", true);
      if (database_authentication == null) {
        database_authentication = new PasswordAuthentication(
            REVSConfiguration.DB_DEFAULT_USER,
            REVSConfiguration.DB_DEFAULT_PASSWORD.toCharArray());
      }

      this.keystore_pass = new char[0];
      while (keystore_pass.length < 6)
        keystore_pass = input.readPassword("KeyStore");

      this.key_pass = new char[0];
      while (key_pass.length < 6)
        key_pass = input.readPassword("Private key");

      this.start(database_authentication);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public void start(PasswordAuthentication auth){
    try {
      ReadConfigurationFile configuration = new ReadConfigurationFile();
      this.dbcommi = new DB_Commissioner(configuration.getDatabaseAddress(),
                                   auth.getUserName(),
                                   new String(auth.getPassword()));
      this.password_generator = new PasswordGenerator();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
    catch (Throwable ex1) {
      ex1.printStackTrace();
    }
  }

  public static void main(String[] args) {

    Commissioner commi = new Commissioner();

    try {
      if((args.length > 0)&&(args[0].equals("-G"))){
        commi.start();
        commi.frame_commissioner.start(commi);
      }else {



    InputTool input = new InputTool();



      char option;
      String line;
      option = '0';

      while (option != 'E'){

        System.out.println("\nC - Create tables");
        System.out.println("D - Delete tables");
        System.out.println("R - Redo tables");
        System.out.println("F - Fill tables");
        System.out.println("G - Graphic mode");

        System.out.println("E - Exit");

        line = input.readLine("Option: ");
        option = '0';
        if (line.length()==1)
          option = line.toCharArray()[0];

        switch (option){
          case 'C':
            //create tables
            commi.dbcommi.createTables();
            commi.initialData();

            break;
          case 'D':
            //delete tables
            commi.dbcommi.removeTables();
            break;
          case 'R':
            //redo tables
            commi.dbcommi.redoTables();
            commi.initialData();

            break;
          case 'F':
            //fill tables
            commi.FillDB(commi.dbcommi,false);
            break;
          case 'G':
            //graphic mode
            commi.frame_commissioner.start(commi);
            break;


          case 'E':
            System.exit(1);
            break;
        }

      }

      }
    }catch (Exception ex) {
      ex.printStackTrace();
    }
    System.exit(0);
  }








 void initialData() throws Exception {

  try {
    //insert initial groups
    this.dbcommi.alterTableVotersGroup(false, 0,
                                       DB_Commissioner.EMPTY_VOTERS_GROUP);
    this.dbcommi.alterTableVotersGroup(false, 0,
                                       DB_Commissioner.ALL_VOTERS_GROUP);


    /**************************************************/


    Server server = new Server();
    server.SetURL("//server/RMIservice");
    server.SetEnable(false);
    for (int i=1; i <= REVSConfiguration.MAX_SERVERS; i++) {
       server.SetId(i);
       server.SetDescription("Distributor "  + i);
       this.dbcommi.alterTableDistributor(false,server);
      // this.dbcommi.updateDistributorKey(i, null);

       server.SetDescription("Administrator "  + i);
       this.dbcommi.alterTableAdministrator(false, server);
       //this.dbcommi.updateAdministratorKey(i, null);

       server.SetDescription("Anonymizer "  + i);
       this.dbcommi.alterTableAnonymizer(false, server);
       //this.dbcommi.updateAnonymizerKey(i, null);

       server.SetDescription("Counter "  + i);
       this.dbcommi.alterTableCounter(false,server);
       //this.dbcommi.updateCounterKey(i, null);
     }


    }

  catch (SQLException ex) {
    ex.printStackTrace();
  }

 }


  /*
   *  Main Menu (MM) actions
   *
   */

  void MMvoters_actionPerformed(ActionEvent e) {
    try {
      this.frame_commissioner.setVotersAdministration(this.dbcommi.getVoters());
    }
    catch (SQLException ex) {
      this.frame_commissioner.newErrorMessage(ex.getMessage());
    }
  }

  void MMelections_actionPerformed(ActionEvent e) {
    try {
      this.frame_commissioner.setElectionAdministration(this.dbcommi.
          getElections(),
          this.dbcommi.getVotersGroups(),
          this.dbcommi.getConfigurationsIdDescription());
    }
    catch (Exception ex) {
      this.frame_commissioner.newErrorMessage(ex.getMessage());
    }

  }

  void MMservers_actionPerformed(ActionEvent e) {
    try {
      this.frame_commissioner.setServerAdministration(this.dbcommi.getDistributors(false),
                                                      this.dbcommi.getAdministrators(false),
                                                      this.dbcommi.getAnonymizers(false),
                                                      this.dbcommi.getCounters(false));

    }
    catch (Exception ex) {
      this.frame_commissioner.newErrorMessage(ex.getMessage());
    }


  }

  void MMconfiguration_actionPerformed(ActionEvent e) {
    try {
          this.frame_commissioner.setConfigurationAdministration(this.dbcommi.getConfigurations());
        }
        catch (Exception ex) {
          this.frame_commissioner.newErrorMessage(ex.getMessage());
        }

  }

  void MMcreate_distribution_files_actionPerformed(ActionEvent e) {
    try {
          this.frame_commissioner.setConfigurationSelection(this.dbcommi.getConfigurations());
        }
        catch (Exception ex) {
          this.frame_commissioner.newErrorMessage(ex.getMessage());
        }

  }



  void MMutilities_actionPerformed(ActionEvent e) {
    try {
      this.frame_commissioner.setUtilities(this.dbcommi.getAdministrators(true));
    }catch (Exception ex) {
      this.frame_commissioner.newErrorMessage(ex.getMessage());
    }

  }





  /*
   *Voter Password Menu (VP) actions
   */


  void VPcreate_passwords_actionPerformed(ActionEvent e) {
    try {
      char[][] password_pin = createPasswords();
      this.frame_commissioner.setPasswordPin(password_pin);
    }
    catch (Exception ex) {
      this.frame_commissioner.newErrorMessage(ex.getMessage());
    }
  }

  private char[][] createPasswords() throws Exception {
    char[][] password_pin = new char[2][];
    password_pin[0] = this.password_generator.CreateRandomStringPassword().
          toCharArray();
    password_pin[1] = this.password_generator.CreateRandomStringPin().
          toCharArray();
    return password_pin;
  }

  void VPok_actionPerformed(ActionEvent e) {
    char[][] password_pin = this.frame_commissioner.getConfirmedPasswordPin();



    if (password_pin == null){
      this.frame_commissioner.clearPasswordPin();
      this.frame_commissioner.newErrorMessage("Passwords don't match.");
    }
    else{

      IdDescription voter = this.frame_commissioner.getActiveVoter();
      try {
        byte[][] final_passwords = ((password_pin[1] != null) ?
                                    this.password_generator.GenerateFinalPasswordArray(REVSConfiguration.MAX_SERVERS, password_pin[0], password_pin[1]) :
                                    this.password_generator.GenerateFinalPasswordArray(REVSConfiguration.MAX_SERVERS, password_pin[0]));

        for(int i=0; i< final_passwords.length; i++)
          final_passwords[i] = this.password_generator.getHashValue(final_passwords[i]);

        this.dbcommi.deletePasswordsByVoter(voter.GetId());
        this.dbcommi.alterTablePasswords(false, voter.GetId(), final_passwords);
        this.frame_commissioner.disposeVoterPassword();
        this.frame_commissioner.newMessage("Voter's " + voter.GetId() + " passwords updated.");
      }
      catch (Exception ex) {
        this.frame_commissioner.newErrorMessage("Error updating voter's "+ voter.GetId() +
                                                " passwords. " + ex.getMessage());
      }
    }

  }


    /*
     * Voter Menu (VM) actions
     */

  void VMok_actionPerformed(ActionEvent e) {
    byte state = this.frame_commissioner.getVotersAdministrationState();
    IdDescription voter = this.frame_commissioner.getActiveVoter();
    switch (state){
      case VoterAdministration.NEW:
        try {
              this.dbcommi.alterTableVoter(false, voter.GetId(), voter.GetDescription());
              this.frame_commissioner.setVotersList(this.dbcommi.getVoters());
              this.frame_commissioner.newMessage("Voter " + voter.GetId() + " created.");
              this.frame_commissioner.setVoterPassword();
            }
            catch (SQLException ex) {
              this.frame_commissioner.newErrorMessage("Error inserting new voter. Verify number of the voter.");
            }
        break;
      case VoterAdministration.UPDATE:
        try {
          this.dbcommi.alterTableVoter(true, voter.GetId(), voter.GetDescription());
          this.frame_commissioner.setVotersList(this.dbcommi.getVoters());
          this.frame_commissioner.newMessage("Voter " + voter.GetId() + " renamed.");
        }
        catch (SQLException ex) {
          this.frame_commissioner.newErrorMessage("Error renaming voter " + voter.GetId()+".");
        }
        break;
    }
  }

  void VMchange_password_actionPerformed(ActionEvent e) {
    this.frame_commissioner.setVoterPassword();
  }


  void VMdelete_actionPerformed(ActionEvent e) {

    IdDescription voter = this.frame_commissioner.getActiveVoter();
    try {
      this.dbcommi.deleteVoter(voter.GetId());
      this.frame_commissioner.setVotersList(this.dbcommi.getVoters());
      this.frame_commissioner.newMessage("Deleted voter " + voter.GetId()+".");

    }
    catch (SQLException ex) {
      this.frame_commissioner.newErrorMessage("Error deleting voter " + voter.GetId() + ex.getMessage());
    }

  }

  void VMgroups_actionPerformed(ActionEvent e) {
    try {
      this.frame_commissioner.setGroupAdministration(this.dbcommi.getVotersGroups());
    }
    catch (SQLException ex) {
      this.frame_commissioner.newErrorMessage(ex.getMessage());
    }
   }


  void VMback_actionPerformed(ActionEvent e) {
    this.frame_commissioner.setMainMenu();
  }


  /*
   * Group Menu (GM) actions
   */

  void GMok_actionPerformed(ActionEvent e) {
    IdDescription group = this.frame_commissioner.getActiveGroup();
    byte state = this.frame_commissioner.getGroupAdministrationState();

    switch (state) {
      case GroupAdministration.NEW:
        try {
          this.dbcommi.alterTableVotersGroup(false, group.GetId(),
                                             group.GetDescription());
          this.frame_commissioner.setGroupList(this.dbcommi.getVotersGroups());
          this.frame_commissioner.newMessage("Group " + group.GetDescription() +
                                             " created.");
        }
        catch (SQLException ex) {
          this.frame_commissioner.newErrorMessage(
              "Error creating group. Verify group name.");
        }

        break;

      case GroupAdministration.UPDATE:
        try {
          this.dbcommi.alterTableVotersGroup(true, group.GetId(),
                                             group.GetDescription());
          this.frame_commissioner.setGroupList(this.dbcommi.getVotersGroups());
          this.frame_commissioner.newMessage("Group renamed to " +
                                             group.GetDescription() + ".");
        }
        catch (SQLException ex) {
          this.frame_commissioner.newErrorMessage(
              "Error renaming group. Verify group name.");
        }
        break;
    }
  }

  void GMmanage_actionPerformed(ActionEvent e) {

    try {
      IdDescription group = this.frame_commissioner.getActiveGroup();
      Vector voters = this.dbcommi.getVoters();
      Voters2Group group_voters = this.dbcommi.getVoters2Group(group.GetId());

      Vector aux = new Vector(voters.size());
      for (int i=0; i<voters.size(); i++)
        aux.add(new Integer(((IdDescription) voters.get(i)).GetId()));

      aux.removeAll(group_voters.GetVoters());
      this.frame_commissioner.setSelectVoters(group.GetDescription(), group_voters, aux);

    }
    catch (SQLException ex) {
      this.frame_commissioner.newErrorMessage("Error: " + ex.getMessage());
    }


  }



  void GMdelete_actionPerformed(ActionEvent e) {
    IdDescription group = this.frame_commissioner.getActiveGroup();

    if ((group.GetId() == DB_Commissioner.ALL_VOTERS_GROUP_ID) ||
        (group.GetId() == DB_Commissioner.EMPTY_VOTERS_GROUP_ID))
    {
      this.frame_commissioner.newMessage(
                "Cannot delete special group " + group.GetDescription() + ".");
      return;
    }


    try {
      if (this.dbcommi.deleteVotersGroup(group.GetId()) == 1) {

        this.frame_commissioner.setGroupList(this.dbcommi.getVotersGroups());
        this.frame_commissioner.newMessage("Group " + group.GetDescription() +
                                           " deleted.");
      }
      else
        this.frame_commissioner.newErrorMessage(
            "Error deleting group " + group.GetDescription() +". Group in use.");
    }
    catch (Exception ex) {
      this.frame_commissioner.newErrorMessage("Error deleting group. " +
                                         ex.getMessage());
    }
  }

  void GMback_actionPerformed(ActionEvent e) {
      this.frame_commissioner.setVotersAdministration();
  }


  /*
   * Select Voters (SV) actions
   */


  void SVok_actionPerformed(ActionEvent e) {

    try {
      Voters2Group voters = this.frame_commissioner.getGroupVoters();
      this.dbcommi.deleteVoter2GroupByGroup(voters.GetIdGroup());
      this.dbcommi.addVoters2Group(voters);
      this.frame_commissioner.newMessage("Group updated.");
      this.frame_commissioner.setGroupAdministration();
    }
    catch (SQLException ex) {
      this.frame_commissioner.newErrorMessage("Error updating group. " +
                                     ex.getMessage());
    }

   }

   void SVcancel_actionPerformed(ActionEvent e) {
     this.frame_commissioner.setGroupAdministration();
   }

   /*
    * Election Administration (EA)
    */

   void EAok_actionPerformed(ActionEvent e) {
     byte state = this.frame_commissioner.getElectionAdministrationState();
     Election election = this.frame_commissioner.getActiveElection();

     switch(state){
       case ElectionAdministration.NEW:
        try {
          this.dbcommi.alterTableElection(false, election);
          this.frame_commissioner.setElectionList(this.dbcommi.getElections());
          this.frame_commissioner.newMessage("Election " + election.GetDescription() + " created.");
        }
        catch (Exception ex) {
          this.frame_commissioner.newErrorMessage("Error creating election. Verify election name.");
        }

         break;

       case ElectionAdministration.UPDATE:
        try {
          this.dbcommi.alterTableElection(true, election);
          this.frame_commissioner.setElectionList(this.dbcommi.getElections());
          this.frame_commissioner.newMessage("Election " + election.GetDescription() + " updated.");
        }
        catch (Exception ex) {
          this.frame_commissioner.newErrorMessage("Error updating election. Verify election name.");
        }

         break;
     }


   }

   void EAdelete_actionPerformed(ActionEvent e) {
     Election election = this.frame_commissioner.getActiveElection();

    try {
      this.dbcommi.deleteElection(election.GetId());
      this.frame_commissioner.setElectionList(this.dbcommi.getElections());
      this.frame_commissioner.newMessage("Election " + election.GetDescription() + " deleted.");
    }
    catch (Exception ex) {
      this.frame_commissioner.newErrorMessage("Error deleting election " + election.GetDescription() + " " +
                                              ex.getMessage());
    }

   }

   void EAback_actionPerformed(ActionEvent e) {
     this.frame_commissioner.setMainMenu();
   }

   /*
    * Configuration Administration CA
    */

   void CAback_actionPerformed(ActionEvent e) {
     this.frame_commissioner.setMainMenu();
   }

   void CAdelete_actionPerformed(ActionEvent e) {
     Configuration configuration = this.frame_commissioner.getActiveConfiguration();

        try {

          if (this.dbcommi.deleteConfiguration(configuration.GetId()) == 1) {

            this.frame_commissioner.setConfigurationList(this.dbcommi.getConfigurations());
            this.frame_commissioner.newMessage("Configuration " + configuration.GetDescription() + " deleted.");

          }
          else
            this.frame_commissioner.newErrorMessage(
        "Error deleting configuration " + configuration.GetDescription() +". Configuration in use.");


        }
        catch (Exception ex) {
          this.frame_commissioner.newErrorMessage("Error deleting configuration " + configuration.GetDescription() + " " +
                                                  ex.getMessage());
        }

   }

   void CAok_actionPerformed(ActionEvent e) {
     byte state = this.frame_commissioner.getConfigurationAdministrationState();
     Configuration configuration = this.frame_commissioner.getActiveConfiguration();

     switch (state) {
       case ConfigurationAdministration.NEW:
         try {
           this.dbcommi.alterTableConfiguration(false, configuration);
           this.frame_commissioner.setConfigurationList(this.dbcommi.
               getConfigurations());
           this.frame_commissioner.newMessage("Configuration " +
                                              configuration.GetDescription() +
                                              " created.");
         }
         catch (Exception ex) {
           this.frame_commissioner.newErrorMessage(
               "Error creating configuration. Verify configuration name.");
         }

         break;

       case ConfigurationAdministration.UPDATE:
         try {
           this.dbcommi.alterTableConfiguration(true, configuration);
           this.frame_commissioner.setConfigurationList(this.dbcommi.getConfigurations());
           this.frame_commissioner.newMessage("Configuration " +
                                              configuration.GetDescription() +
                                              " updated.");
         }
         catch (Exception ex) {
           this.frame_commissioner.newErrorMessage(
               "Error updating configuration. Verify configuration name.");
         }

         break;
     }

   }

   /*
    * Servers Administration SA
    */

   void SAback_actionPerformed(ActionEvent e) {
     this.frame_commissioner.setMainMenu();
   }

   void SAload_key_actionPerformed(ActionEvent e) {
     byte server_type = this.frame_commissioner.getServertype();
     Server server = this.frame_commissioner.getActiveServer();
     try {
       String file = this.frame_commissioner.fileDialog("Certificate file.",
           java.awt.FileDialog.LOAD);

       KeyStoreTool ks = new KeyStoreTool();
       Key key=ks.getPublicKeyFromCertFile(file);

       switch (server_type) {
         case ServersAdministration.DISTRIBUTOR:
           this.dbcommi.updateDistributorKey(server.GetId(), key);
           break;
         case ServersAdministration.ADMINISTRATOR:
           this.dbcommi.updateAdministratorKey(server.GetId(), key);
           break;
         case ServersAdministration.ANONYMIZER:
           this.dbcommi.updateAnonymizerKey(server.GetId(), key);
           break;
         case ServersAdministration.COUNTER:
           this.dbcommi.updateCounterKey(server.GetId(), key);
           break;
       }
     this.frame_commissioner.newMessage("Servers' key updated.");
     }
     catch (Exception ex) {
       this.frame_commissioner.newErrorMessage(
           "Error updating servers' key.");
     }



   }


   void SAok_actionPerformed(ActionEvent e) {
     byte server_type = this.frame_commissioner.getServertype();
     Server server = this.frame_commissioner.getActiveServer();


     switch (server_type) {

       case ServersAdministration.DISTRIBUTOR:
         try {
           this.dbcommi.alterTableDistributor(true, server);
           this.frame_commissioner.setDistributorsList(this.dbcommi.getDistributors(false));
           this.frame_commissioner.newMessage("Distributor " +
                                              server.GetDescription() +
                                              " updated.");
         }
         catch (Exception ex) {
           this.frame_commissioner.newErrorMessage(
               "Error updating distributor. Verify distributor name.");
         }
         break;

       case ServersAdministration.ADMINISTRATOR:
       try {
         this.dbcommi.alterTableAdministrator(true, server);
         this.frame_commissioner.setAdministratorsList(this.dbcommi.getAdministrators(false));
         this.frame_commissioner.newMessage("Administrator " +
                                            server.GetDescription() +
                                            " updated.");
       }
       catch (Exception ex) {
         this.frame_commissioner.newErrorMessage(
             "Error updating administrator. Verify administrator name.");
       }
       break;

     case ServersAdministration.ANONYMIZER:
       try {
         this.dbcommi.alterTableAnonymizer(true, server);
         this.frame_commissioner.setAnonymizersList(this.dbcommi.getAnonymizers(false));
         this.frame_commissioner.newMessage("Anonymizer " +
                                            server.GetDescription() +
                                            " updated.");
       }
       catch (Exception ex) {
         this.frame_commissioner.newErrorMessage(
             "Error updating anonymizer. Verify anonymizer name.");
       }
       break;

     case ServersAdministration.COUNTER:
       try {
         this.dbcommi.alterTableCounter(true, server);
         this.frame_commissioner.setCountersList(this.dbcommi.getCounters(false));
         this.frame_commissioner.newMessage("Counter " +
                                            server.GetDescription() +
                                            " updated.");
       }
       catch (Exception ex) {
         this.frame_commissioner.newErrorMessage(
             "Error updating counter. Verify counter name.");
       }
       break;

   }
  }


/*
 * Select Configuration SC
 */

  void SCback_actionPerformed(ActionEvent e) {
    this.export_elections = false;
    this.frame_commissioner.showKeys(true);
    this.frame_commissioner.setMainMenu();
  }

  void SCnext_actionPerformed(ActionEvent e) {
    try {
      Configuration configuration = this.frame_commissioner.
          getSelectedConfiguration();

      KeyStoreTool kst;

      kst = new KeyStoreTool(REVSConfiguration.PRIVATE_KEYSTORE, this.keystore_pass);
      RSAPrivateKey kpri_commissioner = (RSAPrivateKey) kst.getPrivateKey(
          REVSConfiguration.ALIAS_SERVER_KEY, key_pass);

      /* Export elections */
      if (this.export_elections){
        Vector elections;
        elections = this.dbcommi.getElectionsByConfiguration(configuration.
            GetId());

        this.writeObject2File("Export elections file", elections, kpri_commissioner);

        this.export_elections = false;
      }
      else {
        /* Create Election Data */

        Vector[] administrators_signing_keys = null;
        Vector elections;
        Vector submit_servers = this.verifyConfigurationData(configuration);
        Vector administrators = new Vector(configuration.GetReqAdministrators());

        Vector aux = this.dbcommi.getAdministrators(true);
        for (int i = 0; i < configuration.GetReqAdministrators(); i++)
          administrators.add(aux.get(i));

        if (this.frame_commissioner.generateAdministratorsSigningKeys()) {
          administrators_signing_keys = this.createAdministratorsSigningKeys(
              configuration);
        }

        if (this.frame_commissioner.generateElectionKeys()) {
          this.createElectionKeys(configuration, kpri_commissioner);
        }

        elections = this.dbcommi.getElectionsByConfiguration(configuration.
            GetId());

        this.createAdministratorsFiles(configuration, elections, administrators,
                                       administrators_signing_keys,
                                       kpri_commissioner);

        administrators_signing_keys = null;

        this.createDistributorFile(configuration,
                                   elections, administrators, submit_servers,
                                   kpri_commissioner);

        this.createAnonymizersAndCountersFiles(configuration, elections,
                                               kpri_commissioner);
      }
    }
    catch (Exception ex) {
      this.frame_commissioner.newErrorMessage("Error creating distribution files. " + ex.getMessage());
    }


  }

  /*
   * Utilities
   */

  void UTcreate_passwords_actionPerformed(ActionEvent e) {
    this.frame_commissioner.setCreateVotersPasswords();
  }

  void CVPok_actionPerformed(ActionEvent e){
    boolean all_voters = this.frame_commissioner.allVoters();
    boolean password_pin = this.frame_commissioner.passwordPin();
    this.frame_commissioner.disposeCreateVotersPasswords();

    try {
      Vector voters = all_voters ? this.dbcommi.getVoters() :
          this.dbcommi.getVotersWithoutPassword();
      if (voters.size() == 0)
        return;

      FileTool ft = new FileTool();
      String file = null;
      file = this.frame_commissioner.fileDialog("Passwords file",
                                                java.awt.FileDialog.LOAD);
      if (file == null)return;

      ft.initWrite(file);

      IdDescription voter;
      char[][] passwords;
      byte[][] final_passwords;
      for (int i = 0; i < voters.size(); i++) {
        voter = (IdDescription) voters.get(i);
        ft.write(new String(voter.GetId() + ";"));

        passwords = this.createPasswords();
        if (!password_pin) {
          ft.write(new String (passwords[0]));
          final_passwords = this.password_generator.
              GenerateFinalPasswordArray(
              REVSConfiguration.MAX_SERVERS, passwords[0]);
        }
        else {
          ft.write(new String (passwords[0]) +";");
          ft.write(new String (passwords[1]));
          final_passwords = this.password_generator.
              GenerateFinalPasswordArray(
              REVSConfiguration.MAX_SERVERS, passwords[0], passwords[1]);
        }
        ft.writeLine("");
        for (int p = 0; p < final_passwords.length; p++) {
          final_passwords[p] = this.password_generator.getHashValue(
              final_passwords[p]);
        }
        this.dbcommi.deletePasswordsByVoter(voter.GetId());
        this.dbcommi.alterTablePasswords(false, voter.GetId(), final_passwords);
      }
      ft.closeWriteFile();
    }
    catch (Exception ex) {
      this.frame_commissioner.newErrorMessage("Error creating voters passwords:" +
                                              ex.getMessage());
      return;
    }
    catch (Throwable ex4) {
      ex4.printStackTrace();
      return;
    }
    this.frame_commissioner.newMessage("Voters passwords created.");
  }

  void UTback_actionPerformed(ActionEvent e) {
    this.frame_commissioner.setMainMenu();
  }




  void CVPcancel_actionPerformed(ActionEvent e) {
    this.frame_commissioner.disposeCreateVotersPasswords();
  }



  void UTimport_voters_actionPerformed(ActionEvent e) {
    FileTool ft = new FileTool();
    String file = null;
    file = this.frame_commissioner.fileDialog("Voters file", java.awt.FileDialog.LOAD);
    if (file == null) return;

    try {
      ft.initReadLine(file);
      int line_number = 0;
      String line = ft.readLine(); //id;name[;password[;pin]]
      String data[] = null;
      byte[][] final_passwords;
      while (line != null) {
        line_number++;
        try {
          data = line.split(REVSConfiguration.MARKER);
          if ((data.length < 2)||(data.length > 4)){
            this.frame_commissioner.newMessage("File format error at line:" + line_number);
            line = ft.readLine();
            continue;
          }

          this.dbcommi.alterTableVoter(false, Integer.parseInt(data[0]), data[1]);
          try {
            final_passwords = null;
            switch (data.length) {
              case 3:
                final_passwords = this.password_generator.
                    GenerateFinalPasswordArray(
                    REVSConfiguration.MAX_SERVERS, data[2].toCharArray());
                break;
              case 4:
                final_passwords = this.password_generator.
                    GenerateFinalPasswordArray(
                    REVSConfiguration.MAX_SERVERS, data[2].toCharArray(),
                    data[3].toCharArray());
                break;
            }
            if (final_passwords != null) {
              for (int i = 0; i < final_passwords.length; i++) {
                final_passwords[i] = this.password_generator.getHashValue(
                    final_passwords[i]);
              }

              this.dbcommi.deletePasswordsByVoter(Integer.parseInt(data[0]));
              this.dbcommi.alterTablePasswords(false, Integer.parseInt(data[0]),
                                               final_passwords);
            }
          }
          catch (Exception ex3) {
            this.frame_commissioner.newMessage(
                "Error inserting voter:" + data[0] + " passwords. Line:"
                + line_number);
          }
        }
        catch (NumberFormatException ex1) {
          this.frame_commissioner.newMessage("Voter number:" + data[0] + " not valid at line:" + line_number);
        }
        catch (Exception ex1) {
          this.frame_commissioner.newMessage("Error inserting voter:" + data[0] + ". Line:" + line_number);
          try{
            this.dbcommi.deleteVoter(Integer.parseInt(data[0]));
          }
          catch(Exception ex2){
          }
        }
        line = ft.readLine();
      }
      ft.closeReadFile();
      this.frame_commissioner.newMessage("Voter's file processed.");
    }
    catch (Exception ex) {
      this.frame_commissioner.newErrorMessage("Error reading voters file.");
    }
    catch (Throwable ex4) {
      ex4.printStackTrace();
    }
  }

  void UTimport_groups_actionPerformed(ActionEvent e) {
    FileTool ft = new FileTool();
    String file = null;
    file = this.frame_commissioner.fileDialog("Voters groups file",
                                                java.awt.FileDialog.LOAD);
    if (file == null) return;
    try {
      ft.initReadLine(file);
      int line_number = 0;
      String line = ft.readLine(); //(id | description);id_voter
      String data[] = null;
      int group = 0;

      while (line != null) {
        line_number++;
        data = line.split(REVSConfiguration.MARKER);
        if (data.length != 2) {
          this.frame_commissioner.newMessage("File format error at line:" +
                                             line_number);
          line = ft.readLine();
          continue;
        }

        //verify group
        group = getVotersGroupId(data[0]);
        if (group == 0) {
          this.frame_commissioner.newMessage("Error processing group:" + data[0] + " at line:" +
                                             line_number);
          line = ft.readLine();
          continue;
        }
        //verify voter
        try {
          if (this.dbcommi.getVoter(Integer.parseInt(data[1])) == null) {
            this.frame_commissioner.newMessage(
                "Error. Voter:" + data[1] + " does not exist. Line:" + line_number);
            line = ft.readLine();
            continue;
          }


          //insert voter to group
          this.dbcommi.insertTableVoter2Group(Integer.parseInt(data[1]), group);
        }
        catch (Exception ex1) {
          this.frame_commissioner.newMessage(
              "Error processing voter:" + data[1] + " at Line:" + line_number);
          line = ft.readLine();
          continue;
        }
        line = ft.readLine();
      }
      ft.closeReadFile();
      this.frame_commissioner.newMessage("Voters groups file processed.");
    }
    catch (Exception ex) {
      this.frame_commissioner.newErrorMessage(
          "Error reading voters groups file.");
    }
    catch (Throwable ex4) {
      ex4.printStackTrace();
    }


  }

  private int getVotersGroupId(String group_string){
    int group = 0;
    try {
      try {
        group = Integer.parseInt(group_string);
        if (this.dbcommi.getVotersGroup(group) == null) {
          this.dbcommi.alterTableVotersGroup(false, group, "Group " + group);
        }
      }
      catch (NumberFormatException ex1) {
        group = this.dbcommi.getVotersGroupIdByDescription(group_string);
        if (group == 0) {
          group = this.dbcommi.getAvailableVotersGroupId();
          this.dbcommi.alterTableVotersGroup(false, group, group_string);
        }
      }
    }
    catch (SQLException ex) {
      return 0;
    }
    return group;
  }

  void UTimport_elections_actionPerformed(ActionEvent e){
    FileTool ft = new FileTool();
    String file = null;
    file = this.frame_commissioner.fileDialog("Elections file",
                                              java.awt.FileDialog.LOAD);
    if (file == null) return;
    try {
      ft.initReadLine(file);
      int line_number = 0;
      String line = ft.readLine(); //[id;]description;(id_voters_group | voters_group_description);configuration_description;ballot
      String data[] = null;
      while (line != null) {
        line_number++;
        data = line.split(REVSConfiguration.MARKER);
        if ((data.length < 4)||(data.length > 5)){
          this.frame_commissioner.newMessage("File format error at line:" + line_number);
          line = ft.readLine();
          continue;
        }

        int election=0;
        int start=0;
        String description;
        if (data.length==5){
          try {
            election = Integer.parseInt(data[0]);
          }
          catch (NumberFormatException ex1) {
            this.frame_commissioner.newMessage("Election id:" + data[0] + " not valid at line:" + line_number);
            line = ft.readLine();
            continue;
          }
          description = data[1];
          start=1;
        }
        else{
          election = this.dbcommi.getAvailableElectionId();
          description=data[0];
        }

      //verify group
        int group = this.getVotersGroupId(data[start + 1]);
        if (group == 0) {
          this.frame_commissioner.newMessage("Error processing group:" + data[start + 1] + " at line:" +
                                             line_number);
          line = ft.readLine();
          continue;
        }
        //verify configuration
        int configuration = this.dbcommi.getConfigurationIdByDescription(data[start + 2]);
        if (configuration == 0) {
          this.frame_commissioner.newMessage("Error processing configuration:" + data[start + 2] + " at line:" +
                                             line_number);
          line = ft.readLine();
          continue;
        }

        //insert election
        Election election_data = new Election();
        election_data.SetId(election);
        election_data.SetDescription(description);
        election_data.SetIdVotersGroup(group);
        election_data.SetConfiguration(configuration);
        election_data.SetBallot(data[data.length-1]);
        try {
          this.dbcommi.alterTableElection(false, election_data);
        }
        catch (SQLException ex2) {
          this.frame_commissioner.newMessage("Error processing election:" + data[0] + " at line:" +
                                   line_number);
        }

        line = ft.readLine();
      }
      ft.closeReadFile();
      this.frame_commissioner.newMessage("Elections file processed.");

    }
    catch (Exception ex) {
      this.frame_commissioner.newErrorMessage(
          "Error reading voters groups file.");
    }
    catch (Throwable ex4) {
      ex4.printStackTrace();
    }

  }

  void UTexport_elections_actionPerformed(ActionEvent e) {
    this.export_elections = true;
    try {
      this.frame_commissioner.showKeys(false);
      this.frame_commissioner.setConfigurationSelection(this.dbcommi.
          getConfigurations());
    }
    catch (Exception ex) {
      this.frame_commissioner.newErrorMessage(ex.getMessage());
    }
  }

  void UTimport_adm_sig_keys_actionPerformed(ActionEvent e) {
    int adm = this.frame_commissioner.getSelectedAdministrator();
    if (adm == -1) {
      this.frame_commissioner.newErrorMessage("No Administrator selected");
    }
    else {
      try {
        Key key = this.dbcommi.getAdministratorPublicKey(adm);
        Vector elections = null;
        try {
          elections = (Vector)this.readObjectFromFile(
              "Administrator signing keys", (RSAKey) key);
        }
        catch (Exception ex1) {
          this.frame_commissioner.newErrorMessage("Error reading Administrator's signing keys file.");
        }

        Election election;
        for(int i=0; i<elections.size(); i++){
          election = (Election) elections.get(i);
          this.dbcommi.deleteAdministratorElection(adm, election.GetId());
          this.dbcommi.insertTableAdministratorElection(adm, election.GetId(), election.GetKey());
        }
        this.frame_commissioner.newMessage("Administrator's signing keys: OK");

      }
      catch (Exception ex) {
        this.frame_commissioner.newErrorMessage(ex.getMessage());
      }
    }
  }




/*
 * Distribution files
 */


  Vector verifyConfigurationData(Configuration conf) throws Exception {

  Vector aux;
  Vector aux1;
  Vector elections = this.dbcommi.getElectionsByConfiguration(conf.GetId());
  Election election;
  AdministratorElection adm_election;
  Server server;

  Vector submit_servers;

  //////verify administrators
  //sufficient number
  aux = this.dbcommi.getAdministrators(true);
  if (aux.size() < conf.GetReqAdministrators())
    throw new Exception ("Not enough administrators enable.");
  //signing_keys
  try {
    if (! (this.frame_commissioner.generateAdministratorsSigningKeys())) {

      for (int i = 0; i < elections.size(); i++) {
        election = (Election) elections.get(i);
        aux1 = this.dbcommi.getAdministratorsSigningKeysByElection(election.
            GetId());

        for (int j = 0; j < conf.GetReqAdministrators(); j++) {
          server = (Server) aux.get(j);
          for (int k = 0; ; k++) {
            adm_election = (AdministratorElection) aux1.get(k);
            if (server.GetId() == adm_election.GetAdministrator()) {
              break;
            }
          }
        }

      }
    }
  }
  catch (Exception ex) {
    throw new Exception ("Missing administrators signing keys.");
  }

  //////verify distributors
  //sufficient number
  aux = this.dbcommi.getDistributors(true);
  if (aux.size() < 1)
    throw new Exception ("Not enough ballot distributors enable.");

  //////verify anonymizers
  //sufficient number
  if (conf.UseAnonymizers()){
    submit_servers = this.dbcommi.getAnonymizers(true);
    if (submit_servers.size() < 1)
      throw new Exception("Not enough anonymizers enable.");
  }
  else
    submit_servers = this.dbcommi.getCounters(true);

  //////verify counters
  //sufficient number
  aux = this.dbcommi.getCounters(true);
  if (aux.size() < 1)
    throw new Exception ("Not enough counters enable .");


  //////verify elections keys and voters
  Vector election_vector = new Vector();
  HashSet voters_id;

  if (! (this.frame_commissioner.generateElectionKeys())) {
    for (int i=0; i<elections.size(); i++){
      election = (Election) elections.get(i);
      //key
      if (election.GetKey()==null)
        throw new Exception ("Election " + election.GetDescription() + " has no key.");

      //voters
      election_vector.removeAllElements();
      election_vector.add(election);
      voters_id = getElectionsVotersID(election_vector);
      if (voters_id.isEmpty())
        this.frame_commissioner.newMessage("Election " + election.GetDescription() +
                                           " has no voters.");
    }
  }

  //////verify voters passwords
  IdDescription voter;
  voters_id = getElectionsVotersID(elections);

  Passwords passwords;
  int voter_id;
  Object[] voters_id_array = voters_id.toArray();

  for (int i = 0; i < voters_id_array.length; i++) {
    voter_id = ((Integer) voters_id_array[i]).intValue();
    passwords = (Passwords)this.dbcommi.getPasswords(voter_id);
    if (passwords.GetPasswords() == null)
      throw new Exception("Voter " + voter_id + " has no password.");
  }

  return submit_servers;
}

  HashSet getElectionsVotersID(Vector elections) throws Exception{
    Election election;
    IdDescription voter;
    HashSet voters_id = new HashSet();
    Vector aux;

    loop:  for (int i=0; i<elections.size(); i++){
      election = (Election) elections.get(i);
      switch (election.GetIdVotersGroup()){
        case DB_Commissioner.EMPTY_VOTERS_GROUP_ID:
 //         this.frame_commissioner.newMessage("Election " + election.GetDescription() +
 //                             " has no voters.");
          throw new Exception("Election " + election.GetDescription() +
                              " has no voters.");

        case DB_Commissioner.ALL_VOTERS_GROUP_ID:
          aux = this.dbcommi.getVoters();
          for (int j=0; j<aux.size(); j++){
            voter = (IdDescription) aux.get(j);
            voters_id.add(new Integer(voter.GetId()));
          }
          break loop;

        default:
          aux = ((Voters2Group)this.dbcommi.getVoters2Group(election.GetIdVotersGroup())).GetVoters();
          for(int j=0; j< aux.size();j++){
            voters_id.add(aux.get(j));
          }
      }
    }
    return voters_id;
  }


  void createDistributorFile(Configuration configuration,
                                            Vector elections,
                                            Vector administrators,
                                            Vector submit_servers,
                                            RSAPrivateKey key) throws Exception{

    Vector ballots = new Vector(elections.size());
    Vector groups = new Vector(elections.size());
    Vector clean_elections = new Vector(elections.size());
    Vector voters = new Vector();
    SigningTool sign = new SigningTool();


    Election election;
    for(int i=0; i<elections.size(); i++){
      election = (Election) elections.get(i);
      //ballots
      ballots.add(this.readAndSignBallot(election, configuration,
                                         administrators, submit_servers,
                                         sign, key));

      //groups
      groups.add(this.dbcommi.getVoters2Group(election.GetIdVotersGroup()));

      //elections
      clean_elections.add(new Election(election.GetId(),
                                       election.GetDescription(),
                                       election.GetIdVotersGroup(),
                                       election.GetIdConfiguration()));
    }

    //voters
    HashSet voters_id = this.getElectionsVotersID(elections);
    Object[] voters_array = voters_id.toArray();

    for(int i=0; i< voters_array.length; i++){
      voters.add(this.dbcommi.getVoter(((Integer) voters_array[i]).intValue()));
    }

    ConfigurationDates conf_dates = new ConfigurationDates(configuration.GetId(),
        configuration.GetDescription(), configuration.GetStart(), configuration.GetEnd());

    TransferDistributor td = new TransferDistributor(ballots, groups, clean_elections,
        voters, conf_dates);
    this.writeObject2File("Save distributors file", td, key);

    Vector distributors =  this.dbcommi.getDistributors(true);
    this.writeObject2File("Save list of distributors", distributors, key);

  }


  void createAdministratorsFiles(Configuration configuration, Vector elections,
                                 Vector administrators, Vector[] signing_keys,
                                 RSAPrivateKey kpri_commissioner) throws
      NoSuchAlgorithmException, SQLException, Exception {

    Vector clean_elections = new Vector(elections.size());
    Vector groups = new Vector(elections.size());
    Vector voters_passwords;
    ConfigurationDates conf_dates = new ConfigurationDates(configuration.GetId(),
        configuration.GetDescription(), configuration.GetStart(), configuration.GetEnd());
    MixCipherEnvelope keys = null;

    EncryptionTool encrypt = new EncryptionTool();
    Server administrator;

    Election election;
    for(int i=0; i<elections.size(); i++){
      //elections
      election = (Election) elections.get(i);
      clean_elections.add(new Election(election.GetId(),
                                       election.GetDescription(),
                                       election.GetIdVotersGroup(),
                                       election.GetIdConfiguration()));
      //groups
      groups.add(this.dbcommi.getVoters2Group(election.GetIdVotersGroup()));
    }

    //passwords
    HashSet voters_id = this.getElectionsVotersID(elections);
    Object[] voters_array = voters_id.toArray();
    Passwords passwords;


    RSAPublicKey kpub;
    TransferAdministrator ta;
    for (int i=0; i<administrators.size(); i++){
      administrator = (Server) administrators.get(i);
      kpub = (RSAPublicKey) this.dbcommi.getAdministratorPublicKey(administrator.GetId());
      if (signing_keys != null){
        keys = encrypt.EncryptMixCipherMode(signing_keys[i], kpub);
      }
      voters_passwords = new Vector();
      //passwords
      for(int j=0; j< voters_array.length; j++){
        passwords = this.dbcommi.getPasswords(((Integer) voters_array[j]).intValue());
        voters_passwords.add(new IdPassword(passwords.GetIdVoter(),
                                            passwords.GetPasswords()[administrator.GetId()-1]));
      }

      ta = new TransferAdministrator(clean_elections, groups, conf_dates,
                                     encrypt.EncryptMixCipherMode(voters_passwords, kpub),
                                     keys);
    /*
      ta = new TransferAdministrator(clean_elections, groups, conf_dates,
                                  null,
                                  keys);
*/


      this.writeObject2File("Save " + administrator.GetDescription() + " file", ta, kpri_commissioner);
    }
  }

  void createAnonymizersAndCountersFiles(Configuration configuration,
                                         Vector elections, RSAPrivateKey key) throws
      SQLException, Exception {
    Vector clean_elections = new Vector(elections.size());
    ConfigurationDates conf_dates = new ConfigurationDates(configuration.GetId(),
        configuration.GetDescription(), configuration.GetStart(), configuration.GetEnd());

    Election election;
    for(int i=0; i<elections.size(); i++){
      //elections
      election = (Election) elections.get(i);
      String ballot = this.readBallot(election);
      clean_elections.add(new Election(election.GetId(),
                                       election.GetDescription(),
                                       election.GetIdVotersGroup(),
                                       election.GetIdConfiguration(),
                                       (byte[]) null,
                                       ballot));
    }

    TransferAnonymizerCounter tac = new TransferAnonymizerCounter(clean_elections,
        conf_dates);
    this.writeObject2File("Save anonymizers/counters file", tac, key);

    Vector counters = this.dbcommi.getCounters(true);
    this.writeObject2File("Save counters info", counters, key);


/*
    Vector submitting_servers = counters;
    if (configuration.UseAnonymizers()){
      submitting_servers = this.dbcommi.getAnonymizers(true);
    }

    this.writeObject2File("Save submitting servers info", submitting_servers, key);
 */
  }


  void createElectionKeys(Configuration configuration, RSAPrivateKey kpri_commissioner) throws Exception {
    EncryptionTool encrypt = new EncryptionTool();
    Vector elections = this.dbcommi.getElectionsByConfiguration(configuration.GetId());
    Vector electionsPrivateKeys = new Vector(elections.size());
    Vector adm_elections = new Vector();
    Vector aux;
    KeyPair kp;
    Election election;

    this.frame_commissioner.newMessage("Generating elections' keys.");
    for (int i = 0; i < elections.size(); i++) {
      kp = encrypt.genRSAKeyPair();
      election = (Election) elections.get(i);
      this.dbcommi.updateElectionKey(election.GetId(), kp.getPublic());
      electionsPrivateKeys.add(new Election(election.GetId(),
                                            election.GetDescription(),
                                            election.GetIdVotersGroup(),
                                            election.GetIdConfiguration(),
                                            kp.getPrivate(), null));

      aux = this.dbcommi.getAdministratorsSigningKeysByElection(election.GetId());
      adm_elections.addAll(aux);
      this.frame_commissioner.newMessageSameline(".");
    }

    VerificationData verification = new VerificationData(adm_elections,
       electionsPrivateKeys, configuration);


   SecretKey key = encrypt.getTripleDESkey();
   byte[] data = encrypt.EncryptTripleDES(verification, key);
   this.writeObject2File("Save elections private keys", data, kpri_commissioner);
   this.writeObject2File("Save secret key for elections private keys", key, kpri_commissioner);
  }

  Vector[] createAdministratorsSigningKeys(Configuration configuration) throws
      Exception {
    EncryptionTool encrypt = new EncryptionTool();
    Vector elections = this.dbcommi.getElectionsIdDescriptionByConfiguration(
      configuration.GetId());
    Vector administrators = this.dbcommi.getAdministrators(true);
    Vector[] administrators_signing_keys = new Vector[configuration.
        GetReqAdministrators()];

    KeyPair kp;
    AdministratorElection sign_key;
    Server administrator;
    IdDescription election;
    this.frame_commissioner.newMessage(
      "Generating Administrators' signing keys.");

    for (int i = 0; i < elections.size(); i++) {
      election = (IdDescription) elections.get(i);
      this.dbcommi.deleteAdministratorElectionByElection(election.GetId());
    }

    for (int i = 0; i < configuration.GetReqAdministrators(); i++) {
      administrators_signing_keys[i] = new Vector(elections.size());
      administrator = (Server) administrators.get(i);
      this.frame_commissioner.newMessage(".");
      for (int j = 0; j < elections.size(); j++) {
        kp = encrypt.genRSAKeyPair();
        election = (IdDescription) elections.get(j);
        this.dbcommi.insertTableAdministratorElection(administrator.GetId(),
                                                      election.GetId(),
                                                      kp.getPublic());
        sign_key = new AdministratorElection(administrator.GetId(),
                                             election.GetId(),
                                             kp.getPrivate().getEncoded());
        administrators_signing_keys[i].add(sign_key);
        this.frame_commissioner.newMessageSameline(".");
      }
    }
    return administrators_signing_keys;
  }

  String readBallot(Election election) throws FileNotFoundException,
      IOException {
    FileTool file = new FileTool();
    String ballot = file.readTextFile(election.GetBallot());
    ballot = ballot.replaceFirst("<ballot electionCode=\"0\">",
                            "<ballot electionCode=\"" + election.GetId() + "\">");
    return ballot;
  }

  SignedObject readAndSignBallot(Election election, Configuration configuration,
                                 Vector administrators, Vector submit_servers,
                                 SigningTool sign, PrivateKey key) throws
      FileNotFoundException, IOException, Exception {

    String text_ballot = this.readBallot(election);

      //administrators
    AdministratorElectionAddress[] adm_election =
        new AdministratorElectionAddress[administrators.size()];

    Server server;
    AdministratorElection aux;
    for(int i=0; i<administrators.size(); i++){
      server = (Server) administrators.get(i);
      aux = this.dbcommi.getAdministratorElection(election.GetId(), server.GetId());
      adm_election[i] = new AdministratorElectionAddress(server, aux);
    }
    Ballot ballot = new Ballot(election.GetId(), text_ballot, election.GetPublicKey(),
                               configuration.GetReqSignatures(),
                               adm_election, submit_servers);
    return sign.signObject(key, ballot);

  }

  void writeObject2File (String msg, Object obj, RSAKey key) throws Exception{
   FileEncryptionTool file_tool = new FileEncryptionTool();
   String file = null;
   while (file == null){
     file = this.frame_commissioner.fileDialog(msg, java.awt.FileDialog.SAVE);
   }
   file_tool.writeObject2File(file, obj, key);
  }

  void writeObject2File (String msg, Object obj, SecretKey key) throws Exception{
   FileEncryptionTool file_tool = new FileEncryptionTool();
   String file = null;
   while (file == null){
     file = this.frame_commissioner.fileDialog(msg, java.awt.FileDialog.SAVE);
   }
   file_tool.writeObject2File(file, obj, key);
  }

  void writeObject2File (String msg, Object obj) throws Exception{
   FileTool file_tool = new FileTool();
   String file = null;
   while (file == null){
     file = this.frame_commissioner.fileDialog(msg, java.awt.FileDialog.SAVE);
   }
   file_tool.writeObject2File(file, obj);
  }



  Object readObjectFromFile (String msg, RSAKey key) throws Exception{
    FileEncryptionTool file_tool = new FileEncryptionTool();
    String file = null;
    while (file == null){
      file = this.frame_commissioner.fileDialog(msg, java.awt.FileDialog.LOAD);
    }
    return file_tool.readObjectFromFile(file, key);
  }

  Object readObjectFromFile (String msg, SecretKey key) throws Exception{
    FileEncryptionTool file_tool = new FileEncryptionTool();
    String file = null;
    while (file == null){
      file = this.frame_commissioner.fileDialog(msg, java.awt.FileDialog.LOAD);
    }
    return file_tool.readObjectFromFile(file, key);
  }








  public void FillDB(DB_Commissioner db, boolean update){
    try {
      String line;
      InputTool input = new InputTool();

      //Voters, Passwords
      byte[][] passwords;
      passwords = new byte[inescID.revs.REVSConfiguration.MAX_SERVERS][];
      String pass;
      int voter;

      line = input.readLine("Enter number of voters:");

      int max_voters = Integer.parseInt(line);
      for (int i=1; i<=max_voters; i++){
        db.alterTableVoter(update, i, "voter" + i + (update?"update":""));
        for (int j=0; j<REVSConfiguration.MAX_SERVERS; j++){
          pass = "Voter" + i + "Pass" + j+ (update?"update":"");
          passwords[j] = pass.getBytes();
        }
        db.alterTablePasswords(update, i, passwords);
      }


      //Groups

      line = input.readLine("Enter number of voters groups:");
      int max_groups=Integer.parseInt(line);
      line = input.readLine("Enter number of voters per group:");
      int max_voters2group=Integer.parseInt(line);
      int k=1;
      for (int i=3; i<=max_groups; i++){
        db.alterTableVotersGroup(update, i, "Group" + i + (update?"update":""));
        if(!update){
          //Voters2Group
          for (int j=0; j<max_voters2group; j++, k++){
            db.insertTableVoter2Group(k, i);
          }
        }
      }

      //Configuration

      line = input.readLine("Enter number of configurations:");
      int max_configurations=Integer.parseInt(line);
      Configuration config = new Configuration();
      for (int i=1; i <=max_configurations; i++) {
        config.SetId(i);
        config.SetDescription("condiguration" + i + (update?"update":""));
        config.SetStart(System.currentTimeMillis());
        config.SetEnd(System.currentTimeMillis());
        config.SetUseAnonymizers(update);
        config.SetReqAdministrators(2);
        config.SetReqSignatures(1);
        db.alterTableConfiguration(update,config);
      }

      //Election

      line = input.readLine("Enter number of elections:");
      int max_elections=Integer.parseInt(line);
      Election election = new Election();
      for (int i=1; i <= max_elections; i++) {
        election.SetId(i);
        election.SetDescription("election" + i + (update?"update":""));
        election.SetIdVotersGroup(i);
        election.SetConfiguration(i);
        //election.SetKey(null);
        election.SetBallot("ballot" + i);
        db.alterTableElection(update, election);
      }
/*
//Distributors, Administrators, Anonymizers and Counters
    Server server = new Server();
    server.SetURL("//server/RMIservice");
    server.SetEnable(false);
    for (int i=1; i <= REVSConfiguration.MAX_SERVERS; i++) {
       server.SetId(i);
       server.SetDescription("Distributor "  + i);
       this.dbcommi.alterTableDistributor(update,server);
       server.SetDescription("Administrator "  + i);
       this.dbcommi.alterTableAdministrator(update, server);
       server.SetDescription("Anonymizer "  + i);
       this.dbcommi.alterTableAnonymizer(update, server);
       server.SetDescription("Counter "  + i);
       this.dbcommi.alterTableCounter(update,server);
     }
*/

    }
    catch (Exception ex) {
      ex.printStackTrace();
    }


  }

}
