package inescID.revs.database;


import java.util.*;
import java.sql.*;
import java.security.NoSuchAlgorithmException;

/**
 * <p>Title: revs</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */


import inescID.crypto.PasswordGenerator;
import inescID.revs.database.*;
import inescID.revs.util.Election;
import java.security.spec.InvalidKeySpecException;

public class DB_Administrator
    extends DB_EC_Voters{



  PreparedStatement preInsertVoterElection; //Id_Voter, Id_Election, Signature

  PreparedStatement preGetVoterPassword;
  PreparedStatement preGetVoterElectionSignature;

  PreparedStatement preDeleteVoterElection;

  PasswordGenerator password_generator;

  public DB_Administrator() throws SQLException{
    super();
  }

  public DB_Administrator(String db, String user, String password)
      throws java.lang.ClassNotFoundException, SQLException, NoSuchAlgorithmException {
    super();
    this.DBInit(db, user, password);
    this.init();
    this.password_generator = new PasswordGenerator();
  }

  public void createTables(String error) {
    try {
      this.tableVoterElection();
    }
    catch (SQLException ex) {
      error += "\n VoterElection: " + ex.getMessage();
    }
    super.createTables(error);
  }

  public void removeTables(String error){
    String tables = "DROP TABLE IF EXISTS voter_election";
    try {
      Statement stmt = this.con.createStatement();
      stmt.executeUpdate(tables);
    }
    catch (SQLException ex) {
      error += ex.getMessage();
    }

    super.removeTables(error);
  }


  protected void tableVoterElection () throws SQLException {
      String table = "CREATE TABLE voter_election (" +
                      "Id_Voter int(10) unsigned NOT NULL," +
                      "Id_Election int(10) unsigned NOT NULL," +
                      "Signature blob NOT NULL," +
                      "PRIMARY KEY  (Id_Voter,Id_Election))";
      Statement stmt = this.con.createStatement();
      stmt.executeUpdate(table);
    }


  protected void prepareInsert() throws SQLException {
    super.prepareInsert();
    this.preInsertVoterElection = this.con.prepareStatement(
      "INSERT INTO voter_election VALUES (?, ?, ?)");
  }


 public void alterTableVoter(boolean update, int voter,
                                 byte[] password)
        throws SQLException {
      PreparedStatement statement = (update?this.preUpdateVoter:this.preInsertVoter);
      statement.setString(1, password_generator.ByteToString(password));
      statement.setInt(2, voter);
      statement.executeUpdate();
    }


  public void insertTableVoterElection(int voter, int election, byte[] signature) throws SQLException {
    PreparedStatement statement = this.preInsertVoterElection;
    statement.setInt(1, voter);
    statement.setInt(2, election);
    statement.setBytes(3, signature);
    statement.executeUpdate();
  }

  protected void prepareGet() throws SQLException {
    super.prepareGet();
    this.preGetVoterPassword = this.con.prepareStatement(
      "SELECT Name FROM voter WHERE Id_Voter=?");
    this.preGetVoterElectionSignature = this.con.prepareStatement(
      "SELECT Signature FROM voter_election WHERE Id_Voter=? AND Id_Election=?");
  }

  public Election getElection(int id) throws
        InvalidKeySpecException, SQLException {
     this.preGetElection.setInt(1, id);
     Election election = null;
     ResultSet rs = this.preGetElection.executeQuery();
     if (rs.next()) {
       election = new Election(rs.getInt(1), rs.getString(2), rs.getInt(3), rs.getInt(4),
                               rs.getBytes(5), null);
     }
     return election;
   }


  public byte[] getVoterPassword(int voter) throws SQLException, Exception {
     byte[] password=null;
     this.preGetVoterPassword.setInt(1, voter);
     ResultSet rs = this.preGetVoterPassword.executeQuery();
     if (rs.next()){
       password = password_generator.StringToByte(rs.getString(1));
     }
     return password;
  }

  public byte[] getVoterElectionSignature(int voter, int election) throws SQLException, Exception {
      byte[] signature=null;
      this.preGetVoterElectionSignature.setInt(1, voter);
      this.preGetVoterElectionSignature.setInt(2, election);
      ResultSet rs = this.preGetVoterElectionSignature.executeQuery();
      if (rs.next()){
        signature = rs.getBytes(1);
      }
      return signature;
   }

   protected void prepareDelete()throws SQLException{
     super.prepareDelete();
     this.preDeleteVoterElection = this.con.prepareStatement("DELETE FROM voter_election WHERE Id_Voter=?");
   }

 public void deleteVoterElection(int voter) throws SQLException {
  this.preDeleteElection.setInt(1, voter);
  this.preDeleteElection.executeUpdate();
}



}


