package inescID.revs.database;


import java.util.*;
import java.sql.*;
import java.security.NoSuchAlgorithmException;

/**
 * <p>Title: revs</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */


import inescID.revs.util.*;
import inescID.crypto.EncryptionTool;
import inescID.revs.database.*;


public class DB_Anonymizer
    extends DB_ElectionConfiguration{

  //Id_Election, Encrypted_Vote
  PreparedStatement preInsertEncryptedVote1;
  PreparedStatement preInsertEncryptedVote2;

  PreparedStatement mainInsertEncryptedVote;

  PreparedStatement preDeleteVote;

  PreparedStatement preDeleteVote1;
  PreparedStatement preDeleteVote2;

  PreparedStatement preGetVotes1;
  PreparedStatement preGetVotes2;

  PreparedStatement secondaryGetVotes;

  protected boolean mainVoteTable = false;


  public DB_Anonymizer() throws SQLException{
    super();
    this.changeVoteTables();
  }

  public DB_Anonymizer(String db, String user, String password)
      throws java.lang.ClassNotFoundException, SQLException, NoSuchAlgorithmException {
    super();
    this.DBInit(db, user, password);
    this.init();
    this.changeVoteTables();
  }

  public void changeVoteTables(){
    if (this.mainVoteTable)
      this.setSecondaryVoteTable();
    else
      this.setMainVoteTable();
  }

  synchronized protected void setMainVoteTable(){
    this.mainInsertEncryptedVote = this.preInsertEncryptedVote1;

    this.secondaryGetVotes = this.preGetVotes2;

    this.preDeleteVote = this.preDeleteVote2;
    this.mainVoteTable = true;
  }

  synchronized protected void setSecondaryVoteTable(){
    this.mainInsertEncryptedVote = this.preInsertEncryptedVote2;

    this.secondaryGetVotes = this.preGetVotes1;

    this.preDeleteVote = this.preDeleteVote1;
    this.mainVoteTable = false;
  }


  public void createTables(String error) {
    try {
      this.tableVote1();
    }
    catch (SQLException ex) {
      error += "\n Vote1: " + ex.getMessage();
    }
    try {
      this.tableVote2();
    }
    catch (SQLException ex) {
      error += "\n Vote2: " + ex.getMessage();
    }

    super.createTables(error);
  }

  public void removeTables(String error){
    String tables = "DROP TABLE IF EXISTS vote1, vote2";
    try {
      Statement stmt = this.con.createStatement();
      stmt.executeUpdate(tables);
    }
    catch (SQLException ex) {
      error += ex.getMessage();
    }
    super.removeTables(error);
  }

  private void tableVote1 () throws SQLException {
    String table = "CREATE TABLE vote1 (" +
                    "Id int(10) NOT NULL auto_increment," +
                    "Id_Election int(10) unsigned NOT NULL," +
                    "Encrypted_Vote longblob NOT NULL," +
                    "PRIMARY KEY  (Id));";

    Statement stmt = this.con.createStatement();
    stmt.executeUpdate(table);
  }

  private void tableVote2 () throws SQLException {
    String table = "CREATE TABLE vote2 (" +
                    "Id int(10) NOT NULL auto_increment," +
                    "Id_Election int(10) unsigned NOT NULL," +
                    "Encrypted_Vote longblob NOT NULL," +
                    "PRIMARY KEY  (Id));";

    Statement stmt = this.con.createStatement();
    stmt.executeUpdate(table);
  }



 protected void prepareInsert() throws SQLException {
    super.prepareInsert();
    this.preInsertEncryptedVote1 = this.con.prepareStatement(
      "INSERT INTO vote1 (Id_Election, Encrypted_Vote) VALUES ( ?, ?)");
    this.preInsertEncryptedVote2 = this.con.prepareStatement(
      "INSERT INTO vote2 (Id_Election, Encrypted_Vote) VALUES ( ?, ?)");

  }

  synchronized public void insertVote(int election, byte[] encrypted_vote) throws SQLException {
    this.mainInsertEncryptedVote.setInt(1, election);
    this.mainInsertEncryptedVote.setBytes(2, encrypted_vote);
    this.mainInsertEncryptedVote.executeUpdate();
  }

  protected void prepareGet() throws SQLException {
    super.prepareGet();
    this.preGetVotes1 = this.con.prepareStatement(
      "SELECT * FROM vote1");
    this.preGetVotes2 = this.con.prepareStatement(
      "SELECT * FROM vote2");
  }

  synchronized public Vector getVotes() throws
      SQLException, Exception {
    Vector votes = new Vector();
    VoteInfo vote = null;
    ResultSet rs = this.secondaryGetVotes.executeQuery();
    while (rs.next()) {
      vote = new VoteInfo();
      vote.setId(rs.getInt(1));
      vote.setIdElection(rs.getInt(2));
      vote.setEncryptedVote(rs.getBytes(3));
      votes.add(vote);
     }
     return votes;
   }

   protected void prepareDelete()throws SQLException{
     super.prepareDelete();
     this.preDeleteVote1 = this.con.prepareStatement("DELETE FROM vote1 WHERE Id=?");
     this.preDeleteVote2 = this.con.prepareStatement("DELETE FROM vote2 WHERE Id=?");
   }

   public void deleteVote(int id) throws SQLException {
     this.preDeleteVote.setInt(1, id);
     this.preDeleteVote.executeUpdate();
   }

}


