package inescID.revs.database;


import java.util.*;
import java.sql.*;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.security.Key;


/**
 * <p>Title: revs</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

import inescID.revs.util.*;
import inescID.revs.REVSConfiguration;
import inescID.revs.database.*;
import inescID.crypto.KeyStoreTool;


public class DB_Commissioner extends DB_EC_Voters{


  PreparedStatement preInsertVotersGroup; //Id_VotersGroup, Description
  PreparedStatement preInsertDistributor; //Id_Distributor, Description, URL, Enable
  PreparedStatement preInsertAdministrator; //Id_Administrator, Description, URL
  PreparedStatement preInsertAdministratorElection; //Id_Administrator, Id_Election, Public_Key
  PreparedStatement preInsertAnonymizer; //Id_Anonymizer, Description, URL
  PreparedStatement preInsertCounter; //Id_Counter, Description, URL
  PreparedStatement preInsertPasswords; //Id_Voter, Password_1,...,Password_10

  PreparedStatement preUpdateVotersGroup;
  PreparedStatement preUpdateDistributor;
  PreparedStatement preUpdateDistributorPublicKey;
  PreparedStatement preUpdateAdministrator;
  PreparedStatement preUpdateAdministratorPublicKey;
  PreparedStatement preUpdateAdministratorElection;
  PreparedStatement preUpdateAnonymizer;
  PreparedStatement preUpdateAnonymizerPublicKey;
  PreparedStatement preUpdateCounter;
  PreparedStatement preUpdateCounterPublicKey;
  PreparedStatement preUpdatePasswords;

  PreparedStatement preGetVotersGroup;
  PreparedStatement preGetVotersGroupIdByDescription;
  PreparedStatement preGetVotersGroups;
  PreparedStatement preGetDistributors;
  PreparedStatement preGetDistributorPublicKey;
  PreparedStatement preGetDistributorsEnable;
  PreparedStatement preGetAdministrators;
  PreparedStatement preGetAdministratorPublicKey;
  PreparedStatement preGetAdministratorsEnable;
  PreparedStatement preGetAdministratorsElection;
  PreparedStatement preGetAdministratorElection;
  PreparedStatement preGetAnonymizers;
  PreparedStatement preGetAnonymizerPublicKey;
  PreparedStatement preGetAnonymizersEnable;
  PreparedStatement preGetCounters;
  PreparedStatement preGetCounterPublicKey;
  PreparedStatement preGetCountersEnable;
  PreparedStatement preGetPasswords; //by voter

  PreparedStatement preDeleteVotersGroup;
  PreparedStatement preDeleteAdministratorElection;
  PreparedStatement preDeleteAdministratorElectionByAdministrator;
  PreparedStatement preDeleteAdministratorElectionByElection;
  PreparedStatement preDeletePasswordsByVoter;

  PreparedStatement preCountElectionsByVotersGroup;

  KeyStoreTool kst;

  public DB_Commissioner() throws SQLException, NoSuchAlgorithmException {
    super();
    kst = new KeyStoreTool();
  }

  public DB_Commissioner (String db, String user, String password)
      throws java.lang.ClassNotFoundException, SQLException, NoSuchAlgorithmException {
    super();
    this.DBInit(db, user, password);
    this.init();
    kst = new KeyStoreTool();
  }

  public void createTables(String error) {
    try {
      this.tableVotersGroup();
    }
    catch (SQLException ex) {
      error += "\n Voters Group: " + ex.getMessage();
    }
    try {
      this.tableDistributor();
    }
    catch (SQLException ex) {
      error += "\n Distributor: " + ex.getMessage();
    }

    try {
      this.tableAdministrator();
    }
    catch (SQLException ex) {
      error += "\n Administrator: " + ex.getMessage();
    }
    try {
      this.tableAnonymizer();
    }
    catch (SQLException ex) {
      error += "\n Anonimizer: " + ex.getMessage();
    }
    try {
      this.tableCounter();
    }
    catch (SQLException ex) {
      error += "\n Counter: " + ex.getMessage();
    }

    try {
      this.tablePasswords();
    }
    catch (SQLException ex) {
      error += "\n Passwords: " + ex.getMessage();
    }

    try {
      this.tableAdministratorElection();
    }
    catch (SQLException ex) {
      error += "\n AdministratorElection: " + ex.getMessage();
    }

    super.createTables(error);
  }

  public void removeTables(String error) {

    String tables = "DROP TABLE IF EXISTS voters_group, distributor, administrator, "
        + "anonymizer, counter, passwords, administrator_election";
    try {
      Statement stmt = this.con.createStatement();
      stmt.executeUpdate(tables);
    }
    catch (SQLException ex) {
      error += ex.getMessage();
    }
    super.removeTables(error);
  }

  protected void tableElection () throws SQLException {
    String table = "CREATE TABLE election (" +
                   "Id_Election int(10) unsigned auto_increment," +
                   "Description varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL UNIQUE," +
                   "Id_VotersGroup int(10) unsigned NOT NULL," +
                   "Id_Configuration int(10) unsigned NOT NULL," +
                   "Public_Key blob," +
                   "Ballot varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL," +
                   "PRIMARY KEY  (Id_Election))";
    Statement stmt = this.con.createStatement();
    stmt.executeUpdate(table);
  }

  protected void tableConfiguration () throws SQLException {
    String table = "CREATE TABLE configuration (" +
                   "Id_Configuration int(10) unsigned auto_increment," +
                   "Description varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL UNIQUE," +
                   "Start bigint NOT NULL default '0'," +
                   "End bigint NOT NULL default '0'," +
                   "Req_Administrators int NOT NULL," +
                   "Req_Signatures int NOT NULL," +
                   "Enable_Anonymizers char BINARY NOT NULL," +
                   "PRIMARY KEY  (Id_Configuration))";
    Statement stmt = this.con.createStatement();
    stmt.executeUpdate(table);
  }


  protected void tableVotersGroup () throws SQLException {
    String table = "CREATE TABLE voters_group (" +
                   "Id_VotersGroup int(10) unsigned auto_increment," +
                   "Description varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL UNIQUE," +
                   "PRIMARY KEY  (Id_VotersGroup))";
    Statement stmt = this.con.createStatement();
    stmt.executeUpdate(table);

  }


  protected void tableDistributor () throws SQLException {
    String table = "CREATE TABLE distributor (" +
                   "Id_Distributor int(10) unsigned auto_increment," +
                   "Description varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL UNIQUE," +
                   "URL varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL," +
                   "Enable char BINARY NOT NULL," +
                   "Public_Key blob," +
                   "PRIMARY KEY  (Id_Distributor))";
    Statement stmt = this.con.createStatement();
    stmt.executeUpdate(table);
  }

  protected void tableAdministrator () throws SQLException {
    String table = "CREATE TABLE administrator (" +
                   "Id_Administrator int(10) unsigned auto_increment," +
                   "Description varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL UNIQUE," +
                   "URL varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL," +
                   "Enable char BINARY NOT NULL," +
                   "Public_Key blob," +
                   "PRIMARY KEY  (Id_Administrator))";
    Statement stmt = this.con.createStatement();
    stmt.executeUpdate(table);
  }

  protected void tableAdministratorElection () throws SQLException {
  String table = "CREATE TABLE administrator_election (" +
                 "Id_Administrator int(10) unsigned NOT NULL," +
                 "Id_Election int(10) unsigned NOT NULL," +
                 "Public_Key blob," +
                 "PRIMARY KEY  (Id_Administrator, Id_Election))";
  Statement stmt = this.con.createStatement();
  stmt.executeUpdate(table);
}


  protected void tableAnonymizer () throws SQLException {
    String table = "CREATE TABLE anonymizer (" +
                   "Id_Anonymizer int(10) unsigned auto_increment," +
                   "Description varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL UNIQUE," +
                   "URL varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL," +
                   "Enable char BINARY NOT NULL," +
                   "Public_Key blob," +
                   "PRIMARY KEY  (Id_Anonymizer))";
    Statement stmt = this.con.createStatement();
    stmt.executeUpdate(table);
  }

  protected void tableCounter () throws SQLException {
    String table = "CREATE TABLE counter (" +
                   "Id_Counter int(10) unsigned auto_increment," +
                   "Description varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL UNIQUE," +
                   "URL varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL," +
                   "Enable char BINARY NOT NULL," +
                   "Public_Key blob," +
                   "PRIMARY KEY  (Id_Counter))";
    Statement stmt = this.con.createStatement();
    stmt.executeUpdate(table);
  }

  protected void tablePasswords () throws SQLException {
    String passwords = "";
    for (int administrator=1; administrator <= REVSConfiguration.MAX_SERVERS; administrator++){
      passwords += "Password_" + administrator + " blob NOT NULL,";
    }

    String table = "CREATE TABLE passwords (" +
                     "Id_Voter int(10) unsigned NOT NULL," +
                     passwords +
                     "PRIMARY KEY  (Id_Voter))";
      Statement stmt = this.con.createStatement();
      stmt.executeUpdate(table);
  }

  protected void prepareInsert() throws SQLException {
    super.prepareInsert();
    this.preInsertVotersGroup = this.con.prepareStatement("INSERT INTO voters_group " +
        "(Description, Id_VotersGroup) VALUES (?, ?)");
    this.preInsertDistributor = this.con.prepareStatement("INSERT INTO distributor " +
        "(Description, URL, Enable) VALUES (?, ?, ?)");
    this.preInsertAdministrator = this.con.prepareStatement("INSERT INTO administrator " +
        "(Description, URL, Enable) VALUES (?, ?, ?)");
    this.preInsertAdministratorElection = this.con.prepareStatement("INSERT INTO administrator_election " +
        "(Id_Administrator, Id_Election, Public_Key) VALUES (?, ?, ?)");
    this.preInsertAnonymizer = this.con.prepareStatement("INSERT INTO anonymizer " +
        "(Description, URL, Enable) VALUES (?, ?, ?)");
    this.preInsertCounter = this.con.prepareStatement("INSERT INTO counter " +
        "(Description, URL, Enable) VALUES (?, ?, ?)");
    String passwords = "";
    for (int administrator=0; administrator < REVSConfiguration.MAX_SERVERS; administrator++){
      passwords += ", ?";
    }

    this.preInsertPasswords = this.con.prepareStatement("INSERT INTO passwords " +
            "VALUES (?" + passwords + ")");
  }

  protected void prepareUpdate() throws SQLException {
    super.prepareUpdate();
    this.preUpdateVotersGroup = this.con.prepareStatement("UPDATE voters_group SET " +
        "Description=? WHERE Id_VotersGroup=?");
    this.preUpdateDistributor = this.con.prepareStatement("UPDATE distributor SET " +
        "Description=?, URL=?, Enable=? WHERE Id_Distributor=?");
    this.preUpdateDistributorPublicKey = this.con.prepareStatement("UPDATE distributor SET " +
        "Public_Key=? WHERE Id_Distributor=?");
    this.preUpdateAdministrator = this.con.prepareStatement("UPDATE administrator SET " +
        "Description=?, URL=?, Enable=? WHERE Id_Administrator=?");
    this.preUpdateAdministratorPublicKey = this.con.prepareStatement("UPDATE administrator SET " +
        "Public_Key=? WHERE Id_Administrator=?");
    this.preUpdateAdministratorElection = this.con.prepareStatement("UPDATE administrator_election SET " +
        "Public_Key=? WHERE Id_Administrator=? AND Id_Election=?");
    this.preUpdateAnonymizer = this.con.prepareStatement("UPDATE anonymizer SET " +
        "Description=?, URL=?, Enable=? WHERE Id_Anonymizer=?");
    this.preUpdateAnonymizerPublicKey = this.con.prepareStatement("UPDATE anonymizer SET " +
        "Public_Key=? WHERE Id_Anonymizer=?");
    this.preUpdateCounter = this.con.prepareStatement("UPDATE counter SET " +
        "Description=?, URL=?, Enable=? WHERE Id_Counter=?");
    this.preUpdateCounterPublicKey= this.con.prepareStatement("UPDATE counter SET " +
        "Public_Key=? WHERE Id_Counter=?");

    String passwords = "";
    for (int administrator=1; administrator <= REVSConfiguration.MAX_SERVERS; administrator++){
      passwords += ", Password_" + administrator + "=?";
    }
    this.preUpdatePasswords = this.con.prepareStatement("UPDATE passwords SET " +
            "Id_Voter=?" + passwords + "WHERE Id_Voter=?");
  }


  public void updateTableAdministratorElection(int administrator, int election,
                                               Key kpub) throws SQLException {
    PreparedStatement statement = this.preUpdateAdministratorElection;
    statement.setBytes(1, kpub.getEncoded());
    statement.setInt(2, administrator);
    statement.setInt(3, election);
    statement.executeUpdate();
  }

  public void updateDistributorKey(int id, Key kpub) throws SQLException {
    PreparedStatement statement = this.preUpdateDistributorPublicKey;
    statement.setBytes(1, kpub.getEncoded());
    statement.setInt(2, id);
    statement.executeUpdate();
  }

  public void updateAdministratorKey(int id, Key kpub) throws SQLException {
    PreparedStatement statement = this.preUpdateAdministratorPublicKey;
    statement.setBytes(1, kpub.getEncoded());
    statement.setInt(2, id);
    statement.executeUpdate();
  }

  public void updateAnonymizerKey(int id, Key kpub) throws SQLException {
    PreparedStatement statement = this.preUpdateAnonymizerPublicKey;
    statement.setBytes(1, kpub.getEncoded());
    statement.setInt(2, id);
    statement.executeUpdate();
  }

  public void updateCounterKey(int id, Key kpub) throws SQLException {
    PreparedStatement statement = this.preUpdateCounterPublicKey;
    statement.setBytes(1, kpub.getEncoded());
    statement.setInt(2, id);
    statement.executeUpdate();
  }




  public void createAdministratorsElection(int election)
      throws SQLException {
    PreparedStatement statement = this.preInsertAdministratorElection;

    for (int administrator=0; administrator < REVSConfiguration.MAX_SERVERS; administrator++){
      statement.setInt(1, administrator);
      statement.setInt(2, election);
      statement.setBytes(1, null);
      statement.executeUpdate();
    }

  }



  public void alterTableVotersGroup(boolean update, int id,
                                     String description)
      throws SQLException {
    PreparedStatement statement = (update?this.preUpdateVotersGroup:this.preInsertVotersGroup);
    statement.setString(1, description);
    statement.setInt(2, id);
    statement.executeUpdate();
  }

  public void insertTableAdministratorElection(int administrator, int election,
                                               Key key) throws SQLException {
    PreparedStatement statement = this.preInsertAdministratorElection;
    statement.setInt(1, administrator);
    statement.setInt(2, election);
    statement.setBytes(3, key.getEncoded());
    statement.executeUpdate();
  }

  public void insertTableAdministratorElection(int administrator, int election,
                                               byte[] key) throws SQLException {
    PreparedStatement statement = this.preInsertAdministratorElection;
    statement.setInt(1, administrator);
    statement.setInt(2, election);
    statement.setBytes(3, key);
    statement.executeUpdate();
  }



  public void alterTableDistributor(boolean update, Server server)
        throws SQLException {
      PreparedStatement statement = (update?this.preUpdateDistributor:this.preInsertDistributor);
      statement.setString(1, server.GetDescription());
      statement.setString(2, server.GetURL());
      statement.setBoolean(3, server.IsEnable());
      if (update)
        statement.setInt(4, server.GetId());
      statement.executeUpdate();
    }


    public void alterTableAdministrator(boolean update, Server server) throws
        SQLException {
      PreparedStatement statement = (update ? this.preUpdateAdministrator :
                                     this.preInsertAdministrator);
      statement.setString(1, server.GetDescription());
      statement.setString(2, server.GetURL());
      statement.setBoolean(3, server.IsEnable());
      if (update)
        statement.setInt(4, server.GetId());
      statement.executeUpdate();

    }

    public void alterTableAnonymizer(boolean update, Server server) throws
        SQLException {
      PreparedStatement statement = (update ? this.preUpdateAnonymizer :
                                     this.preInsertAnonymizer);
      statement.setString(1, server.GetDescription());
      statement.setString(2, server.GetURL());
      statement.setBoolean(3, server.IsEnable());
      if (update)
        statement.setInt(4, server.GetId());
      statement.executeUpdate();
    }

    public void alterTableCounter(boolean update, Server server) throws
        SQLException {
      PreparedStatement statement = (update ? this.preUpdateCounter :
                                     this.preInsertCounter);
      statement.setString(1, server.GetDescription());
      statement.setString(2, server.GetURL());
      statement.setBoolean(3, server.IsEnable());
      if (update)
        statement.setInt(4, server.GetId());
      statement.executeUpdate();
    }

    public void alterTablePasswords(boolean update,
                                    int voter, byte[][] passwords) throws
        SQLException {
    PreparedStatement statement = (update?this.preUpdatePasswords:this.preInsertPasswords);
    statement.setInt(1, voter);
    for(int i=0; i < passwords.length; i++)
      statement.setBytes(i + 2, passwords[i]);
    if (update)
      statement.setInt(passwords.length + 2, voter);
    statement.executeUpdate();
  }

  protected void prepareGet() throws SQLException {
    super.prepareGet();
    this.preGetVotersGroups = this.con.prepareStatement(
      "SELECT * FROM voters_group ORDER BY Id_VotersGroup");
    this.preGetVotersGroupIdByDescription = this.con.prepareStatement(
      "SELECT Id_VotersGroup FROM voters_group WHERE Description LIKE ?");
    this.preGetVotersGroup = this.con.prepareStatement(
      "SELECT * FROM voters_group WHERE Id_VotersGroup = ?");
    this.preGetDistributors = this.con.prepareStatement(
      "SELECT * FROM distributor ORDER BY Id_Distributor");
    this.preGetDistributorsEnable = this.con.prepareStatement(
      "SELECT * FROM distributor WHERE Enable=1 ORDER BY Id_Distributor");
    this.preGetDistributorPublicKey = this.con.prepareStatement(
      "SELECT Public_Key FROM distributor WHERE Id_Distributor = ?");
    this.preGetAdministrators = this.con.prepareStatement(
      "SELECT * FROM administrator ORDER BY Id_Administrator");
    this.preGetAdministratorsEnable = this.con.prepareStatement(
      "SELECT * FROM administrator WHERE Enable=1 ORDER BY Id_Administrator");
    this.preGetAdministratorsElection = this.con.prepareStatement(
      "SELECT * FROM administrator_election WHERE Id_Election=?");
    this.preGetAdministratorElection = this.con.prepareStatement(
      "SELECT * FROM administrator_election WHERE Id_Election=? AND Id_Administrator=?");
    this.preGetAdministratorPublicKey = this.con.prepareStatement(
      "SELECT Public_Key FROM administrator WHERE Id_Administrator = ?");
    this.preGetAnonymizers = this.con.prepareStatement(
      "SELECT * FROM anonymizer ORDER BY Id_Anonymizer");
    this.preGetAnonymizersEnable = this.con.prepareStatement(
      "SELECT * FROM anonymizer WHERE Enable=1 ORDER BY Id_Anonymizer");
    this.preGetAnonymizerPublicKey = this.con.prepareStatement(
    "SELECT Public_Key FROM anonymizer WHERE Id_Anonymizer = ?");
    this.preGetCounters = this.con.prepareStatement(
      "SELECT * FROM counter ORDER BY Id_Counter");
    this.preGetCounterPublicKey = this.con.prepareStatement(
      "SELECT Public_Key FROM counter WHERE Id_Counter = ?");
    this.preGetCountersEnable = this.con.prepareStatement(
      "SELECT * FROM counter WHERE Enable=1 ORDER BY Id_Counter");
    this.preGetPasswords = this.con.prepareStatement(
      "SELECT * FROM passwords WHERE Id_Voter=?");
  }

  public int getVotersGroupIdByDescription(String description) throws SQLException {
    this.preGetVotersGroupIdByDescription.setString(1, description);
    ResultSet rs = this.preGetVotersGroupIdByDescription.executeQuery();
    if (rs.next()) {
      return rs.getInt(1);
    }
    return 0;
  }

  public int getAvailableVotersGroupId() throws SQLException {
    Vector groups = this.getVotersGroups();
    if(groups.size()==0) return 1;
    int group = ((IdDescription) groups.lastElement()).GetId() + 1;
    return group;
  }



 public Vector getAdministratorsSigningKeysByElection(int election) throws
      InvalidKeySpecException, SQLException {
   this.preGetAdministratorsElection.setInt(1,election);
   AdministratorElection adm = null;
   Vector administrators = new Vector();

   ResultSet rs = this.preGetAdministratorsElection.executeQuery();
   while (rs.next()) {
     adm = new AdministratorElection(rs.getInt(1), rs.getInt(2), rs.getBytes(3));
     administrators.add(adm);
   }
   return administrators;
 }

 public AdministratorElection getAdministratorElection
     (int election, int administrator) throws
      InvalidKeySpecException, SQLException {
   this.preGetAdministratorElection.setInt(1,election);
   this.preGetAdministratorElection.setInt(2,administrator);
   AdministratorElection adm = null;
   ResultSet rs = this.preGetAdministratorElection.executeQuery();
   if (rs.next()) {
     adm = new AdministratorElection(rs.getInt(1), rs.getInt(2), rs.getBytes(3));
   }
   return adm;
 }


 public Vector getVotersGroups() throws SQLException {
    Vector groups = new Vector();
    IdDescription group;
    ResultSet rs = this.preGetVotersGroups.executeQuery();
    while (rs.next()){
      group = new IdDescription(rs.getInt(1), rs.getString(2));
      groups.add(group);
    }
    return groups;
 }

 public IdDescription getVotersGroup(int id_group) throws SQLException {
     IdDescription group=null;
     this.preGetVotersGroup.setInt(1, id_group);
     ResultSet rs = this.preGetVotersGroup.executeQuery();
     if (rs.next()){
       group = new IdDescription(rs.getInt(1), rs.getString(2));
     }
     return group;
  }


public Vector getDistributors(boolean enable) throws SQLException {
    Vector distributors = new Vector();
    Server distributor;
    ResultSet rs = (enable ? this.preGetDistributorsEnable.executeQuery() : this.preGetDistributors.executeQuery());
    while (rs.next()){
      distributor = new Server(rs.getInt(1), rs.getString(2), rs.getString(3), rs.getBoolean(4));
      distributors.add(distributor);
    }
    return distributors;
 }

public Vector getAdministrators(boolean enable) throws SQLException {
    Vector administrators = new Vector();
    Server administrator;
    ResultSet rs = (enable ? this.preGetAdministratorsEnable.executeQuery() : this.preGetAdministrators.executeQuery());
    while (rs.next()){
      administrator = new Server(rs.getInt(1), rs.getString(2), rs.getString(3), rs.getBoolean(4));
      administrators.add(administrator);
    }
    return administrators;
 }

 public Vector getAnonymizers(boolean enable) throws SQLException {
     Vector anonymizers = new Vector();
     Server anonymizer;
     ResultSet rs = (enable ? this.preGetAnonymizersEnable.executeQuery() : this.preGetAnonymizers.executeQuery());
     while (rs.next()){
       anonymizer = new Server(rs.getInt(1), rs.getString(2), rs.getString(3), rs.getBoolean(4));
       anonymizers.add(anonymizer);
     }
     return anonymizers;
  }

 public Vector getCounters(boolean enable) throws SQLException {
      Vector counters = new Vector();
      Server counter;
      ResultSet rs = (enable ? this.preGetCountersEnable.executeQuery() : this.preGetCounters.executeQuery());
      while (rs.next()){
        counter = new Server(rs.getInt(1), rs.getString(2), rs.getString(3), rs.getBoolean(4));
        counters.add(counter);
      }
    return counters;
   }

public Passwords getPasswords(int id) throws SQLException{
     Passwords passwords = new Passwords();
     passwords.SetIdVoter(id);
     this.preGetPasswords.setInt(1, id);
     ResultSet rs = this.preGetPasswords.executeQuery();
     byte[][] pwd = null;
     if(rs.next()){
       pwd = new byte[REVSConfiguration.MAX_SERVERS][];
       for (int i = 0; i<REVSConfiguration.MAX_SERVERS; i++) {
         pwd[i] = rs.getBytes(i+2);
       }
     }
     passwords.SetPasswords(pwd);
     return passwords;
}

   public Key getDistributorPublicKey(int id) throws
        InvalidKeySpecException, SQLException {
     this.preGetDistributorPublicKey.setInt(1, id);
     ResultSet rs = this.preGetDistributorPublicKey.executeQuery();
     if (rs.next()) {
       return kst.getRSAPublicKey(rs.getBytes(1));
     }
     return null;
   }

   public Key getAdministratorPublicKey(int id) throws
       InvalidKeySpecException, SQLException {
     this.preGetAdministratorPublicKey.setInt(1, id);
     ResultSet rs = this.preGetAdministratorPublicKey.executeQuery();
     if (rs.next()) {
       return kst.getRSAPublicKey(rs.getBytes(1));
     }
     return null;
   }

   public Key getAnonymizerPublicKey(int id) throws
        InvalidKeySpecException, SQLException {
     this.preGetAnonymizerPublicKey.setInt(1, id);
     ResultSet rs = this.preGetAnonymizerPublicKey.executeQuery();
     if (rs.next()) {
       return kst.getRSAPublicKey(rs.getBytes(1));
     }
     return null;
   }

   public Key getCounterPublicKey(int id) throws
       InvalidKeySpecException, SQLException {
     this.preGetCounterPublicKey.setInt(1, id);
     ResultSet rs = this.preGetCounterPublicKey.executeQuery();
     if (rs.next()) {
       return kst.getRSAPublicKey(rs.getBytes(1));
     }
     return null;
   }


protected void prepareDelete()throws SQLException{
  super.prepareDelete();
  this.preDeleteElection = this.con.prepareStatement("DELETE FROM election WHERE Id_Election=?");
  this.preDeleteVotersGroup = this.con.prepareStatement("DELETE FROM voters_group WHERE Id_VotersGroup=?");
  this.preDeleteVoter = this.con.prepareStatement("DELETE FROM voter WHERE Id_Voter=?");
  this.preDeleteAdministratorElection = this.con.prepareStatement("DELETE FROM administrator_election WHERE Id_Administrator=? AND Id_Election=?");
  this.preDeleteAdministratorElectionByAdministrator = this.con.prepareStatement("DELETE FROM administrator_election WHERE Id_administrator=?");
  this.preDeleteAdministratorElectionByElection = this.con.prepareStatement("DELETE FROM administrator_election WHERE Id_Election=?");
  this.preDeletePasswordsByVoter = this.con.prepareStatement("DELETE FROM passwords WHERE Id_Voter=?");
 }

 public int deleteConfiguration(int id)throws Exception{
   if (0==this.countElectionsByConfiguration(id)){
     this.preDeleteConfiguration.setInt(1, id);
     this.preDeleteConfiguration.executeUpdate();
     return 1;
   }
   return 0;
 }

 public void deleteElection(int id) throws SQLException {
   this.deleteAdministratorElectionByElection(id);
   this.preDeleteElection.setInt(1, id);
   this.preDeleteElection.executeUpdate();
 }

 public int deleteVotersGroup(int id)throws Exception{
   if (0==this.countElectionsByVotersGroup(id)){
     this.deleteVoter2GroupByGroup(id);
     this.preDeleteVotersGroup.setInt(1, id);
     this.preDeleteVotersGroup.executeUpdate();
     return 1;
   }
   return 0;
 }

 public void deleteVoter(int id)throws SQLException {
   this.deletePasswordsByVoter(id);
   this.deleteVoter2GroupByVoter(id);
   this.preDeleteVoter.setInt(1, id);
   this.preDeleteVoter.executeUpdate();
 }

 public void deleteAdministratorElection(int adm, int election)throws SQLException{
   this.preDeleteAdministratorElection.setInt(1, adm);
   this.preDeleteAdministratorElection.setInt(2, election);
   this.preDeleteAdministratorElection.executeUpdate();
 }


 public void deleteAdministratorElectionByElection(int id)throws SQLException{
   this.preDeleteAdministratorElectionByElection.setInt(1, id);
   this.preDeleteAdministratorElectionByElection.executeUpdate();
 }

 public void deleteAdministratorElectionByAdministrator(int id)throws SQLException{
  this.preDeleteAdministratorElectionByAdministrator.setInt(1, id);
  this.preDeleteAdministratorElectionByAdministrator.executeUpdate();
}


 public void deletePasswordsByVoter(int id)throws SQLException{
   this.preDeletePasswordsByVoter.setInt(1, id);
   this.preDeletePasswordsByVoter.executeUpdate();
 }

 void prepareCount()throws SQLException{
   super.prepareCount();
   this.preCountElectionsByVotersGroup =
       this.con.prepareStatement("SELECT COUNT(Id_Election) FROM election WHERE Id_VotersGroup=?");
 }

 public int countElectionsByVotersGroup(int id) throws Exception {
   int count=0;
   this.preCountElectionsByVotersGroup.setInt(1,id);
   ResultSet rs = this.preCountElectionsByVotersGroup.executeQuery();
   if(rs.next())
     count = rs.getInt(1);
   return count;
 }
}


