package inescID.revs.database;


import java.util.*;
import java.sql.*;
import java.security.NoSuchAlgorithmException;

/**
 * <p>Title: revs</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */


import inescID.revs.util.*;
import inescID.crypto.EncryptionTool;
import inescID.revs.database.*;


public class DB_Counter
    extends DB_ElectionConfiguration{

  //Id_Election, Encrypted_Vote
  PreparedStatement preInsertEncryptedVote;
  //Id, Vote, RandomValue, Signatures
  PreparedStatement preUpdateVote;

  PreparedStatement preGetVotes;
  PreparedStatement preGetEncryptedVotes;
  PreparedStatement preGetEncryptedVotesByElection;
  PreparedStatement preGetVotesByElection;



  public DB_Counter() throws SQLException{
    super();
  }

  public DB_Counter(String db, String user, String password)
      throws java.lang.ClassNotFoundException, SQLException, NoSuchAlgorithmException {
    super();
    this.DBInit(db, user, password);
    this.init();
  }

  public void createTables(String error) {
    try {
          this.tableVote();
        }
        catch (SQLException ex) {
          error += "\n Vote: " + ex.getMessage();
        }
    super.createTables(error);
  }

  public void removeTables(String error){
    String tables = "DROP TABLE IF EXISTS vote";
    try {
      Statement stmt = this.con.createStatement();
      stmt.executeUpdate(tables);
    }
    catch (SQLException ex) {
      error += ex.getMessage();
    }
    super.removeTables(error);
  }

  private void tableVote () throws SQLException {
    String table = "CREATE TABLE vote (" +
                    "Id int(10) NOT NULL auto_increment," +
                    "Id_Election int(10) unsigned NOT NULL," +
                    "Encrypted_Vote longblob NOT NULL," +
                    "Vote longtext," +
                    "Random_Value blob," +
                    "Signatures longblob," +
                    "Valid bit, " +
                    "PRIMARY KEY  (Id));";

    Statement stmt = this.con.createStatement();
    stmt.executeUpdate(table);
  }

 protected void prepareInsert() throws SQLException {
    super.prepareInsert();
    this.preInsertEncryptedVote = this.con.prepareStatement(
      "INSERT INTO vote (Id_Election, Encrypted_Vote) VALUES ( ?, ?)");
  }

  public void insertVote(int election, byte[] encrypted_vote) throws SQLException {
    this.preInsertEncryptedVote.setInt(1, election);
    this.preInsertEncryptedVote.setBytes(2, encrypted_vote);
    this.preInsertEncryptedVote.executeUpdate();
  }

  protected void prepareUpdate() throws SQLException {
    super.prepareUpdate();
    this.preUpdateVote = this.con.prepareStatement(
      "UPDATE vote SET Vote=?, Random_Value=?, Signatures=?, Valid=? WHERE Id = ?");
  }

  public void updateVote(int id, String vote, byte[] random_value,
                         AdministratorSignature[] signatures, boolean valid) throws
      SQLException, Exception {
    EncryptionTool encrypt = new EncryptionTool();
    this.preUpdateVote.setString(1, vote);
    this.preUpdateVote.setBytes(2, random_value);
    this.preUpdateVote.setBytes(3, encrypt.ObjectToByteArray(signatures));
    this.preUpdateVote.setBoolean(4, valid);
    this.preUpdateVote.setInt(5, id);
    this.preUpdateVote.executeUpdate();
  }

  protected void prepareGet() throws SQLException {
    super.prepareGet();
    this.preGetVotes = this.con.prepareStatement(
      "SELECT * FROM vote");;
    this.preGetEncryptedVotes = this.con.prepareStatement(
      "SELECT Id, Id_Election, Encrypted_Vote FROM vote");
    this.preGetEncryptedVotesByElection = this.con.prepareStatement(
      "SELECT Id, Id_Election, Encrypted_Vote FROM vote WHERE Id_Election=?");
    this.preGetVotesByElection = this.con.prepareStatement(
      "SELECT * FROM vote WHERE Id_Election=? AND Valid=? Group by Random_Value");


  }

  public Vector getVotes() throws
      SQLException, Exception {
    EncryptionTool encrypt = new EncryptionTool();
    Vector votes = new Vector();
    VoteInfo vote = null;
    ResultSet rs = this.preGetVotes.executeQuery();
    while (rs.next()) {
      vote = new VoteInfo();
      vote.setId(rs.getInt(1));
      vote.setIdElection(rs.getInt(2));
      vote.setEncryptedVote(rs.getBytes(3));
      vote.setAnswers(rs.getString(4));
      vote.setRandomValue(rs.getBytes(5));
      vote.setValid(rs.getBoolean(6));
      vote.setSignatures((AdministratorSignature[])(encrypt.ByteArrayToObject(rs.getBytes(6))));
      votes.add(vote);
     }
     return votes;
   }

   public Vector getVotesByElection(int election, boolean valid) throws
       SQLException, Exception {
     EncryptionTool encrypt = new EncryptionTool();
     Vector votes = new Vector();
     VoteInfo vote = null;
     this.preGetVotesByElection.setInt(1, election);
     this.preGetVotesByElection.setBoolean(2, valid);
     ResultSet rs = this.preGetVotesByElection.executeQuery();
     while (rs.next()) {
       vote = new VoteInfo();
       vote.setId(rs.getInt(1));
       vote.setIdElection(rs.getInt(2));
       vote.setEncryptedVote(rs.getBytes(3));
       vote.setAnswers(rs.getString(4));
       vote.setRandomValue(rs.getBytes(5));
       vote.setValid(rs.getBoolean(6));
       vote.setSignatures( (AdministratorSignature[]) (encrypt.ByteArrayToObject(
           rs.getBytes(6))));
       votes.add(vote);
     }
     return votes;
   }

   public Vector getEncryptedVotes() throws
      SQLException, Exception {
    Vector votes = new Vector();
    VoteInfo vote = null;
    ResultSet rs = this.preGetEncryptedVotes.executeQuery();
    while (rs.next()) {
      vote = new VoteInfo();
      vote.setId(rs.getInt(1));
      vote.setIdElection(rs.getInt(2));
      vote.setEncryptedVote(rs.getBytes(3));
      votes.add(vote);
     }
     return votes;
   }

   public Vector getEncryptedVotesByElection(int election) throws
      SQLException, Exception {
    Vector votes = new Vector();
    VoteInfo vote = null;
    this.preGetEncryptedVotesByElection.setInt(1, election);
    ResultSet rs = this.preGetEncryptedVotesByElection.executeQuery();
    while (rs.next()) {
      vote = new VoteInfo();
      vote.setId(rs.getInt(1));
      vote.setIdElection(rs.getInt(2));
      vote.setEncryptedVote(rs.getBytes(3));
      votes.add(vote);
     }
     return votes;
   }

}


