package inescID.revs.database;


import java.util.*;
import java.sql.*;
import java.security.NoSuchAlgorithmException;
import java.security.SignedObject;

/**
 * <p>Title: revs</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */


import inescID.revs.REVSConfiguration;
import inescID.revs.util.*;
import inescID.revs.database.*;
import inescID.crypto.EncryptionTool;


public class DB_Distributor
    extends DB_EC_Voters{

  PreparedStatement preUpdateElectionBallot;
  PreparedStatement preGetElectionBallot;
  EncryptionTool encrypt;

  public DB_Distributor() throws SQLException{
    super();
  }

  public DB_Distributor(String db, String user, String password)
      throws java.lang.ClassNotFoundException, SQLException, NoSuchAlgorithmException,
      Exception {
    super();
    this.DBInit(db, user, password);
    this.init();
    this.encrypt = new EncryptionTool();
  }


  protected void tableElection () throws SQLException {
    String table = "CREATE TABLE election (" +
                   "Id_Election int(10) unsigned NOT NULL," +
                   "Description varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL UNIQUE," +
                   "Id_VotersGroup int(10) unsigned," +
                   "Id_Configuration int(10) unsigned," +
                   "Ballot blob, " +
                   "PRIMARY KEY  (Id_Election))";
    Statement stmt = this.con.createStatement();
    stmt.executeUpdate(table);
  }




  protected void prepareInsert() throws SQLException {
    super.prepareInsert();
    this.preInsertElection = this.con.prepareStatement("INSERT INTO election " +
        "(Description, Id_VotersGroup, Id_Configuration, Ballot, Id_Election) " +
        "VALUES (?, ?, ?, ?, ?)");
  }

  protected void prepareUpdate() throws SQLException {
    super.prepareUpdate();
    this.preUpdateElection = this.con.prepareStatement("UPDATE election SET " +
        "Description=?, Id_VotersGroup=?, Id_Configuration=?, Ballot=? " +
        "WHERE Id_Election = ?");
    this.preUpdateElectionBallot = this.con.prepareStatement(
      "UPDATE election SET Ballot=? WHERE Id_Election = ?");
  }

  public void alterTableElection(boolean update, Election election)
      throws SQLException {
    PreparedStatement statement = (update?this.preUpdateElection:this.preInsertElection);
    statement.setString(1, election.GetDescription());
    statement.setInt(2, election.GetIdVotersGroup());
    statement.setInt(3, election.GetIdConfiguration());
    statement.setBytes(4, null);
    statement.setInt(5, election.GetId());
    statement.executeUpdate();
}

public void updateElectionBallot(int election, SignedObject ballot)
    throws
      SQLException, Exception {
  PreparedStatement statement = this.preUpdateElectionBallot;
  statement.setBytes(1, encrypt.ObjectToByteArray(ballot));
  statement.setInt(2, election);
  statement.executeUpdate();
}


  protected void prepareGet() throws SQLException {
    super.prepareGet();
    this.preGetElectionBallot = this.con.prepareStatement(
      "SELECT Ballot FROM election WHERE Id_Election=?");
  }

  public Election getElection(int id) throws
       SQLException {
    this.preGetElection.setInt(1, id);
    Election election = null;
    ResultSet rs = this.preGetElection.executeQuery();
    if (rs.next()) {
      election = new Election(rs.getInt(1), rs.getString(2), rs.getInt(3), rs.getInt(4));
    }
    return election;
  }

  public Vector getElectionsByConfiguration(int configuration) throws
      SQLException {
    Vector elections = new Vector();
    Election election = null;
    this.preGetElectionsByConfiguration.setInt(1, configuration);
    ResultSet rs = this.preGetElectionsByConfiguration.executeQuery();
    while (rs.next()) {
      election = new Election(rs.getInt(1), rs.getString(2), rs.getInt(3), rs.getInt(4));
      elections.add(election);
     }
     return elections;
   }

   public Vector getElections() throws
        SQLException {
      Vector elections = new Vector();
      Election election = null;
      ResultSet rs = this.preGetElections.executeQuery();
      while (rs.next()) {
        election = new Election(rs.getInt(1), rs.getString(2), rs.getInt(3), rs.getInt(4));
        elections.add(election);
       }
       return elections;
     }

     public SignedObject getElectionBallot(int election) throws
      SQLException, Exception {
       this.preGetElectionBallot.setInt(1, election);
       ResultSet rs = this.preGetElectionBallot.executeQuery();
       if (rs.next()) {
         return (SignedObject) encrypt.ByteArrayToObject(rs.getBytes(1));
       }
       return null;
     }

     public Vector getElectionsByVoter(int voter) throws
         SQLException {
       Vector elections = new Vector();
       IdDescription election = null;
       Vector groups = this.getGroupsByVoter(voter);
       for (int i = 0; i < groups.size(); i++) {
         this.preGetElectionsByGroup.setInt(1, ((Integer)groups.get(i)).intValue());
         ResultSet rs = this.preGetElectionsByGroup.executeQuery();
         while (rs.next()) {
           election = new IdDescription(rs.getInt(1), rs.getString(2));
           elections.add(election);
         }
       }
       return elections;
     }

/*
     public Vector getElectionsByVoter(int voter) throws
            SQLException {
          Vector elections = new Vector();
          IdDescription election = null;
          this.preGetElectionsByVoter.setInt(1, voter);
          ResultSet rs = this.preGetElectionsByVoter.executeQuery();
          while (rs.next()) {
            election = new IdDescription(rs.getInt(1), rs.getString(2));
            elections.add(election);
           }
           return elections;
         }
*/
}


