package inescID.revs.database;

import java.util.*;
import java.sql.*;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.security.Key;


/**
 * <p>Title: revs</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

import inescID.revs.util.*;
import inescID.revs.REVSConfiguration;

public class DB_EC_Voters
    extends DB_ElectionConfiguration{

  public static final String EMPTY_VOTERS_GROUP = "Empty";
  public static final String ALL_VOTERS_GROUP = "All Voters";

  public static final int EMPTY_VOTERS_GROUP_ID = 1;
  public static final int ALL_VOTERS_GROUP_ID = 2;

  PreparedStatement preInsertVoter; //Id_Voter, Name
  PreparedStatement preUpdateVoter;
  PreparedStatement preGetVoter;
  PreparedStatement preGetVoters;
  PreparedStatement preGetVotersWithoutPassword;
  PreparedStatement preDeleteVoter;


  PreparedStatement preInsertVoter2Group; //Id_VotersGroup, Id_Voter
  PreparedStatement preGetVoters2Group;
  PreparedStatement preGetAllVotersID;
  PreparedStatement preGetElectionsByGroup;
  PreparedStatement preGetGroupsByVoter;
  PreparedStatement preVerifyVoterElection;

  PreparedStatement preDeleteVoter2GroupByVoter;
  PreparedStatement preDeleteVoter2GroupByGroup;

  public DB_EC_Voters() throws SQLException{
    super();
  }

  public DB_EC_Voters(String db, String user, String password)
      throws java.lang.ClassNotFoundException, SQLException, NoSuchAlgorithmException {
    super();
    this.DBInit(db, user, password);
    this.init();
  }

  public void createTables(String error) {
    try {
      this.tableVoter();
    }
    catch (SQLException ex) {
      error += "\n Voter: " + ex.getMessage();
    }
    try {
      this.tableVoter2Group();
    }
    catch (SQLException ex) {
      error += "\n Voter 2 Group " + ex.getMessage();
    }

    super.createTables(error);

  }

  public void removeTables(String error) {
    String tables = "DROP TABLE IF EXISTS voter, voters2group";
    try {
      Statement stmt = this.con.createStatement();
      stmt.executeUpdate(tables);
    }
    catch (SQLException ex) {
      error += ex.getMessage();
    }

    super.removeTables(error);
  }

  protected void tableVoter () throws SQLException {
      String table = "CREATE TABLE voter (" +
                     "Id_Voter int(10) unsigned NOT NULL UNIQUE," +
                     "Name varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ")," +
                     "PRIMARY KEY  (Id_Voter))";
      Statement stmt = this.con.createStatement();
      stmt.executeUpdate(table);
    }

  protected void tableVoter2Group () throws SQLException {
      String table = "CREATE TABLE voters2group (" +
                     "Id_Voter int(10) unsigned NOT NULL," +
                     "Id_VotersGroup int(10) unsigned NOT NULL," +
                     "PRIMARY KEY  (Id_Voter, Id_VotersGroup))";
      Statement stmt = this.con.createStatement();
      stmt.executeUpdate(table);
  }


  protected void prepareInsert() throws SQLException {
    super.prepareInsert();

    this.preInsertVoter = this.con.prepareStatement("INSERT INTO voter " +
        "(Name, Id_Voter) VALUES (?, ?)");
    //Id_VotersGroup, Id_Voter
    this.preInsertVoter2Group = this.con.prepareStatement("INSERT INTO voters2group " +
        "VALUES (?, ?)");
  }

  protected void prepareUpdate() throws SQLException {
    super.prepareUpdate();
    this.preUpdateVoter = this.con.prepareStatement("UPDATE voter SET " +
        "Name=? WHERE Id_Voter=?");
  }


  public void alterTableVoter(boolean update, int id,
                                String name)
       throws SQLException {
     PreparedStatement statement = (update?this.preUpdateVoter:this.preInsertVoter);
     statement.setString(1, name);
     statement.setInt(2, id);
     statement.executeUpdate();
   }


  public void addVoters2Group(Voters2Group group)throws SQLException{
    for(int i=0; i<group.GetVoters().size();i++){
      this.insertTableVoter2Group(((Integer) group.GetVoters().get(i)).intValue(),
                                  group.GetIdGroup());
    }
  }

  public void insertTableVoter2Group(int voter, int group)
      throws SQLException {
    this.preInsertVoter2Group.setInt(1, voter);
    this.preInsertVoter2Group.setInt(2, group);
    this.preInsertVoter2Group.executeUpdate();
  }


  protected void prepareGet() throws SQLException {
    super.prepareGet();
    this.preGetVoter = this.con.prepareStatement(
      "SELECT * FROM voter WHERE Id_Voter=?");
    this.preGetVoters = this.con.prepareStatement(
      "SELECT * FROM voter ORDER BY Id_Voter");
    this.preGetVotersWithoutPassword = this.con.prepareStatement(
      "SELECT voter.* FROM voter LEFT JOIN passwords ON voter.Id_Voter=passwords.Id_Voter "+
      "WHERE passwords.Id_Voter IS NULL ORDER BY Id_Voter");
    this.preGetVoters2Group = this.con.prepareStatement(
      "SELECT Id_Voter FROM voters2group WHERE Id_VotersGroup=?");

    this.preGetAllVotersID = this.con.prepareStatement(
      "SELECT Id_Voter FROM voter ORDER BY Id_Voter");


    this.preGetElectionsByGroup = this.con.prepareStatement(
      "SELECT * " +
      "FROM election "+
      "WHERE Id_VotersGroup=?"
      );

    this.preGetGroupsByVoter = this.con.prepareStatement(
        "SELECT Id_VotersGroup " +
        "FROM voters2group "+
        "WHERE Id_Voter=?"
      );

    this.preVerifyVoterElection = this.con.prepareStatement(
      "SELECT voter.Id_Voter "+
      "FROM voter, voters2group, election "+
      "WHERE voter.Id_Voter=? "+
      "AND election.Id_Election=? "+
      "AND ((election.Id_VotersGroup=voters2group.Id_VotersGroup "+
      "AND voter.Id_Voter=voters2group.Id_Voter) "+
      "OR election.Id_VotersGroup = 2) "+
      "Group By voter.Id_Voter "
      );

  }

  public Vector getElectionsByVoter(int voter) throws
         InvalidKeySpecException, SQLException {
       Vector elections = new Vector();
       Election election = null;
       Vector groups = this.getGroupsByVoter(voter);
       for(int i=0; i<groups.size();i++){
         this.preGetElectionsByGroup.setInt(1, ((Integer)groups.get(i)).intValue());
         ResultSet rs = this.preGetElectionsByGroup.executeQuery();
         while (rs.next()) {
           election = new Election(rs.getInt(1), rs.getString(2), rs.getInt(3),
                                   rs.getInt(4),
                                   rs.getBytes(5),
                                   rs.getString(6));
           elections.add(election);
         }
       }
        return elections;
      }


  public boolean verifyVoterElection(int voter, int election) throws SQLException, Exception {
    this.preVerifyVoterElection.setInt(1, voter);
    this.preVerifyVoterElection.setInt(2, election);
    ResultSet rs = this.preVerifyVoterElection.executeQuery();
    if (rs.next())
      return true;
    return false;
 }


  public IdDescription getVoter(int id) throws SQLException {
     IdDescription voter=null;
     this.preGetVoter.setInt(1, id);
     ResultSet rs = this.preGetVoter.executeQuery();
     if (rs.next()){
       voter = new IdDescription(rs.getInt(1), rs.getString(2));
     }
     return voter;
  }

   public Vector getVoters() throws SQLException {
      Vector voters = new Vector();
      IdDescription voter;
      ResultSet rs = this.preGetVoters.executeQuery();
      while (rs.next()){
        voter = new IdDescription(rs.getInt(1), rs.getString(2));
        voters.add(voter);
      }
      return voters;
   }

   public Vector getVotersWithoutPassword() throws SQLException {
      Vector voters = new Vector();
      IdDescription voter;
      ResultSet rs = this.preGetVotersWithoutPassword.executeQuery();
      while (rs.next()){
        voter = new IdDescription(rs.getInt(1), rs.getString(2));
        voters.add(voter);
      }
      return voters;
   }


 public Vector getGroupsByVoter(int id_voter) throws SQLException {
     this.preGetGroupsByVoter.setInt(1, id_voter);
     Vector groups = new Vector();
     ResultSet rs = this.preGetGroupsByVoter.executeQuery();
     while (rs.next()) {
       groups.add(new Integer(rs.getInt(1)));
     }
     return groups;
   }


public Voters2Group getVoters2Group(int id_group) throws SQLException {
  Vector voters = new Vector();
  Voters2Group group = new Voters2Group();
  PreparedStatement aux;
  if (id_group == this.ALL_VOTERS_GROUP_ID)
    aux = this.preGetAllVotersID;
  else {
    this.preGetVoters2Group.setInt(1, id_group);
    aux = this.preGetVoters2Group;
  }
  group.SetIdGroup(id_group);
  ResultSet rs = aux.executeQuery();
  while (rs.next()) {
    voters.add(new Integer(rs.getInt(1)));
  }
  group.SetVoters(voters);
  return group;
}

protected void prepareDelete()throws SQLException{
  super.prepareDelete();
  this.preDeleteVoter = this.con.prepareStatement("DELETE FROM voter WHERE Id_Voter=?");
  this.preDeleteVoter2GroupByVoter = this.con.prepareStatement("DELETE FROM voters2group WHERE Id_Voter=?");
  this.preDeleteVoter2GroupByGroup = this.con.prepareStatement("DELETE FROM voters2group WHERE Id_VotersGroup=?");;
}

public void deleteVoter(int id)throws SQLException {
  this.deleteVoter2GroupByVoter(id);
  this.preDeleteVoter.setInt(1, id);
  this.preDeleteVoter.executeUpdate();
}


 public void deleteVoter2GroupByVoter(int id)throws SQLException{
   this.preDeleteVoter2GroupByVoter.setInt(1, id);
   this.preDeleteVoter2GroupByVoter.executeUpdate();
 }

 public void deleteVoter2GroupByGroup(int id)throws SQLException{
   this.preDeleteVoter2GroupByGroup.setInt(1, id);
   this.preDeleteVoter2GroupByGroup.executeUpdate();
 }
}


