package inescID.revs.database;

import java.util.*;
import java.sql.*;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.security.Key;



/**
 * <p>Title: revs</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */
import inescID.crypto.KeyStoreTool;
import inescID.revs.util.*;
import inescID.revs.REVSConfiguration;
import inescID.revs.database.*;


public class DB_ElectionConfiguration
    extends CommonDatabase{

  PreparedStatement preInsertElection; //Id_Election, Description, Id_VotersGroup, Id_Configuration, Public_Key, Ballot
  PreparedStatement preInsertConfiguration; //Id_Configuration, Description, Start, End, Req_Administrators, Req_Signatures, Enable_Anonymizers

  PreparedStatement preUpdateElection;
  PreparedStatement preUpdateElectionKey;
  PreparedStatement preUpdateConfiguration;

  PreparedStatement preGetElectionsIdDescription;
  PreparedStatement preGetElection;
  PreparedStatement preGetElectionIdByDescription;
  PreparedStatement preGetElections;
  PreparedStatement preGetElectionsByConfiguration;
  PreparedStatement preGetElectionsIdDescriptionByConfiguration;
  PreparedStatement preGetConfigurationsIdDescription;
  PreparedStatement preGetConfiguration;
  PreparedStatement preGetConfigurationIdByDescription;
  PreparedStatement preGetConfigurations;

  PreparedStatement preDeleteElection;
  PreparedStatement preDeleteConfiguration;

  PreparedStatement preGetLastId;
  PreparedStatement preCountElectionsByConfiguration;


  public DB_ElectionConfiguration() throws SQLException{
    super();
  }

  public DB_ElectionConfiguration(String db, String user, String password)
      throws java.lang.ClassNotFoundException, SQLException, NoSuchAlgorithmException {
    super();
    this.DBInit(db, user, password);
    this.init();
  }

  public void createTables(String error) {
    try {
      this.tableElection();
    }
    catch (SQLException ex) {
      error += "\n Election: " + ex.getMessage();
    }
    try {
      this.tableConfiguration();
    }
    catch (SQLException ex) {
      error += "\n Configuration: " + ex.getMessage();
    }
    super.createTables(error);
  }

  public void removeTables(String error) {
    String tables = "DROP TABLE IF EXISTS election, configuration";
    try {
      Statement stmt = this.con.createStatement();
      stmt.executeUpdate(tables);
    }
    catch (SQLException ex) {
      error += ex.getMessage();
    }
    super.removeTables(error);
  }


  protected void tableElection () throws SQLException {
    String table = "CREATE TABLE election (" +
                   "Id_Election int(10) unsigned NOT NULL," +
                   "Description varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL UNIQUE," +
                   "Id_VotersGroup int(10) unsigned," +
                   "Id_Configuration int(10) unsigned," +
                   "Public_Key blob," +
                   "Ballot longtext," +
                   "PRIMARY KEY  (Id_Election))";
    Statement stmt = this.con.createStatement();
    stmt.executeUpdate(table);
  }

  protected void tableConfiguration () throws SQLException {
    String table = "CREATE TABLE configuration (" +
                   "Id_Configuration int(10) unsigned NOT NULL," +
                   "Description varchar(" + REVSConfiguration.MAX_STRING_LENGTH + ") NOT NULL UNIQUE," +
                   "Start bigint default '0'," +
                   "End bigint default '0'," +
                   "Req_Administrators int," +
                   "Req_Signatures int," +
                   "Enable_Anonymizers char BINARY," +
                   "PRIMARY KEY  (Id_Configuration))";
    Statement stmt = this.con.createStatement();
    stmt.executeUpdate(table);
  }



  protected void prepareInsert() throws SQLException {
    this.preInsertElection = this.con.prepareStatement("INSERT INTO election " +
        "(Description, Id_VotersGroup, Id_Configuration, Public_Key, Ballot, Id_Election) " +
        "VALUES (?, ?, ?, ?, ?, ?)");
    this.preInsertConfiguration = this.con.prepareStatement("INSERT INTO configuration " +
        "(Description, Start, End, Req_Administrators, Req_Signatures, Enable_Anonymizers, Id_Configuration) " +
        "VALUES (?, ?, ?, ?, ?, ?, ?)");
  }

  protected void prepareUpdate() throws SQLException {
    this.preUpdateElection = this.con.prepareStatement("UPDATE election SET " +
        "Description=?, Id_VotersGroup=?, Id_Configuration=?, Public_Key=?, Ballot=? " +
        "WHERE Id_Election = ?");
    this.preUpdateElectionKey = this.con.prepareStatement(
      "UPDATE election SET Public_Key=? WHERE Id_Election = ?");
    this.preUpdateConfiguration = this.con.prepareStatement("UPDATE configuration SET " +
        "Description=?, Start=?, End=?, Req_Administrators=?, Req_Signatures=?, Enable_Anonymizers=? " +
        "WHERE Id_Configuration=?");
  }


  public void alterTableElection(boolean update, Election election)
      throws SQLException {
    PreparedStatement statement = (update?this.preUpdateElection:this.preInsertElection);
    statement.setString(1, election.GetDescription());
    statement.setInt(2, election.GetIdVotersGroup());
    statement.setInt(3, election.GetIdConfiguration());

    statement.setBytes(4, election.GetKey());

    statement.setString(5, election.GetBallot());
    statement.setInt(6, election.GetId());
    statement.executeUpdate();
  }

  public void updateElectionKey(int election, Key key)
      throws SQLException {
    PreparedStatement statement = this.preUpdateElectionKey;
    statement.setBytes(1, key.getEncoded());
    statement.setInt(2, election);
    statement.executeUpdate();
  }


  public void alterTableConfiguration(boolean update, Configuration configuration)
      throws SQLException {
    PreparedStatement statement = (update?this.preUpdateConfiguration:this.preInsertConfiguration);
    statement.setString(1, configuration.GetDescription());
    statement.setLong(2, configuration.GetStart());
    statement.setLong(3, configuration.GetEnd());
    statement.setInt(4, configuration.GetReqAdministrators());
    statement.setInt(5, configuration.GetReqSignatures());
    statement.setBoolean(6, configuration.UseAnonymizers());
    statement.setInt(7, configuration.GetId());
    statement.executeUpdate();
  }

  protected void prepareGet() throws SQLException {
    this.preGetLastId = this.con.prepareStatement(
      "SELECT LAST_INSERT_ID()");
    this.preGetElectionsIdDescription = this.con.prepareStatement(
      "SELECT Id_Election, Description FROM election ORDER BY Id_Election");
    this.preGetElectionsIdDescriptionByConfiguration = this.con.prepareStatement(
      "SELECT Id_Election, Description FROM election WHERE Id_Configuration=?");
    this.preGetElectionsByConfiguration = this.con.prepareStatement(
      "SELECT * FROM election WHERE Id_Configuration=?");
    this.preGetElection = this.con.prepareStatement(
      "SELECT * FROM election WHERE Id_Election=?");
    this.preGetElections = this.con.prepareStatement(
      "SELECT * FROM election ORDER BY Id_Election");
    this.preGetConfigurationsIdDescription = this.con.prepareStatement(
      "SELECT Id_Configuration, Description FROM configuration ORDER BY Id_Configuration");
    this.preGetConfiguration = this.con.prepareStatement(
      "SELECT * FROM configuration WHERE Id_Configuration=?");
    this.preGetConfigurations = this.con.prepareStatement(
      "SELECT * FROM configuration ORDER by Id_Configuration");
    this.preGetElectionIdByDescription = this.con.prepareStatement(
      "SELECT Id_Election FROM election WHERE Description LIKE ?");
    this.preGetConfigurationIdByDescription = this.con.prepareStatement(
      "SELECT Id_Configuration FROM configuration WHERE Description LIKE ?");
  }

public int getLastID() throws SQLException {
   int id=-1;
   ResultSet rs = this.preGetLastId.executeQuery();
   if(rs.next()){
       id = rs.getInt(1);
   }
   return id;
}

 public int getElectionIdByDescription(String description) throws SQLException {
   this.preGetElectionIdByDescription.setString(1, description);
   ResultSet rs = this.preGetElectionIdByDescription.executeQuery();
   if (rs.next()) {
     return rs.getInt(1);
   }
   return 0;
 }

 public int getAvailableElectionId() throws SQLException {
   Vector elections = this.getElectionsIdDescription();
   if(elections.size()==0) return 1;
   int election = ((IdDescription) elections.lastElement()).GetId() + 1;
   return election;
 }

 public int getAvailableConfigurationId() throws SQLException {
   Vector configurations = this.getConfigurationsIdDescription();
   if(configurations.size()==0) return 1;
   int configuration = ( (IdDescription) configurations.lastElement()).GetId() + 1;
   return configuration;
 }


 public int getConfigurationIdByDescription(String description) throws SQLException {
   this.preGetConfigurationIdByDescription.setString(1, description);
   ResultSet rs = this.preGetConfigurationIdByDescription.executeQuery();
   if (rs.next()) {
     return rs.getInt(1);
   }
   return 0;
 }




public Vector getElectionsIdDescription() throws SQLException {
   Vector elections = new Vector();
   IdDescription election;
   ResultSet rs = this.preGetElectionsIdDescription.executeQuery();
   while (rs.next()){
     election = new IdDescription(rs.getInt(1), rs.getString(2));
     elections.add(election);
   }
   return elections;
}

public Vector getElectionsIdDescriptionByConfiguration(int configuration) throws SQLException {
    Vector elections = new Vector();
    IdDescription election;
    this.preGetElectionsIdDescriptionByConfiguration.setInt(1,configuration);
    ResultSet rs = this.preGetElectionsIdDescriptionByConfiguration.executeQuery();
    while (rs.next()){
      election = new IdDescription(rs.getInt(1), rs.getString(2));
      elections.add(election);
    }
    return elections;
 }


 public Election getElection(int id) throws
      InvalidKeySpecException, SQLException {
   this.preGetElection.setInt(1, id);
   Election election = null;
   ResultSet rs = this.preGetElection.executeQuery();
   if (rs.next()) {
     election = new Election(rs.getInt(1), rs.getString(2), rs.getInt(3), rs.getInt(4),
                             rs.getBytes(5),
                             rs.getString(6));
   }
   return election;
 }

 public Vector getElectionsByConfiguration(int configuration) throws
     InvalidKeySpecException, SQLException {
   Vector elections = new Vector();
   Election election = null;
   this.preGetElectionsByConfiguration.setInt(1, configuration);
   ResultSet rs = this.preGetElectionsByConfiguration.executeQuery();
   while (rs.next()) {
     election = new Election(rs.getInt(1), rs.getString(2), rs.getInt(3), rs.getInt(4),
                             rs.getBytes(5),
                             rs.getString(6));
     elections.add(election);
    }
    return elections;
  }

  public Vector getElections() throws
       InvalidKeySpecException, SQLException {
     Vector elections = new Vector();
     Election election = null;
     ResultSet rs = this.preGetElections.executeQuery();
     while (rs.next()) {
       election = new Election(rs.getInt(1), rs.getString(2), rs.getInt(3), rs.getInt(4),
                               rs.getBytes(5),
                               rs.getString(6));
       elections.add(election);
      }
      return elections;
    }


 public Vector getConfigurationsIdDescription() throws SQLException {
    Vector configurations = new Vector();
    IdDescription configuration;
    ResultSet rs = this.preGetConfigurationsIdDescription.executeQuery();
    while (rs.next()){
      configuration = new IdDescription(rs.getInt(1), rs.getString(2));
      configurations.add(configuration);
    }
    return configurations;
 }

 public Vector getConfigurations() throws SQLException {
    Vector configurations = new Vector();
    Configuration configuration;
    ResultSet rs = this.preGetConfigurations.executeQuery();
    while (rs.next()){
      configuration = new Configuration(rs.getInt(1), rs.getString(2), rs.getLong(3), rs.getLong(4),
                             rs.getInt(5), rs.getInt(6), rs.getBoolean(7));
      configurations.add(configuration);
    }
    return configurations;
 }


 public Configuration getConfiguration(int id) throws SQLException {
   this.preGetConfiguration.setInt(1, id);
   Configuration configuration = null;
   ResultSet rs = this.preGetConfiguration.executeQuery();
   if (rs.next()) {
     configuration = new Configuration(rs.getInt(1), rs.getString(2), rs.getLong(3), rs.getLong(4),
                             rs.getInt(5), rs.getInt(6), rs.getBoolean(7));
   }
   return configuration;
 }





protected void prepareDelete()throws SQLException{
  this.preDeleteElection = this.con.prepareStatement("DELETE FROM election WHERE Id_Election=?");
  this.preDeleteConfiguration = this.con.prepareStatement("DELETE FROM configuration WHERE Id_Configuration=?");
}

 public void deleteElection(int id) throws SQLException {
  this.preDeleteElection.setInt(1, id);
  this.preDeleteElection.executeUpdate();
}


 public int deleteConfiguration(int id)throws Exception{
   this.preDeleteConfiguration.setInt(1, id);
   this.preDeleteConfiguration.executeUpdate();
   return 1;
 }

 void prepareCount()throws SQLException{
   this.preCountElectionsByConfiguration =
       this.con.prepareStatement("SELECT COUNT(Id_Election) FROM election WHERE Id_Configuration=?");
 }


 public int countElectionsByConfiguration(int id) throws Exception {
   int count=0;
   this.preCountElectionsByConfiguration.setInt(1,id);
   ResultSet rs = this.preCountElectionsByConfiguration.executeQuery();
   if(rs.next())
     count = rs.getInt(1);
   return count;
}

public boolean verifyElectionPeriod (int election) throws SQLException,
      InvalidKeySpecException {
  Election e = this.getElection(election);
  Configuration conf = this.getConfiguration(e.GetIdConfiguration());

  long now = System.currentTimeMillis();
  if ((now < conf.GetStart()) || (now > conf.GetEnd()))
    return false;
  return true;
}

  public void init() throws NoSuchAlgorithmException, SQLException {
    prepareInsert();
    prepareUpdate();
    prepareGet();
    prepareDelete();
    prepareCount();
  }
}


