package inescID.revs.servers;

import java.rmi.RemoteException;
import java.security.KeyStoreException;
import java.security.cert.CertificateException;
import java.security.NoSuchAlgorithmException;
import java.security.interfaces.RSAPrivateKey;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.net.PasswordAuthentication;
import java.security.UnrecoverableKeyException;
import java.rmi.RemoteException;
import java.util.Vector;
import java.util.Arrays;
import java.rmi.*;
import java.sql.*;
import java.security.KeyPair;



/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

import inescID.crypto.*;
import inescID.crypto.blindSignatures.RSABlindSignature;
import inescID.revs.util.*;
import inescID.revs.util.protocol.*;
import inescID.revs.database.DB_Administrator;
import inescID.revs.REVSConfiguration;
import inescID.util.*;
import java.security.spec.*;


public class AdministratorEngine
    extends ServerEngine implements IAdministrator{
  DB_Administrator database;
  RSABlindSignature blind_signature;
  PasswordGenerator password_generator;
  FileTool log;
  //long start;
  //long end;
  //long aux;

  public AdministratorEngine(String address, char[] keystore_pass, char[] key_pass)
      throws RemoteException, KeyStoreException, CertificateException, NoSuchAlgorithmException,
      IOException, UnrecoverableKeyException {
    super(address, keystore_pass, key_pass);
  }

  public AdministratorEngine(String address, char[] keystore_pass, char[] key_pass,
                      boolean noSSL) throws RemoteException, IOException, NoSuchAlgorithmException, CertificateException, KeyStoreException,
      FileNotFoundException, UnrecoverableKeyException {
    super(address, keystore_pass, key_pass, true);
  }


  public Vector createElectionsSigningKeys(Vector elections){
    try {
      Election election;
      Vector elections_keys = new Vector();
      EncryptionTool encrypt = new EncryptionTool();
      KeyPair kp;
      System.out.println("Creating Administrator's signing keys.");
      for (int i = 0; i < elections.size(); i++) {
        election = (Election) elections.get(i);
        kp = encrypt.genRSAKeyPair();
        election.SetKey(kp.getPrivate());
        this.database.deleteElection(election.GetId());
        this.database.alterTableElection(false, election);

        election.SetKey(kp.getPublic());
        elections_keys.add(election);
        System.out.print(".");
      }
      System.out.println();
      return elections_keys;
    }
    catch (Exception ex) {
      return null;
    }
  }


  /**
   * updateDatabase
   *
   * @param obj Object
   */
  public void updateDatabase(String file) {
    try {
      FileEncryptionTool file_tool = new FileEncryptionTool();
      //configuration
      TransferAdministrator ta = (TransferAdministrator) file_tool.readObjectFromFile(file, this.commissioner_key);
      Configuration conf = new Configuration(ta.getConfiguration());
      this.database.deleteConfiguration(conf.GetId());
      this.database.alterTableConfiguration(false, conf);
      Vector aux;

      //Voters2Group
      aux = ta.getVotersGroups();
      Voters2Group group;
      for(int i=0; i<aux.size(); i++){
        group = (Voters2Group) aux.get(i);
        this.database.deleteVoter2GroupByGroup(group.GetIdGroup());
        this.database.addVoters2Group(group);
      }

      //elections
      aux = ta.getElections();
      Election election;
      for(int i=0; i<aux.size(); i++){
        election = (Election) aux.get(i);
        try {
            Election current = this.database.getElection(election.GetId());
            election.SetKey(current.GetKey());
            this.database.alterTableElection(true, election);

            System.out.println("Election: " + election.GetDescription() +
                               " updated.");
        }
        catch (Exception ex2) {
          this.database.alterTableElection(false, election);
          System.out.println("Election: " + election.GetDescription() + " added.");
        }

      }

      EncryptionTool encrypt = new EncryptionTool();
      MixCipherEnvelope envelope;

      //passwords
      IdPassword password;
      envelope = ta.getPasswords();
      byte[] data = encrypt.DecryptMixCipherMode(envelope, this.server_key);
      aux = (Vector) encrypt.ByteArrayToObject(data);
      for (int i = 0; i < aux.size(); i++) {
        password = (IdPassword) aux.get(i);
        //    this.database.deleteVoter(password.getId());
        try {
          this.database.alterTableVoter(false, password.getId(), password.getPassword());
        }
        catch (SQLException ex1) {
          this.database.alterTableVoter(true, password.getId(), password.getPassword());
          System.err.println("Voter " + password.getId() +
                             " password replaced.");
        }
      }

      //signing keys
      envelope = ta.getSigningKeys();
      KeyStoreTool kst = new KeyStoreTool();
      AdministratorElection signing_key;
      if(envelope != null){
        data = encrypt.DecryptMixCipherMode(envelope, this.server_key);
        aux = (Vector) encrypt.ByteArrayToObject(data);
        for (int i = 0; i < aux.size(); i++) {
          signing_key = (AdministratorElection) aux.get(i);
          this.database.updateElectionKey(signing_key.GetElection(),
                                          kst.getRSAPrivateKey(signing_key.GetKey()));
        }
      }
    }
    catch (Exception ex) {
      System.err.println("Error processing data file. Process aborted. " + ex.getMessage());
      ex.printStackTrace();
    }
  }



  /**
   * init
   *
   * @param database_address String
   * @param db_authentication PasswordAuthentication
   */
  public void init(String database_address,
                   PasswordAuthentication db_authentication) {
    try {
      this.database = new DB_Administrator(database_address,
                                         db_authentication.getUserName(),
                                         new String(db_authentication.getPassword()));
      this.db_alias = this.database;
    }
    catch (Exception ex) {
      System.err.println("Error connecting to database. " + ex.getMessage());
    }

    try {
      this.blind_signature = new RSABlindSignature();
//      this.blind_signature.initBlindSign(this.server_key);
      this.password_generator = new PasswordGenerator();
    }
    catch (NoSuchAlgorithmException ex1) {
      System.err.println("Error initializing crypto engine. " + ex1.getMessage());
    }

/*
    try {
      this.log = new FileTool();
      this.log.initWrite("Adm" + REVSConfiguration.LOG);
    }
    catch (Exception ex2) {
      System.err.println("Error openning log file. " + ex2.getMessage());
    }
*/

  }



  /**
   * sign
   *
   * @param voter int
   * @param password byte[]
   * @param election int
   * @param vote byte[]
   * @return Reply
   */
  synchronized public Reply sign(int voter, byte[] password, int election, byte[] vote) {

    try {
   //   start = System.currentTimeMillis();
      Reply reply;
      if (!(this.database.verifyElectionPeriod(election)))
        reply = new Reply(ErrorCodes.ERROR_TIME, null);
      else{
        byte[] passwd = this.database.getVoterPassword(voter);
        password = this.password_generator.getHashValue(password);

        if (Arrays.equals(passwd, password)) {
          byte[] signature = this.database.getVoterElectionSignature(voter,
              election);
          if (signature == null) {
      //      aux = System.currentTimeMillis();

            this.blind_signature.initBlindSign( (RSAPrivateKey)
                                               this.database.getElection(
                election).GetPrivateKey());
            signature = this.blind_signature.blindSign(vote);

        //    end = System.currentTimeMillis();
          //  this.log.writeLine("Signature - " + (end-aux));

            this.database.insertTableVoterElection(voter, election, signature);
            reply = new Reply(ErrorCodes.OK, signature);
          }
          else
           reply = new Reply(ErrorCodes.PREVIOUS_SIGNATURE, signature);
        }
        else
          reply = new Reply(ErrorCodes.ERROR_AUTHENTICATION, null);
      }
    //  end = System.currentTimeMillis();
    //  this.log.writeLine(reply.getErrorCode() + " - " + (end-start));
      return reply;
    }
    catch (Exception ex) {
      ex.printStackTrace();
    //  end = System.currentTimeMillis();
      /*
      try{
        this.log.writeLine("Error - " + (end - start));
      }
      catch (Exception e){System.err.println("Log error");}
      */
      return new Reply(ErrorCodes.ERROR, null);
    }

  }

}
