package inescID.revs.servers;

import java.rmi.RemoteException;
import java.security.KeyStoreException;
import java.security.cert.CertificateException;
import java.security.NoSuchAlgorithmException;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.net.PasswordAuthentication;
import java.security.UnrecoverableKeyException;
import java.rmi.RemoteException;
import java.util.Vector;
import java.rmi.*;
import java.sql.*;



/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

import inescID.crypto.*;
import inescID.revs.util.*;
import inescID.revs.database.DB_Anonymizer;
import inescID.revs.util.protocol.*;
import java.security.*;
import java.security.*;
import java.io.*;
import inescID.revs.REVSConfiguration;
import inescID.util.InputTool;
import java.net.*;

public class AnonymizerEngine
    extends ServerEngine implements ISubmission{
  DB_Anonymizer database;
  String forward_server = null;
  int max_votes2forward;
  int max_forward_interval;
  AnonymizerSubmission time;

  public AnonymizerEngine(String address, char[] keystore_pass, char[] key_pass)
      throws RemoteException, KeyStoreException, CertificateException, NoSuchAlgorithmException,
      IOException, UnrecoverableKeyException {
    super(address, keystore_pass, key_pass);
  }

  public AnonymizerEngine(String address, char[] keystore_pass, char[] key_pass,
                      boolean noSSL) throws RemoteException, IOException, NoSuchAlgorithmException, CertificateException, KeyStoreException,
      FileNotFoundException, UnrecoverableKeyException {
  super(address, keystore_pass, key_pass, true);
  }


  public void setForwardServer(String url){
    this.forward_server = url;
  }

  public void setMaxVotes2Forward(int max_votes){
    this.max_votes2forward = max_votes;
  }


  public void setMaxForwardInterval(int millis){
    this.max_forward_interval = millis;
  }


  public boolean existsForwardServer(){
    return (forward_server == null)?false:true;
  }


  /*
   * updateDatabase
   */
  public void updateDatabase(String file) {
    try {
      FileEncryptionTool file_tool = new FileEncryptionTool();
      //configuration
      TransferAnonymizerCounter tac = (TransferAnonymizerCounter)
          file_tool.readObjectFromFile(file, this.commissioner_key);
      Configuration conf = new Configuration(tac.getConfiguration());
      this.database.deleteConfiguration(conf.GetId());
      this.database.alterTableConfiguration(false, conf);

      //elections
      Vector elections = tac.getElections();
      Election election;
      for(int i=0; i<elections.size(); i++){
        election = (Election) elections.get(i);
        this.database.deleteElection(election.GetId());
        this.database.alterTableElection(false, election);
      }
   }
    catch (Exception ex) {
      System.err.println("Error processing data file. Process aborted. " + ex.getMessage());
      ex.printStackTrace();
    }
  }





  public boolean start(){

    if (!existsForwardServer())
      return false;

    try {
      InputTool input = new InputTool();
      String aux = input.readLine(
          "Max votes per submission. Enter for defaults: ");

      try {
        this.max_votes2forward = Integer.parseInt(aux);
      }
      catch (Exception ex1) {
        aux = null;
      }
      if (aux == null) {
        this.max_votes2forward = REVSConfiguration.DEFAULT_VOTES2FORWARD;
      }

      aux = input.readLine("Max submission interval (ms). Enter for defaults: ");
      try {
        this.max_forward_interval = Integer.parseInt(aux);
      }
      catch (Exception ex2) {
        aux = null;
      }
      if (aux == null) {
        this.max_forward_interval = REVSConfiguration.DEFAULT_FORWARD_INTERVAL;
      }

      this.time = new AnonymizerSubmission(this, this.max_forward_interval);
      this.time.start();
    }
    catch (Exception ex) {
      ex.printStackTrace();
      return false;
    }


    return super.start();

  }

  public void stop() {
    super.stop();
    this.time.stop_safe();
  }


  /**
   * init
   *
   * @param database_address String
   * @param db_authentication PasswordAuthentication
   */
  public void init(String database_address,
                   PasswordAuthentication db_authentication) {
    try {
      this.database = new DB_Anonymizer(database_address,
                                         db_authentication.getUserName(),
                                         new String(db_authentication.getPassword()));
      this.db_alias = this.database;
    }
    catch (Exception ex) {
      System.err.println("Error connecting to database. " + ex.getMessage());
    }
  }

  /**
   * submitVote
   *
   * @param election int
   * @param vote Vote
   * @return Reply
   */
  synchronized public Reply submitVote(int election, byte[] vote) {
    try {
      // ISubmission forward = (ISubmission) Naming.lookup(this.forward_server);
      // Reply reply = forward.submitVote(election, vote);
      // return reply;

      this.database.insertVote(election, vote);
      return new Reply(ErrorCodes.OK, null);
    }
    catch (Exception ex) {
      ex.printStackTrace();
      return new Reply(ErrorCodes.ERROR, null);
    }
  }


  synchronized public void forwardVotes() {

    ISubmission forward;
    try {
      forward = (ISubmission) Naming.lookup(this.forward_server);
    }
    catch (Exception ex1) {
      System.err.println("Error: counter not available");
      return;
    }


    try {
      this.database.changeVoteTables();
      Vector votes = this.database.getVotes();

      PasswordGenerator random = new PasswordGenerator();
      int n_votes = random.GetRandomNumber(0, (votes.size()>this.max_votes2forward)?
                                           this.max_votes2forward :
                                           votes.size());
//      System.out.println("votes: " + n_votes);
      int aux;
      VoteInfo vote;
      Reply reply;

      for(int v=0; v < n_votes; v++){
        aux = random.GetRandomNumber(0, votes.size()-1);
        vote = (VoteInfo) votes.get(aux);
        reply = forward.submitVote(vote.getElection(), vote.getEncryptedVote());
        if (reply.getErrorCode() == ErrorCodes.OK) {
          this.database.deleteVote(vote.getId());
          votes.remove(aux);
        }
      }
      }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }



}
