package inescID.revs.servers;

import java.rmi.RemoteException;
import java.security.KeyStoreException;
import java.security.cert.CertificateException;
import java.security.NoSuchAlgorithmException;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.net.PasswordAuthentication;
import java.security.UnrecoverableKeyException;
import java.rmi.RemoteException;
import java.util.Vector;
import java.rmi.*;
import java.sql.*;
import javax.crypto.SecretKey;
import java.security.interfaces.*;


/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

import inescID.crypto.*;
import inescID.crypto.blindSignatures.*;
import inescID.util.*;
import inescID.revs.util.*;
import inescID.revs.database.DB_Counter;
import inescID.revs.util.protocol.*;
import java.security.*;
import java.security.*;
import java.io.*;
import inescID.revs.REVSConfiguration;
import inescID.crypto.blindSignatures.RSABlindSignature;
import java.net.*;
import java.security.spec.*;

public class CounterEngine
    extends ServerEngine implements ICounter, ISubmission{
  DB_Counter database;


  public CounterEngine(String address, char[] keystore_pass, char[] key_pass)
      throws RemoteException, KeyStoreException, CertificateException, NoSuchAlgorithmException,
      IOException, UnrecoverableKeyException {
    super(address, keystore_pass, key_pass);
  }

  public CounterEngine(String address, char[] keystore_pass, char[] key_pass,
                      boolean noSSL) throws RemoteException, IOException, NoSuchAlgorithmException, CertificateException, KeyStoreException,
      FileNotFoundException, UnrecoverableKeyException {
  super(address, keystore_pass, key_pass, true);
  }



  public void updateDatabase(String file) {
    try {
      FileEncryptionTool file_tool = new FileEncryptionTool();
      //configuration
      TransferAnonymizerCounter tac = (TransferAnonymizerCounter)
          file_tool.readObjectFromFile(file, this.commissioner_key);
      Configuration conf = new Configuration(tac.getConfiguration());
      this.database.deleteConfiguration(conf.GetId());
      this.database.alterTableConfiguration(false, conf);

      //elections
      Vector elections = tac.getElections();
      Election election;
      for(int i=0; i<elections.size(); i++){
        election = (Election) elections.get(i);
        this.database.deleteElection(election.GetId());
        this.database.alterTableElection(false, election);
      }
   }
    catch (Exception ex) {
      System.err.println("Error processing data file. Process aborted. " + ex.getMessage());
      ex.printStackTrace();
    }
  }



  /**
   * init
   *
   * @param database_address String
   * @param db_authentication PasswordAuthentication
   */
  public void init(String database_address,
                   PasswordAuthentication db_authentication) {
    try {
      this.database = new DB_Counter(database_address,
                                         db_authentication.getUserName(),
                                         new String(db_authentication.getPassword()));
      this.db_alias = this.database;
    }
    catch (Exception ex) {
      System.err.println("Error connecting to database. " + ex.getMessage());
    }
  }

  /**
   * submitVote
   *
   * @param election int
   * @param vote Vote
   * @return Reply
   */
  synchronized public Reply submitVote(int election, byte[] vote) {
    try {
      this.database.insertVote(election, vote);
      return new Reply(ErrorCodes.OK, null);
    }
    catch (SQLException ex) {
      ex.printStackTrace();
      return new Reply(ErrorCodes.ERROR, null);
    }
  }

  /**
   * getVotes
   *
   * @return Reply
   */
  synchronized public Reply getVotes() {
    try {
      Vector votes = this.database.getEncryptedVotes();
      return new Reply(ErrorCodes.OK, votes);
    }
    catch (Exception ex) {
      return new Reply(ErrorCodes.ERROR, null);
    }
  }

  synchronized public void gatherVotes(Vector counters) {
    Vector votes;
    VoteInfo vote;
    Server counter;
    for (int i=0; i<counters.size(); i++){
      counter = (Server) counters.get(i);
      if (!counter.GetURL().equals(this.server_address)){
        try {
          ICounter aux = (ICounter) Naming.lookup(counter.GetURL());
          Reply rep = aux.getVotes();
          votes = (Vector) rep.getObject();
          for (int v=0; v<votes.size(); v++){
            vote = (VoteInfo) votes.get(v);
            this.database.insertVote(vote.getElection(), vote.getEncryptedVote());
          }
        }
        catch (Exception ex) {
          System.err.println("Error collecting votes from " + counter.GetDescription());
        }
      }
    }
  }

  synchronized public void decipherVotes() {
    InputTool input = new InputTool();

    try {
      String elections_keys_file = input.readLine("Elections keys file: ");
      String secret_key_file = input.readLine("Secret key file: ");

      FileEncryptionTool ft = new FileEncryptionTool();
      byte[] data_encrypted =
          (byte[]) ft.readObjectFromFile(elections_keys_file, this.commissioner_key);
      SecretKey secret_key =
          (SecretKey) ft.readObjectFromFile(secret_key_file, this.commissioner_key);

      EncryptionTool encrypt = new EncryptionTool();
      byte[] data = encrypt.DecryptTripleDES(data_encrypted, secret_key);
      VerificationData verification_data =
          (VerificationData) encrypt.ByteArrayToObject(data);

      Configuration configuration = verification_data.getConfiguration();


      if(configuration.GetEnd() > System.currentTimeMillis()){
        System.err.println("Election period is not over.");
        return;
      }

      System.out.println("\n\n" + configuration.GetDescription());
      System.out.println("Required signatures: " + configuration.GetReqSignatures());
      System.out.println("Of " + configuration.GetReqAdministrators() + " administrators.");

      Vector aux = verification_data.getSigningKeys();
      Vector elections_keys = verification_data.getElectionsKeys();
      AdministratorElection adm;
      AdministratorSignature[] signatures;
      IdKey election_key;
      Vector encrypted_votes;
      Election election;
      MixCipherEnvelope envelope;
      VoteInfo vote_info;
      Vote vote;
      int valid_signatures;
      byte[] signature;
      RSABlindSignature sign = new RSABlindSignature();
      PasswordGenerator pass_gen = new PasswordGenerator();
      byte[] vote_bytes=null;
      byte[]vote_hash;
      byte[] committed_vote;
      boolean valid;


      int max_adm_id = 0;
      for (int i = 0; i < aux.size(); i++) {
        adm = (AdministratorElection) aux.get(i);
        if (max_adm_id < adm.GetAdministrator())
          max_adm_id = adm.GetAdministrator();
      }
      AdministratorElection[] administrators =
          new AdministratorElection[max_adm_id + 1];

      for (int e=0; e<elections_keys.size(); e++){
        election = (Election) elections_keys.get(e);
        System.out.println("\n\n" + election.GetDescription());

        for (int i = 0; i < aux.size(); i++) {
          adm = (AdministratorElection) aux.get(i);
          if (adm.GetElection()==election.GetId())
            administrators[adm.GetAdministrator()] = adm;
        }

        encrypted_votes = this.database.getEncryptedVotesByElection(election.GetId());

        for(int v=0; v < encrypted_votes.size(); v++){
          try {
            vote_info = (VoteInfo) encrypted_votes.get(v);
            System.out.println("\nVote " + vote_info.getId());

            envelope = (MixCipherEnvelope)
                encrypt.ByteArrayToObject(vote_info.getEncryptedVote());
            vote = (Vote) encrypt.ByteArrayToObject(encrypt.
                DecryptMixCipherMode(
                envelope, (RSAPrivateKey) election.GetPrivateKey()));

            signatures = vote.getSignatures();
            valid_signatures = 0;
            for (int s = 0; s < signatures.length; s++) {
              try {
                if (signatures[s]==null)
                  continue;

                adm = administrators[signatures[s].getAdministrator()];
                signature = signatures[s].getSignature();
                System.out.print("Administrator " + adm.GetAdministrator());

                vote_bytes = vote.getAnswers().getBytes(REVSConfiguration.CHARSET);
                vote_hash = pass_gen.getHashValue(vote_bytes);
                committed_vote = pass_gen.getCommittedHashValue(vote_hash,
                    vote.getRandomValue());
                sign.initVerify(kst.getRSAPublicKey(adm.GetKey()));
                if (sign.verifyBlindSignature(committed_vote, signature)) {
                  System.out.println(" Signature valid.");
                  valid_signatures++;
                }
                else {
                  System.out.println(" Signature invalid.");
                }
              }
              catch (Exception ex1) {
                System.out.println("Error verifying signature.");
                ex1.printStackTrace();
              }
            }


            if (valid_signatures < configuration.GetReqSignatures()) {
              System.out.println(
                  "Vote invalid.\nNot enough valid signatures:\nRequired: " +
                  configuration.GetReqSignatures() +
                  "\nSignatures: " + valid_signatures);
              valid = false;
            }
            else {
              System.out.println("Vote valid.");
              valid = true;
            }

            this.database.updateVote(vote_info.getId(), vote.getAnswers(),
                         vote.getRandomValue(),
                         vote.getSignatures(), valid);
          }
          catch (Exception ex2) {
            System.out.println("Error decyphering vote.");
            ex2.printStackTrace();
          }
        }

    }

    this.publishResults();
  }
    catch (Exception ex) {
      ex.printStackTrace();
    }

  }



  protected void publishResults() {

    Vector elections=null;
    Vector votes=null;
    Election election;
    Vote vote;

    try {
      elections = this.database.getElections();
    }
    catch (Exception ex) {
    }

    int invalid_votes=0;
    int valid_votes=0;

    File dir = null;
    FileTool index = new FileTool();
    String result_path_init = "results";

 //   FileTool resultstotals = new FileTool();

    try {
      dir = new File(result_path_init);
      dir.mkdirs();

  //    resultstotals.initWrite(result_path_init + File.separator + "totals.htm");

      index.initWrite(result_path_init + File.separator + "index.htm");
      index.writeLine("<html>");
      index.writeLine("<head>");
      index.writeLine("<title>REVS results report</title>");
      index.writeLine("</head>");
      index.writeLine("<body>");
      index.writeLine("<H1><FONT face=\"Monotype Corsiva\" size=\"7\">REVS</FONT></H1>");
      index.writeLine("<H2>Results report</H2>");
      index.writeLine("<TABLE cellSpacing=\"1\" cellPadding=\"3\" border=\"1\">");
      index.writeLine("<tr>");
      index.writeLine("<th>Code</th>");
      index.writeLine("<th>Election description</th>");
      index.writeLine("<th colspan=\"2\">Results</th>");
      index.writeLine("<th>Total of votes</th>");
      index.writeLine("<th>Valid votes</th>");
      index.writeLine("<th>Invalid votes</th>");
      index.writeLine("</tr>");

    }
    catch (Exception ex2) {
      System.err.println("Error: Cannot create results directory/file");
    }

    for (int e = 0; e < elections.size(); e++) {
      election = (Election) elections.get(e);
      try {
        votes = this.database.getVotesByElection(election.GetId(), false);
        invalid_votes = votes.size();
        votes = this.database.getVotesByElection(election.GetId(), true);
        valid_votes = votes.size();
      }
      catch (Exception ex1) {
      }

      FileTool results = new FileTool();
      FileTool results2 = new FileTool();
      String result_path = result_path_init + File.separator + "election" +
          election.GetId();
      try {
        dir = new File(result_path);
        dir.mkdirs();

        result_path += File.separator;
        results.initWrite(result_path + "final_tally.xml");
        results2.initWrite(result_path + "final_tally2.xml");

        results.writeLine("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
        results.writeLine(
            "<?xml-stylesheet type=\"text/xsl\" href=\"resultsXSLT.xslt\"?>");

        results2.writeLine("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
        results2.writeLine(
            "<?xml-stylesheet type=\"text/xsl\" href=\"results2XSLT.xslt\"?>");



        String aux = new String();
        aux += "<results>\n";
        aux += "<election code=\"" + election.GetId() + "\">" +
                          election.GetDescription() + "</election>\n";
        aux += "<totalVotes>" + (valid_votes + invalid_votes) + "</totalVotes>\n";
        aux += "<validVotes>" + valid_votes + "</validVotes>\n";
        aux += "<invalidVotes>" + invalid_votes + "</invalidVotes>\n";
        aux += "<ballot>ballot.xml</ballot>";

        results.writeLine(aux);
        results2.writeLine(aux);

        String ballot_path = result_path + "ballot.xml";
/*
        results.writeLine("<results>");
        results.writeLine("<election code=\"" + election.GetId() + "\">" +
                          election.GetDescription() + "</election>");

        results.writeLine("<totalVotes>" + (valid_votes + invalid_votes) + "</totalVotes>");
        results.writeLine("<validVotes>" + valid_votes + "</validVotes>");
        results.writeLine("<invalidVotes>" + invalid_votes + "</invalidVotes>");

        String ballot_path = result_path + "ballot.xml";
        results.writeLine("<ballot>ballot.xml</ballot>");
*/

        index.writeLine("<tr>");
        index.writeLine("<td>"+ election.GetId() +"</td>");
        index.writeLine("<td>"+ election.GetDescription() + "</td>");
        index.writeLine("<td><a href=\"" + "election" + election.GetId() +
                        "/final_tally2.xml" + "\">Overall</a></td>");
        index.writeLine("<td><a href=\"" + "election" + election.GetId() +
                                "/final_tally.xml" + "\">Details</a></td>");
        index.writeLine("<td>" + (valid_votes + invalid_votes) + "</td>");
        index.writeLine("<td>" + valid_votes + "</td>");
        index.writeLine("<td>" + invalid_votes + "</td>");
        index.writeLine("</tr>");



        Election e1 = this.database.getElection(election.GetId());
        FileTool f_ballot = new FileTool();
        f_ballot.initWrite(ballot_path);
        f_ballot.write(e1.GetBallot());
        f_ballot.closeWriteFile();

      }
      catch (Exception ex3) {
        System.err.println("Error: Cannot create results directory/file");
      }
      catch (Throwable t1) {
        System.err.println("Error: Cannot create results directory/file");
      }



      for (int v = 0; v < votes.size(); v++) {
        vote = (Vote) votes.get(v);
        try {
          FileTool vote_file = new FileTool();
          String vote_path = result_path + "vote" + v + ".xml";
          vote_file.initWrite(vote_path);
          vote_file.writeLine("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
          vote_file.write(vote.getAnswers());
          vote_file.closeWriteFile();

          results.writeLine("<vote>vote" + v + ".xml</vote>");
          results2.writeLine("<vote>vote" + v + ".xml</vote>");

   //       resultstotals.writeLine("<vote>" + vote_path + "</vote>");
        }
        catch (Exception ex5) {
          System.err.println("Error writing ballot to file.");
        }
        catch (Throwable ex4) {
          System.err.println("Error writing ballot to file.");
        }

      }

      try {
        results.writeLine("</results>");
        results.closeWriteFile();
        results2.writeLine("</results>");
        results2.closeWriteFile();
        this.resultViews(result_path);
      } catch (Exception ex5) {
        System.err.println("Error writing results file");
      }
      catch (Throwable ex4) {
        System.err.println("Error writing results file");
      }
    }
    try {
      index.writeLine("</TABLE>");
      index.writeLine("</body>");
      index.writeLine("</html>");

      index.closeWriteFile();
  //    resultstotals.closeWriteFile();
    }
    catch (IOException ex6) {
      System.err.println("Error writing results file");
    }
    catch (Throwable ex6) {
      System.err.println("Error writing results file");
    }

  }


  protected void resultViews(String result_path){
    try {
      FileTool.copy(REVSConfiguration.CONFIGURATION_PATH + "resultsXSLT.xslt" ,
                    result_path + "resultsXSLT.xslt");
      FileTool.copy(REVSConfiguration.CONFIGURATION_PATH + "results2XSLT.xslt" ,
              result_path + "results2XSLT.xslt");

    }
    catch (IOException ex) {
      System.err.println("Error writing result views");
    }
  }
}



