package inescID.revs.servers;

import java.rmi.RemoteException;
import java.security.KeyStoreException;
import java.security.cert.CertificateException;
import java.security.NoSuchAlgorithmException;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.net.PasswordAuthentication;
import java.security.UnrecoverableKeyException;
import java.rmi.RemoteException;
import java.util.Vector;
import java.security.SignedObject;




/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

import inescID.crypto.*;
import inescID.revs.util.*;
import inescID.revs.util.protocol.*;
import inescID.revs.database.DB_Distributor;
import inescID.revs.REVSConfiguration;
import inescID.util.*;

import java.rmi.*;
import java.sql.*;



public class DistributorEngine extends ServerEngine implements IDistributor{
  DB_Distributor database;
  FileTool log;
 // long start;
 // long end;

  public DistributorEngine(String address, char[] keystore_pass, char[] key_pass)
      throws RemoteException, KeyStoreException, CertificateException, NoSuchAlgorithmException,
      IOException, UnrecoverableKeyException {
    super(address, keystore_pass, key_pass);
  }

  public DistributorEngine(String address, char[] keystore_pass, char[] key_pass, boolean noSSL)
      throws RemoteException, IOException, NoSuchAlgorithmException, CertificateException, KeyStoreException,
      FileNotFoundException, UnrecoverableKeyException {
    super(address, keystore_pass, key_pass, true);
  }

  /**
   * updateDatabase
   *
   * @param obj Object
   */
  public void updateDatabase(String file) {
    try {
      FileEncryptionTool file_tool = new FileEncryptionTool();
      //configuration
      TransferDistributor td = (TransferDistributor) file_tool.readObjectFromFile(file, this.commissioner_key);
      Configuration conf = new Configuration(td.getConfiguration());
      this.database.deleteConfiguration(conf.GetId());
      this.database.alterTableConfiguration(false, conf);

      //voters
      Vector aux = td.getVoters();
      IdDescription voter;
      for(int i=0; i< aux.size(); i++){
        voter = (IdDescription) aux.get(i);
        this.database.deleteVoter(voter.GetId());
        this.database.alterTableVoter(false, voter.GetId(), voter.GetDescription());
      }

      //Voters2Group
      aux = td.getVotersGroups();
      Voters2Group group;
      for(int i=0; i<aux.size(); i++){
        group = (Voters2Group) aux.get(i);
        this.database.deleteVoter2GroupByGroup(group.GetIdGroup());
        this.database.addVoters2Group(group);
      }

      //elections
      aux = td.getElections();
      Election election;
      for(int i=0; i<aux.size(); i++){
        election = (Election) aux.get(i);
        this.database.deleteElection(election.GetId());
        this.database.alterTableElection(false, election);
      }

      //ballots
      SignedObject so;
      Ballot ballot;
      SigningTool sign = new SigningTool();
      aux = td.getBallots();
      for(int i=0; i<aux.size(); i++){
        so = (SignedObject) aux.get(i);
        if (!sign.verifyObject(this.commissioner_key, so))
          throw new Exception("Error verifying signature on ballot.");
        ballot = (Ballot) so.getObject();
        this.database.updateElectionBallot(ballot.getElection(), so);
      }
    }
    catch (Exception ex) {
      System.err.println("Error processing data file. Process aborted. " + ex.getMessage());
      ex.printStackTrace();
    }
  }



  /**
   * init
   *
   * @param database_address String
   * @param db_authentication PasswordAuthentication
   */
  public void init(String database_address,
                   PasswordAuthentication db_authentication) {
    try {
      this.database = new DB_Distributor(database_address,
                                         db_authentication.getUserName(),
                                         new String(db_authentication.getPassword()));
      this.db_alias = this.database;
    }
    catch (Exception ex) {
      System.err.println("Error connecting to database. " + ex.getMessage());
    }

/*
    try {
      this.log = new FileTool();
      this.log.initWrite("Dist" + REVSConfiguration.LOG);
    }
    catch (Exception ex2) {
      System.err.println("Error openning log file. " + ex2.getMessage());
    }
*/
  }





  /**
   * getElectionBallot
   *
   * @param voter int
   * @param election int
   * @return Reply
   */
  synchronized public Reply getElectionBallot(int voter, int election) {
    Reply reply;

    try {
   //   start = System.currentTimeMillis();
      if (!(this.database.verifyElectionPeriod(election)))
        return new Reply(ErrorCodes.ERROR_TIME, null);

      if (this.database.verifyVoterElection(voter, election))
        reply = new Reply(ErrorCodes.OK, this.database.getElectionBallot(election));
      else
        reply = new Reply(ErrorCodes.VOTER_NOT_IN_ELECTION, null);
    }
    catch (Exception ex) {
      reply = new Reply(ErrorCodes.ERROR, null);
      System.err.println("Error: GetElectionBallot. " + ex.getMessage());
    }

/*
    try {
      end = System.currentTimeMillis();
      this.log.writeLine("Get ballot - " + (end - start));
    }
    catch (Exception ex1) {
      System.err.println("Log Error. " + ex1.getMessage());
    }
*/
  return reply;
  }


  /**
   * getElections
   *
   * @param voter int
   * @return Reply
   */
  synchronized public Reply getElections(int voter) {
    try {
     // start = System.currentTimeMillis();

      IdDescription voterID = this.database.getVoter(voter);
      if (voterID == null)
        return new Reply(ErrorCodes.ERROR_AUTHENTICATION, null);

      Vector elections = this.database.getElectionsByVoter(voter);
      Vector final_elections = new Vector();
      IdDescription election;
      for(int i=0; i<elections.size(); i++){
        election = (IdDescription) elections.get(i);
        if (this.database.verifyElectionPeriod(election.GetId()))
          final_elections.add(election);
      }

/*
      try {
        end = System.currentTimeMillis();
        this.log.writeLine("Get Elections - " + (end - start));
      }
      catch (Exception ex1) {
        System.err.println("Log Error. " + ex1.getMessage());
      }
*/

      return new Reply(ErrorCodes.OK,
                       new VoterElection(voterID, final_elections));
    }
    catch (Exception ex) {
      System.err.println("Error: GetElections. " + ex.getMessage());
      ex.printStackTrace();
      return new Reply(ErrorCodes.ERROR, null);

    }
   }

}
