package inescID.revs.servers;

import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.server.UnicastRemoteObject;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.interfaces.*;
import java.security.KeyStoreException;
import java.io.FileNotFoundException;
import java.security.UnrecoverableKeyException;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

import inescID.RMIwithSSL.*;
import inescID.crypto.KeyStoreTool;
import inescID.revs.database.*;
import inescID.revs.REVSConfiguration;

public abstract class ServerEngine extends UnicastRemoteObject implements CommonServerInterface{
  String server_address;
  KeyStoreTool kst;
  RSAPrivateKey server_key;
  RSAPublicKey commissioner_key;
  CommonDatabase db_alias;


  public ServerEngine(String address, char[] keystore_pass,
                      char[] key_pass) throws RemoteException, IOException,
      NoSuchAlgorithmException, CertificateException, KeyStoreException,
      FileNotFoundException, UnrecoverableKeyException, KeyStoreException,
      CertificateException, NoSuchAlgorithmException, IOException {

    super(0, new RMISSLClientSocketFactory(),
          new RMISSLServerSocketFactory());
    this.configure_server(address, keystore_pass, key_pass);
  }

  public ServerEngine(String address, char[] keystore_pass, char[] key_pass, boolean noSSL) throws
      RemoteException, KeyStoreException, CertificateException,
      NoSuchAlgorithmException, IOException, UnrecoverableKeyException,
      KeyStoreException, CertificateException, NoSuchAlgorithmException,
      IOException {

    super();
    this.configure_server(address, keystore_pass, key_pass);
  }

 void configure_server(String address, char[] keystore_pass, char[] key_pass) throws
      IOException, NoSuchAlgorithmException, CertificateException,
      KeyStoreException, FileNotFoundException, KeyStoreException,
      NoSuchAlgorithmException, UnrecoverableKeyException {
    kst = new KeyStoreTool(REVSConfiguration.PRIVATE_KEYSTORE, keystore_pass);
    this.commissioner_key = (RSAPublicKey) kst.getPublicKeyFromCertFile(REVSConfiguration.COMMISSIONER_CERT_FILE);
    this.server_key = (RSAPrivateKey) kst.getPrivateKey(REVSConfiguration.ALIAS_SERVER_KEY,
        key_pass);
    this.server_address = address;
  }

  /**
   * createDatabase
   */
  public void createDatabase() {
    db_alias.createTables();
  }

  /**
   * deleteDatabase
   */
  public void deleteDatabase() {
    db_alias.removeTables();
  }

  /**
   * redoDatabase
   */
  public void redoDatabase() {
    db_alias.redoTables();
  }

  /**
   * start
   */
  public boolean start() {
    if (System.getSecurityManager() == null) {
      System.setSecurityManager(new RMISecurityManager());
    }

    try {
      // Bind this object instance
      Naming.rebind(this.server_address, this);
      System.out.println("Server bound to \"" + this.server_address +
                         "\" in registry");
      return true;
    }
    catch (Exception ex) {
      System.err.println("Cannot bind server. " + ex.getMessage());
      ex.printStackTrace();
      return false;
    }
  }

  /**
   * stop
   */
  public void stop() {
    try {
      Naming.unbind(this.server_address);
    }
    catch (Exception ex){
      System.err.println("Error  unbinding server. " + ex.getMessage());
    }
  }


}
