package inescID.revs.servers;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.PasswordAuthentication;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

import inescID.util.*;
import inescID.revs.servers.*;
import inescID.revs.REVSConfiguration;
import inescID.crypto.FileEncryptionTool;
import inescID.revs.util.Server;
import java.security.*;
import java.security.*;
import java.security.cert.*;
import java.security.*;
import java.rmi.*;
import java.util.Vector;

public class StartServer {

  static final byte NO_SSL = 0;
  static final byte CONFIGURATION = 1;
  static final byte START = 2;
  static final String[] COMMANDS = {
      "noSSL",
      "configuration",
  };

  static final int DISTRIBUTOR = 0;
  static final int ADMINISTRATOR = 1;
  static final int ANONYMIZER = 2;
  static final int COUNTER = 3;

  static final String[] SERVERS = {
      "0 - Distributor",
      "1 - Administrator",
      "2 - Anonymizer",
      "3 - Counter"
  };

  static final int CREATE_DATABASE = 'C';
  static final int DELETE_DATABASE = 'D';
  static final int REDO_DATABASE = 'R';
  static final int UPDATE_DATABASE = 'U';
  static final int START_SERVER = 'S';
  static final int EXIT = 'E';

  static final int STOP_SERVER = 'S';

  static final String[] SERVER_BASIC_ACTIONS = {
      "C - Create database",
      "D - Delete database",
      "R - Redo database",
      "U - Update database",
      "S - Start server",
      "E - Exit",
      "S - Stop server"
  };

  static final int CREATE_SIGNING_KEYS = 'K';
  static final int FORWARD_COUNTER_SELECTION = 'F';
  static final int GATHER_VOTES = 'G';
  static final int TALLY_VOTES = 'T';


  protected String[][] SERVER_SPECIFIC_ACTIONS = {
      {},
      {"K - Create signing keys"},
      {"F - Forward Counter selection"},
      {
        "G - Gather votes",
        "T - Tally votes"
      }
  };


  protected boolean IsStarted = false;
  protected int server_type;

  PasswordAuthentication database_authentication = null;
  PasswordAuthentication keystore_authentication = null;

  boolean SSL = true;
  String configuration_file=null;

  InputTool input;

  public StartServer() {
    this.input = new InputTool();
  }

  public void noSSL(){
    this.SSL = false;
  }

  public void setConfigurationFile(String file){
    this.configuration_file = file;
  }

  public void start() throws FileNotFoundException, IOException, Exception,
      FileNotFoundException, IOException, Exception, Throwable {
    ReadConfigurationFile configuration = null;
    try {
      configuration = (configuration_file != null) ?
          new ReadConfigurationFile(configuration_file) :
          new ReadConfigurationFile();
    }
    catch (Exception ex) {
      System.err.println("Error reading configuration file.");
      System.exit(-1);
    }

   this.server_type = this.getServerType();

   PasswordAuthentication database_authentication = this.input.readUserPassword("Database", true);
   if (database_authentication==null)
     database_authentication = new PasswordAuthentication(
        REVSConfiguration.DB_DEFAULT_USER, REVSConfiguration.DB_DEFAULT_PASSWORD.toCharArray());


   char[] keystore_pass = new char[0];
   while (keystore_pass.length < 6)
     keystore_pass = this.input.readPassword("KeyStore");

   char[] key_pass = new char[0];
   while (key_pass.length < 6)
     key_pass = this.input.readPassword("Private key");

   startEngine(configuration, database_authentication, keystore_pass, key_pass);
 }

  private void startEngine(ReadConfigurationFile configuration,
                           PasswordAuthentication database_authentication,
                           char[] keystore_pass, char[] key_pass) throws
      UnrecoverableKeyException, IOException, NoSuchAlgorithmException,
      CertificateException, KeyStoreException, RemoteException, Exception {

    if (this.SSL) {
      System.setProperty("javax.net.ssl.trustStore", REVSConfiguration.TRUSTSTORE);
      System.setProperty("javax.net.ssl.trustStorePassword", new String(key_pass));
      System.setProperty("javax.net.ssl.keyStore", REVSConfiguration.PRIVATE_KEYSTORE);
      System.setProperty("javax.net.ssl.keyStorePassword", new String(key_pass));
    }

    ServerEngine server=null;
    switch(server_type){
      case DISTRIBUTOR:
        server = (this.SSL ? new DistributorEngine(configuration.getServerAddress(),
                                                   keystore_pass,key_pass) :
                  new DistributorEngine(configuration.getServerAddress(),
                                        keystore_pass,key_pass, true));
        break;
      case ADMINISTRATOR:
        server = (this.SSL ? new AdministratorEngine(configuration.getServerAddress(),
                                                   keystore_pass,key_pass) :
                  new AdministratorEngine(configuration.getServerAddress(),
                                        keystore_pass,key_pass, true));
      break;
      case ANONYMIZER:
        server = (this.SSL ?
                  new AnonymizerEngine(configuration.getServerAddress(),
                                              keystore_pass,key_pass)
                  :
                  new AnonymizerEngine(configuration.getServerAddress(),
                                        keystore_pass,key_pass, true));
        break;
      case COUNTER:
        server = (this.SSL ?
                  new CounterEngine(configuration.getServerAddress(),
                                              keystore_pass,key_pass)
                  :
                  new CounterEngine(configuration.getServerAddress(),
                                        keystore_pass,key_pass, true));
        break;
    }
    server.init(configuration.getDatabaseAddress(), database_authentication);
    this.serverActions(server);
  }


 byte getServerType() throws Exception {

   String line;
   int option;
   while (true) {
    try {
      System.out.println("Select server type");
      for (int i = 0; i < SERVERS.length; i++) {
        System.out.println(SERVERS[i]);
      }
      line = this.input.readLine("Server type: ");
      option = Integer.parseInt(line);
      if ( (option >= 0) && (option < SERVERS.length)) {
        return (byte) option;
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
   }
 }


 void serverActions(ServerEngine server) throws Exception {
   String line;
   int option;
   while(true){
     System.out.println("\n\n");
     if (!this.IsStarted){
       for (int i = 0; i < SERVER_SPECIFIC_ACTIONS[this.server_type].length; i++)
         System.out.println(SERVER_SPECIFIC_ACTIONS[this.server_type][i]);

       for (int i = 0; i < SERVER_BASIC_ACTIONS.length - 1; i++)
         System.out.println(SERVER_BASIC_ACTIONS[i]);
     }
     else
       System.out.println(SERVER_BASIC_ACTIONS[SERVER_BASIC_ACTIONS.length -1]);

     try {
       line = this.input.readLine("Option: ");
       if (line.length()!=1){
         System.out.println("Bad command.\n");
         continue;
       }
       option = line.toCharArray()[0];
     }
     catch (Exception ex) {
       continue;
     }

     if (this.IsStarted){
       if (option==STOP_SERVER){
         server.stop();
         this.IsStarted = false;
       }
       continue;
     }

     switch (option) {
       case CREATE_DATABASE:
         server.createDatabase();
         break;
       case DELETE_DATABASE:
         server.deleteDatabase();
         break;
       case REDO_DATABASE:
         server.redoDatabase();
         break;
       case UPDATE_DATABASE:
         line = this.input.readLine("Enter filename: ");

         server.updateDatabase(line);
         break;
       case START_SERVER:
         if (server.start()){
           this.IsStarted = true;
         } else {
           System.err.println("Error: unable to start the service.");
         }
         break;
       case EXIT:
         System.exit(1);
         break;
       default:
         handleSpecificCommands(server, option);
         break;
     }

   }
 }


void handleSpecificCommands(ServerEngine server, int option){

   switch(server_type){
     case DISTRIBUTOR:
       break;
     case ADMINISTRATOR:
       if (option == CREATE_SIGNING_KEYS)
       {
        try {
          String file = this.input.readLine("Enter elections file: ");
          FileEncryptionTool fet = new FileEncryptionTool();
          Vector elections = (Vector) fet.readObjectFromFile(file, server.commissioner_key);
          Vector keys = ((AdministratorEngine) server).createElectionsSigningKeys(elections);
          file = this.input.readLine("Save elections keys in file: ");
          fet.writeObject2File(file, keys, server.server_key);
        }
        catch (Exception ex) {
          System.err.println("Error creating elections signing keys.\n" + ex.getMessage());
        }
       }
       break;
     case ANONYMIZER:
       if (option == FORWARD_COUNTER_SELECTION)
       {
         try {
           String file = this.input.readLine("Enter counters info file: ");
           FileEncryptionTool fet = new FileEncryptionTool();
           Vector counters = (Vector) fet.readObjectFromFile(file,
               server.commissioner_key);

           Server counter;
           int selection = -1;
           String line;
           while ((selection < 0) || (selection > counters.size())){
             for (int i = 0; i < counters.size(); i++)
             {
               counter = (Server) counters.get(i);
               System.out.println( (i+1) + " - " + counter.GetDescription());
             }
             line = this.input.readLine("Counter: ");
            try {
              selection = Integer.parseInt(line);
            }
            catch (NumberFormatException ex1) {
              selection = -1;
            }
           }
           counter = (Server) counters.get(selection -1);
           ((AnonymizerEngine) server).setForwardServer(counter.GetURL());
           System.out.println("Active forward server: " + counter.GetDescription());
         }
         catch (Exception ex) {
           System.err.println(ex.getMessage());
         }
       }
       break;
     case COUNTER:
       switch (option){
         case GATHER_VOTES:
           try {
             String file = this.input.readLine("Enter counters info file: ");
             FileEncryptionTool fet = new FileEncryptionTool();
             Vector counters = (Vector) fet.readObjectFromFile(file,
                 server.commissioner_key);
             ((CounterEngine) server).gatherVotes(counters);
           }
           catch (Exception ex) {
             System.err.println("Error collecting votes.\n" +
                                ex.getMessage());
           }
           break;
         case TALLY_VOTES:
           ((CounterEngine) server).decipherVotes();
//           ((CounterEngine) server).publishResults();
           break;
       }
       break;
   }
 }


  public static void main(String[] args) throws FileNotFoundException,
      IOException, Exception, FileNotFoundException, IOException, Exception,
      Throwable {

    StartServer start_server = new StartServer();
    int i, j;

   for(i=0; i < args.length;) {
      for(j=0; j < COMMANDS.length; j++) {
        if(args[i].equals(COMMANDS[j])) break;
      }
      i++;
      switch (j) {
        case NO_SSL:
          start_server.noSSL();
          break;
        case CONFIGURATION:
          start_server.setConfigurationFile(args[i]);
          i++;
          break;
        default:
          System.out.println("Command help:");
          System.out.println(COMMANDS[0]);
          System.out.println(COMMANDS[1] + " [configuration_file]");
          System.exit(-1);
      }
    }
     start_server.start();
  }
}

