package inescID.revs.util;

import java.util.Vector;
import java.security.interfaces.RSAPublicKey;

/**
 * <p>Title: REVS</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

import inescID.crypto.KeyStoreTool;
import java.security.*;
import java.security.spec.InvalidKeySpecException;


public class VoteState extends Vote
    implements java.io.Serializable{

  AdministratorElectionAddress[] administrators;
  int req_signatures;
  byte[][] blindFactors;
  Vector submit_servers;
  byte[] election_key;

  public VoteState(Vote vote, Ballot ballot){


    this.setAnswers(vote.getAnswers());
    this.setIdElection(vote.getElection());
    this.setRandomValue(vote.getRandomValue());
    this.setSignatures(vote.getSignatures());
    this.administrators = ballot.getAdministrators();
    this.req_signatures = ballot.required_signatures;
    this.blindFactors = null;
    this.submit_servers = ballot.getSubmitServers();
    this.election_key = ballot.getElectionKey().getEncoded();

  }

public AdministratorElectionAddress[] getAdministratorElectionAddress(){
  return this.administrators;
}

public int getRequiredSignatures(){
  return this.req_signatures;
}

public byte[][] getBlindFactors(){
  return this.blindFactors;
}

public void setBlindFactors(byte[][] blind_factors){
  this.blindFactors = blind_factors;
}


public int numberOfSignatures(){
  int sig=0;
  for (int i=0; (this.signatures!=null) && (i< this.signatures.length); i++){
    if (this.signatures[i] != null)
      sig++;
  }
  return sig;
}

public Vote getVote(){
  return new Vote(this.id_election, this.xml_answers,
                  this.random_value, this.signatures);
}

public Vector getSubmitServers () {
  return this.submit_servers;
}

public RSAPublicKey getElectionKey () {
  try {
     KeyStoreTool kst = new KeyStoreTool();
     return kst.getRSAPublicKey(this.election_key);
   }
   catch (InvalidKeySpecException ex) {
     return null;
   }
   catch (NoSuchAlgorithmException ex) {
     return null;
   }
}


}
