package inescID.revs.voter;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Vector;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

import inescID.revs.displayBallot.*;
import inescID.revs.xml.ParsedBallot;

public class FrameVoter extends JFrame {
  PanelWelcome welcome = new PanelWelcome();
  PanelAuthentication authentication = new PanelAuthentication();
  PanelElectionList elections = new PanelElectionList();
  PanelBallot ballot_panel = new PanelBallot();
  PanelReport report = new PanelReport();
  JPanel current_panel;
  boolean portuguese = false;

  JPanel contentPane;
  JMenuBar jMenuBar1 = new JMenuBar();
  JMenu jMenuFile = new JMenu();
  JMenuItem jMenuFileExit = new JMenuItem();
  JMenu jMenuHelp = new JMenu();
  JMenuItem jMenuHelpAbout = new JMenuItem();
  BorderLayout borderLayout1 = new BorderLayout();
  PanelButtonsControl control = new PanelButtonsControl();
  JMenuItem jMenuItemResumeVoting = new JMenuItem();



  //Construct the frame
  public FrameVoter() {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  //Component initialization
  private void jbInit() throws Exception  {
    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(borderLayout1);
    this.setSize(new Dimension(800, 600));
    this.setTitle("REVS Voter Module");
    jMenuFile.setText("File");
    jMenuFileExit.setText("Exit");
    jMenuFileExit.addActionListener(new Frame1_jMenuFileExit_ActionAdapter(this));
    jMenuHelp.setText("Help");
    jMenuHelpAbout.setText("About");
    jMenuHelpAbout.addActionListener(new Frame1_jMenuHelpAbout_ActionAdapter(this));
    jMenuItemResumeVoting.setText("Resume Voting");
    jMenuFile.add(jMenuItemResumeVoting);
    jMenuFile.add(jMenuFileExit);
    jMenuHelp.add(jMenuHelpAbout);
    jMenuBar1.add(jMenuFile);
    jMenuBar1.add(jMenuHelp);
    this.setJMenuBar(jMenuBar1);
    contentPane.add(control, BorderLayout.SOUTH);
  }

  //File | Exit action performed
  public void jMenuFileExit_actionPerformed(ActionEvent e) {
    System.exit(0);
  }

  //Help | About action performed
  public void jMenuHelpAbout_actionPerformed(ActionEvent e) {
    FrameVoter_AboutBox dlg = new FrameVoter_AboutBox(this);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.setModal(true);
    dlg.pack();
    dlg.show();
  }

  //Overridden so we can exit when window is closed
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      jMenuFileExit_actionPerformed(null);
    }
  }

  private void setPanel(JPanel panel){
    if(this.current_panel!= null){
      this.current_panel.setVisible(false);
      this.getContentPane().remove(this.current_panel);
    }
    this.current_panel=panel;
    this.getContentPane().add(panel, BorderLayout.CENTER);
    this.current_panel.setVisible(true);
  }

  public void setWelcomeMessage(String msg){
    this.welcome.setWelcomeMessage(msg);
  }

  public void setWelcomePanel(){
    this.control.setOk();
    this.setPanel(this.welcome);
  }

  public void setAuthenticationPanel(){
    this.control.setOkCancel();
    this.setPanel(this.authentication);
  }

  public String getVoter(){
    return this.authentication.getVoter();
  }

  public char[][] getPasswords(){
    return this.authentication.getPasswords();
  }


  public void setElectionsPanel(Vector elections){
    this.control.setOkCancel();
    this.elections.setElections(elections);
    this.setPanel(this.elections);
  }

  public int getSelectedElection(){
   return this.elections.getSelectedElection();
 }

  public void setBallotPanel(){
    this.control.setOkCancel();
    this.setPanel(this.ballot_panel);
  }


  public void setPortuguese(){
    this.portuguese = true;
  }

  public void setBallot(ParsedBallot ballot){
    this.ballot_panel.setBallot(ballot);
    if (this.portuguese)
      this.ballot_panel.setPortuguese();

  }

  public void setBallotElectionDescription(String description){
    this.ballot_panel.setElectionDescription(description);
  }

  public ParsedBallot getAnswers(){
    return this.ballot_panel.getAnswers();
  }


  public void setReportPanel(String msg){
    this.control.setOkElections();
    this.report.setReport(msg);
    this.setPanel(this.report);
  }

  public void setActionListener(IVoterEngine app) {
   this.control.setActionListener(app);
   jMenuItemResumeVoting.addActionListener(new FrameVoter_jMenuItemResumeVoting_actionAdapter(app));

 }

 public void newMessage(String msg){
   JOptionPane.showMessageDialog(this, msg, "Message", JOptionPane.INFORMATION_MESSAGE);
 }


 public void newErrorMessage(String msg){
    JOptionPane.showMessageDialog(this, msg, "Error", JOptionPane.ERROR_MESSAGE);
  }

  public void newDialogWarningMessage(String title, String msg){
    JOptionPane.showMessageDialog(this, msg, title, JOptionPane.WARNING_MESSAGE);
  }

  public boolean newYesNoQuestionDialog(String title, String msg){
    JTextArea text = new JTextArea();
    text.setEditable(false);
    text.setBackground(SystemColor.control);
    text.setText(msg);
    int option = JOptionPane.showConfirmDialog(this, text, title,
                                               JOptionPane.YES_NO_OPTION,
                                               JOptionPane.QUESTION_MESSAGE);
    if (option==JOptionPane.YES_OPTION)
      return true;
    return false;
   }

   public boolean newSimNaoQuestionDialog(String title, String msg){
       JTextArea text = new JTextArea();
       text.setEditable(false);
       text.setBackground(SystemColor.control);
       text.setText(msg);

       Object[] options = {"Sim", "No"};
       int option = JOptionPane.showOptionDialog(this, text, title,
                                                  JOptionPane.YES_NO_OPTION,
                                                  JOptionPane.QUESTION_MESSAGE,
                                                  null,
                                                  options,
                                                  options[1]);
       if (option==JOptionPane.YES_OPTION)
         return true;
       return false;
     }



     public char[][] newInputPasswordDialog(String title) {
       PanelAuthentication panel = new PanelAuthentication();
       panel.setJustPasswords();
       /*
       JTextArea text = new JTextArea();
       text.setEditable(false);
       text.setBackground(SystemColor.control);
       text.setText(msg);
       JPasswordField password = new JPasswordField();
       JPasswordField pin = new JPasswordField();
       VerticalFlowLayout borderLayout = new VerticalFlowLayout();
       panel.setLayout(borderLayout);
       panel.add(text, BorderLayout.CENTER);
       panel.add(password, BorderLayout.SOUTH);
*/
       int option = JOptionPane.showConfirmDialog(this, panel, title,
                                                  JOptionPane.OK_CANCEL_OPTION,
                                                  JOptionPane.PLAIN_MESSAGE);

       if (option==JOptionPane.CANCEL_OPTION)
         return null;

       return panel.getPasswords();
     }

   public String fileDialog(String title, int mode) {
     FileDialog f = new FileDialog(this, title,
                                   (mode == FileDialog.LOAD ? FileDialog.LOAD : FileDialog.SAVE));
     f.show();
     String file = f.getDirectory() + f.getFile();
     if((f.getDirectory() != null) && (f.getFile() != null))
       return file;
     return null;
   }




}








class Frame1_jMenuFileExit_ActionAdapter implements ActionListener {
  FrameVoter adaptee;

  Frame1_jMenuFileExit_ActionAdapter(FrameVoter adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.jMenuFileExit_actionPerformed(e);
  }
}

class Frame1_jMenuHelpAbout_ActionAdapter implements ActionListener {
  FrameVoter adaptee;

  Frame1_jMenuHelpAbout_ActionAdapter(FrameVoter adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.jMenuHelpAbout_actionPerformed(e);
  }
}

class FrameVoter_jMenuItemResumeVoting_actionAdapter implements java.awt.event.ActionListener {
  IVoterEngine adaptee;

  FrameVoter_jMenuItemResumeVoting_actionAdapter(IVoterEngine adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.jMenuItemResumeVoting_actionPerformed(e);
  }
}
