package inescID.revs.voter;

import java.rmi.*;
import java.security.*;
import java.security.interfaces.*;
import java.util.*;
import javax.crypto.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import inescID.crypto.*;
import inescID.crypto.blindSignatures.*;
import inescID.revs.*;
import inescID.revs.servers.*;
import inescID.revs.util.*;
import inescID.revs.util.protocol.*;
/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */
import inescID.revs.xml.*;


public class VoterEngine
    implements IVoterEngine{

  static final int WELCOME = 0;
  static final int AUTHENTICATION = 1;
  static final int ELECTION_LIST = 2;
  static final int BALLOT_VIEW = 3;
  static final int REQUIRE_SIGNTURES_NO_STATE = 4;
  static final int REQUIRE_SIGNTURES_WITH_STATE = 5;
  static final int SUBMIT_NO_STATE = 6;
  static final int SUBMIT_WITH_SIGNATURES_STATE = 7;
  static final int SUBMIT_BEFORE_SINGATURES_STATE = 8;
  static final int REPORT = 9;


  private boolean demo_mode = false;
  private String demo_ballot;

  int state = 0;
  Vector elections;
  Vector elections_id_description;
  int election_index;
  int voter;
  byte[][] passwords;
  boolean SAVE_STATE;
  RSAPublicKey commissioner_key;
  PasswordGenerator pass_gen;
  SigningTool sign;
  REVSParser parser = new REVSParser();
  RSABlindSignature blind_singature_engine;
  EncryptionTool encrypt;
  KeyStoreTool kst;
  Ballot ballot;
  String report="";

  FrameVoter frame_voter;
  boolean packFrame = false;

  boolean resume_voting = false;

  FileEncryptionTool fet;
  Vector distributors;
  Server distributor;

  public VoterEngine(String ballot) {
    this.demo_mode=true;
    this.demo_ballot = ballot;
    start();
  }

  public VoterEngine() {
    start();
  }

  //Construct the application
  private void start() {
    System.setProperty("javax.net.ssl.trustStore", REVSConfiguration.TRUSTSTORE);
    System.setProperty("javax.net.ssl.trustStorePassword",
                       new String(REVSConfiguration.DEFAULT_KEYSTORE_PASS));

    try {
      fet = new FileEncryptionTool();
    }
    catch (Exception ex2) {
      ex2.printStackTrace();
      System.exit(-1);
    }

    /**@todo chave voter*/

 /*
    System.setProperty("javax.net.ssl.keyStore",
                       REVSConfiguration.PRIVATE_KEYSTORE);
    System.setProperty("javax.net.ssl.keyStorePassword",
                       new String(REVSConfiguration.DEFAULT_KEYSTORE_PASS));
    System.setProperty("java.security.policy", "policy.txt");
*/

    frame_voter = new FrameVoter();
    //Validate frames that have preset sizes
    //Pack frames that have useful preferred size info, e.g. from their layout
    if (packFrame) {
      frame_voter.pack();
    }
    else {
      frame_voter.validate();
    }
    //Center the window
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = frame_voter.getSize();
    if (frameSize.height > screenSize.height) {
      frameSize.height = screenSize.height;
    }
    if (frameSize.width > screenSize.width) {
      frameSize.width = screenSize.width;
    }
    frame_voter.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);


    String welcome;
    try {
      welcome = fet.readTextFile(REVSConfiguration.WELCOME_MESSAGE_FILE);
    }
    catch (Exception ex1) {
      welcome = "<P></br><H3 align=center>Welcome to REVS voting system</H3></P>";
    }

    frame_voter.setWelcomeMessage(welcome);
    frame_voter.setWelcomePanel();
    this.state = WELCOME;
    this.frame_voter.setActionListener(this);

    try {
      this.pass_gen = new PasswordGenerator();
      this.sign = new SigningTool();
      this.blind_singature_engine = new RSABlindSignature();
      this.encrypt = new EncryptionTool();
      kst = new KeyStoreTool();
      this.commissioner_key = (RSAPublicKey) kst.getPublicKeyFromCertFile(REVSConfiguration.COMMISSIONER_CERT_FILE);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    /**@todo parametro de aplicacao*/
    SAVE_STATE = true;
    frame_voter.setVisible(true);
  }

  public void elections_actionPerformed(ActionEvent e) {
    this.report = "";
    this.frame_voter.setElectionsPanel(this.elections_id_description);
    this.state = ELECTION_LIST;

  }

  public void ok_actionPerformed(ActionEvent e) {
    switch(this.state){
      case WELCOME:
        this.frame_voter.setAuthenticationPanel();
        if (this.demo_mode){
          try {
            this.report = "DEMO MODE.\n\n";
            ParsedBallot p_ballot = parser.parseBallotFile(this.demo_ballot);
            this.frame_voter.setBallot(p_ballot);

            this.frame_voter.setBallotElectionDescription("Demo Ballot");
            this.frame_voter.setBallotPanel();
            this.state = BALLOT_VIEW;
          }
          catch (Exception ex) {
            ex.printStackTrace();
          }
          break;
        }
        else{
          this.state = AUTHENTICATION;
          break;
        }
      case AUTHENTICATION :
        if (this.resume_voting){
          this.election_index = -1;
          this.resumeVoting();
        }
        else{
          if (!this.verifyVoter())
            break;
          this.state = ELECTION_LIST;
        }
        break;
      case ELECTION_LIST:
        this.election_index = this.frame_voter.getSelectedElection();
        if (election_index==-1)
          break;
        IdDescription election =(IdDescription) this.elections.get(election_index);
        this.frame_voter.setBallotElectionDescription(election.GetDescription());
        if (!getBallot(election.GetId()))
          break;
        this.report = "\n\n" + ((IdDescription) this.elections.get(election_index)).GetDescription();
        this.report += "\n";
        this.frame_voter.setBallotPanel();
        this.state = BALLOT_VIEW;
        break;
      case BALLOT_VIEW:
        if(! this.frame_voter.newYesNoQuestionDialog("Validate",
                                                   "Validate vote?"))
          return;

        ParsedBallot parsed_ballot = this.frame_voter.getAnswers();
        String xml_answers = this.parser.getXMLanswers(parsed_ballot);
        if (this.demo_mode){
          this.report += "\n" + xml_answers;
          this.report += "\n\nDEMO MODE.";
        }
        else{
          Vote vote = new Vote(ballot.getElection(), xml_answers,
                               null, null);
          VoteState vs = new VoteState(vote, ballot);
          if(!this.submitVote(vs))
            report += "\n\nVOTE NOT SUBMITTED";
        }
        this.frame_voter.setReportPanel(report);
        this.state = REPORT;
        break;
      case REPORT:
        this.report = "";
        this.frame_voter.setWelcomePanel();
        this.state = WELCOME;
        break;
    }

  }

  private void createPasswords(char[][] passwords) throws Exception {
    this.passwords = ((passwords[1] == null) ?
                      pass_gen.GenerateFinalPasswordArray(
      REVSConfiguration.MAX_SERVERS + 2, passwords[0])
                      :
                      pass_gen.GenerateFinalPasswordArray(
      REVSConfiguration.MAX_SERVERS + 2, passwords[0], passwords[1]));
  }


  protected boolean readAuthentication(){
    char[][] passwords = this.frame_voter.getPasswords();
    String voter = this.frame_voter.getVoter();
    try {
      this.voter = Integer.parseInt(voter);
      this.createPasswords(passwords);
    }
    catch (Exception ex) {
      this.voter = -1;
    }
    if ((this.voter == -1) || (this.passwords == null)){
      this.frame_voter.newErrorMessage("Voter Id or password error.");
      return false;
    }
    return true;
  }



  public boolean verifyVoter() {
    if (!this.readAuthentication()) return false;

    distributors = null;
    try {
      distributors = (Vector) fet.readObjectFromFile(REVSConfiguration.DISTRIBUTORS_FILE,
                             this.commissioner_key);
    }
    catch (Exception ex1) {
      this.frame_voter.newErrorMessage("Distributors file corrupted.");
      return false;
    }

    try {

      Reply reply = null;
      for (int i=0; i < distributors.size(); i++){
        distributor = (Server) distributors.get(i);

        try {
          IDistributor dist = (IDistributor) Naming.lookup(distributor.GetURL());
          reply = dist.getElections(this.voter);
          break;
        }
        catch (Exception ex2) {
          continue;
        }
      }

      if (reply==null){
        this.frame_voter.newErrorMessage("No ballot distributor is reachable.");
        return false;
      }


      if(reply.getErrorCode()!=ErrorCodes.OK){
        this.frame_voter.newErrorMessage(ErrorCodes.ERROR_DESCRIPTION[reply.getErrorCode()]);
        return false;
      }
      VoterElection ve = (VoterElection) reply.getObject();
      boolean verify = this.frame_voter.newYesNoQuestionDialog("Voter",
                                              "VOTER VERIFICATION:\nVoter id: " +
                                              ve.GetId() + "\nVoter name: " +
                                              ve.GetDescription() +
                                              "\n\nIs this information correct?");
      if (!verify)
        return false;
      this.elections = ve.GetElections();
      Vector aux = new Vector(this.elections.size());
      IdDescription election;
      for(int i=0; i<this.elections.size(); i++){
        election = (IdDescription) this.elections.get(i);
        aux.add(election.GetDescription());
      }
      this.elections_id_description = aux;
      this.frame_voter.setElectionsPanel(this.elections_id_description);
      return true;
    }
    catch (Exception ex) {
      this.frame_voter.newErrorMessage("Error contaction ballot distributor. " +
                                       ex.getMessage());
      ex.printStackTrace();
      return false;
    }
  }

  public boolean getBallot(int election) {
    try {

      Reply reply = null;
      IDistributor dist;

      try {
        dist = (IDistributor) Naming.lookup(distributor.GetURL());
        reply = dist.getElectionBallot(this.voter, election);
      }
      catch (Exception ex3) {
        reply = null;
      }

      if (reply == null) {
        for (int i = 0; i < distributors.size(); i++) {
          distributor = (Server) distributors.get(i);

          try {
            dist = (IDistributor) Naming.lookup(distributor.GetURL());
            reply = dist.getElections(this.voter);
            break;
          }
          catch (Exception ex2) {
            reply = null;
            continue;
          }
        }
      }

      if (reply == null) {
        this.frame_voter.newErrorMessage("No ballot distributor is reachable.");
        return false;
      }


      if (reply.getErrorCode() != ErrorCodes.OK) {
        this.frame_voter.newErrorMessage(ErrorCodes.ERROR_DESCRIPTION[reply.
                                         getErrorCode()]);
        return false;
      }
      SignedObject so = (SignedObject) reply.getObject();
      if (!this.sign.verifyObject(this.commissioner_key, so)) {
        this.frame_voter.newErrorMessage("Cannot verify ballot signature.");
        return false;
      }
      ballot = (Ballot) so.getObject();
      if (ballot.getElection() != election){
        this.frame_voter.newErrorMessage("Ballot not valid.");
        return false;
      }
      try {
        ParsedBallot p_ballot = parser.parseBallotString(ballot.getBallot());
        this.frame_voter.setBallot(p_ballot);
      }
      catch (Exception ex1) {
        this.frame_voter.newErrorMessage("Cannot parse ballot." +
                                         ex1.getMessage());
        return false;
      }
      return true;
    }
    catch (Exception ex) {
      this.frame_voter.newErrorMessage("Error contaction ballot distributor. " +
                                       ex.getMessage());
      ex.printStackTrace();
      return false;
    }

  }


  public boolean submitVote(VoteState vote_state){

    AdministratorElectionAddress[] administrators;
    AdministratorSignature[] signatures;
    byte[][] blind_factors;

    this.report += "\n";

    if (vote_state.numberOfSignatures()==0 && (!this.resume_voting)){//first time. Set initial data
      administrators = vote_state.getAdministratorElectionAddress();

      signatures = new AdministratorSignature[administrators.length];
      vote_state.setSignatures(signatures);

      vote_state.setRandomValue(this.pass_gen.CreateRandomBytes(
          REVSConfiguration.COMMITMENT_LEGTH));

      blind_factors = new byte[administrators.length][];
      AdministratorElectionAddress adm;
      for (int i=0; i<blind_factors.length; i++){
        adm = administrators[i];
        try {
          this.blind_singature_engine.initBlindEngine(kst.getRSAPublicKey(adm.GetKey()));
          blind_factors[i] = this.blind_singature_engine.generateBlindFactorBytes();
        }
        catch (Exception ex1) {
          this.report += "\nError creating submition package: 1";
          return false;
        }
      }
      vote_state.setBlindFactors(blind_factors);


      this.saveState(vote_state);
    }
    else{
      administrators = vote_state.getAdministratorElectionAddress();
      signatures = vote_state.getSignatures();
      blind_factors = vote_state.getBlindFactors();
    }

    byte[] vote_bytes=null;
    byte[]vote_hash;
    byte[] committed_vote;
    try {
      vote_bytes = vote_state.getAnswers().getBytes(REVSConfiguration.CHARSET);
      vote_hash = this.pass_gen.getHashValue(vote_bytes);
      committed_vote = this.pass_gen.getCommittedHashValue(vote_hash,
          vote_state.getRandomValue());
    }
    catch (Exception ex2) {
      this.report += "\nError creating submition package: 2";
      return false;
    }

    int number_of_signatures = vote_state.numberOfSignatures();

    byte[] blind_vote;
    byte[] unblinded_signature;
    byte[] blind_signature;
    Reply reply;
    IAdministrator adm;

    int new_signatures = 0;
    boolean authentication = true;

    for (int i = 0; (number_of_signatures < vote_state.getRequiredSignatures())
         && (i < administrators.length); i++) {
      try {
        if (signatures[i] == null) {
          adm = (IAdministrator) Naming.lookup(administrators[i].
                                               getAdministratorURL());
          this.blind_singature_engine.initBlindEngine(
              kst.getRSAPublicKey(administrators[i].GetKey()));
          blind_vote = this.blind_singature_engine.blindMessage(committed_vote,
              blind_factors[i]);
          reply = adm.sign(this.voter,
                           this.passwords[administrators[i].GetAdministrator() -
                           1],
                           vote_state.getElection(), blind_vote);

          if ( (reply.getErrorCode() != ErrorCodes.OK) &&
              (reply.getErrorCode() != ErrorCodes.PREVIOUS_SIGNATURE)) {
           // this.frame_voter.newErrorMessage(ErrorCodes.ERROR_DESCRIPTION[reply.
           //                                  getErrorCode()]);

            if (reply.getErrorCode() == ErrorCodes.ERROR_AUTHENTICATION)
              authentication = false;

            report += "\nAdministrator " +
                administrators[i].getAdministratorDescription() +
                " reply: " + ErrorCodes.ERROR_DESCRIPTION[reply.getErrorCode()];
            continue;
          }
          authentication = true;
          report += "\nAdministrator: " +
              administrators[i].getAdministratorDescription() +
              " reply: " + ErrorCodes.ERROR_DESCRIPTION[reply.getErrorCode()];

          blind_signature = (byte[]) reply.getObject();
          unblinded_signature = this.blind_singature_engine.unblindSignature(
              blind_signature,
              blind_factors[i]);
          this.blind_singature_engine.initVerify(
              kst.getRSAPublicKey(administrators[i].GetKey()));
          if (this.blind_singature_engine.verifyBlindSignature(committed_vote,
              unblinded_signature)) {
            signatures[i] = new AdministratorSignature(administrators[i].
                GetAdministrator(),
                unblinded_signature);
            number_of_signatures++;
            report += "\nSignature verification: OK";
            new_signatures++;
          }
          else
            report += "\nSignature verification: ERROR";
        }
        else {
          report += "\nAdministrator: " +
              administrators[i].getAdministratorDescription() +
              " saved signature.";
        }

      }
      catch (Exception ex) {
        /*
            this.frame_voter.newErrorMessage(
            "Error processing administrator. " +
            ex.getMessage());
            ex.printStackTrace();
        */
       report += "\nError processing administrator: " +
            administrators[i].getAdministratorDescription() + ".";
      }
    }

    if (new_signatures > 0){
      vote_state.setSignatures(signatures);

      this.saveState(vote_state);
    }

    if(!authentication){
      for (int i=0; i<3; i++){
          char[][] passwords = new char[2][];
          this.frame_voter.newErrorMessage("Authentication error: wrong password.");
          passwords = this.frame_voter.newInputPasswordDialog("Insert correct password");
          if (passwords==null)
            return false;
          try {
            this.createPasswords(passwords);
            return this.submitVote(vote_state);
          }
          catch (Exception ex4) {
            ex4.printStackTrace();
            continue;
          }
        }

      return false;
    }

    if (number_of_signatures < vote_state.getRequiredSignatures()) {
      this.frame_voter.newErrorMessage("Not enough signatures.");
      return false;
    }

    if(! this.frame_voter.newYesNoQuestionDialog("Submit",
                                                 "Submit vote?")){
      this.report += "\n\nSubmission aborted by the voter.";
      return false;
    }


      Vote vote = vote_state.getVote();
      byte[] encrypted_vote = null;
      try {
        encrypted_vote = this.encrypt.ObjectToByteArray(
            this.encrypt.EncryptMixCipherMode(vote, vote_state.getElectionKey()));
      }
      catch (Exception ex3) {
        this.report += "\nError3 creating submition package.";
        return false;
      }
      ISubmission submit_server;
      Vector submit = vote_state.getSubmitServers();
      Server server=null;
      boolean submitted = false;

      for(int i=0; i < submit.size(); i++){
        try{
          server = (Server) submit.get(i);
          submit_server = (ISubmission) Naming.lookup(server.GetURL());
          reply = submit_server.submitVote(vote_state.getElection(),
                                           encrypted_vote);
          if(reply.getErrorCode() == ErrorCodes.OK)
            submitted = true;

          report += "\nSubmit server " + server.GetDescription() + " reply: " +
              ErrorCodes.ERROR_DESCRIPTION[reply.getErrorCode()];
        }
        catch(Exception ex){
          report += "\nError processing submit server: " + server.GetDescription() + ".";
        }
      }
      if (submitted == false) {
        this.frame_voter.newErrorMessage("Error submiting vote.");
        report += "\n\nVOTE NOT SUBMITTED";
      }
      else {
        report += "\n\nVOTE SUBMITTED";
        if(this.election_index != -1){
          this.elections.remove(this.election_index);
          this.elections_id_description.remove(this.election_index);
          this.election_index = -1;
        }
      }

      return submitted;
    }


private SecretKey getStatePassword() throws Exception {
  byte[] passwd = new byte[24];
  System.arraycopy(this.passwords[this.passwords.length-2], 0, passwd, 0,
                   this.passwords[this.passwords.length-2].length);
  System.arraycopy(this.passwords[this.passwords.length-1], 0, passwd,
                   this.passwords[this.passwords.length-2].length -1,
                   24 - this.passwords[this.passwords.length-2].length);
  return this.encrypt.getTripleDESkey(passwd);
}


    public void saveState(VoteState state){
      try {
        SecretKey key = this.getStatePassword();
        String file_name = this.frame_voter.fileDialog("Save voting state", FileDialog.SAVE);
        if (file_name == null)
          return;
        FileEncryptionTool file = new FileEncryptionTool();
        file.writeObject2File(file_name, state, key);
      }
      catch (Exception ex) {
        this.frame_voter.newErrorMessage("Error: unable to save voting state.");
      }

    }

    public void resumeVoting(){
      this.readAuthentication();

      report += "\nResume voting.";
      VoteState vs = null;
      try {
        SecretKey key = this.getStatePassword();
        String file_name = this.frame_voter.fileDialog("Load voting state",
            FileDialog.LOAD);
        FileEncryptionTool file = new FileEncryptionTool();
        Object obj = file.readObjectFromFile(file_name, key);
        vs = (VoteState) obj;

        boolean ok = this.submitVote(vs);
        if(!ok)
          report += "\nVOTE NOT SUBMITTED";
      }
      catch (Exception ex) {
        this.frame_voter.newErrorMessage("Error: unable to load voting state.");
        report += "\nError: unable to load voting state.";
        ex.printStackTrace();
      }
      this.resume_voting = false;
      this.frame_voter.setReportPanel(report);
      this.state = REPORT;
    }


  public void cancel_actionPerformed(ActionEvent e) {
    switch(this.state){
      case AUTHENTICATION :
        this.frame_voter.getPasswords();
        this.frame_voter.getVoter();
        this.frame_voter.setWelcomePanel();
        this.state = WELCOME;
        break;
      case ELECTION_LIST:
        this.frame_voter.setWelcomePanel();
        this.state = WELCOME;
        break;
      case BALLOT_VIEW:
        this.frame_voter.setElectionsPanel(this.elections_id_description);
        this.state = ELECTION_LIST;
        break;
    }

  }


  public void jMenuItemResumeVoting_actionPerformed(ActionEvent e) {

    if (this.state == WELCOME){
      this.resume_voting = true;
      this.ok_actionPerformed(null);
    }
 }




  //Main method
  public static void main(String[] args) {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    if(args.length > 0)
      new VoterEngine(args[0]);
    else
      new VoterEngine();
  }
}
