package inescID.revs.xml;

import javax.xml.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.sax.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;
import java.io.IOException;
import org.xml.sax.SAXException;
import java.io.ByteArrayInputStream;
import java.util.Vector;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company Inesc-ID</p>
 * @author Rui Joaquim
 * @version 1.0
 * @license REVS is freely available under the GNU GENERAL PUBLIC LICENCE
 */

public class REVSParser {

  static final int OPEN_ANSWER_CODE = 0;

  public REVSParser() {
  }

  public static void main(String[] args) throws SAXException, IOException,
      ParserConfigurationException, FactoryConfigurationError {
    REVSParser ballotParser1 = new REVSParser();
    //ParsedBallot ballot = ballotParser1.parseBallotFile("/ficheiros/ballot.xml");
    ballotParser1.getXMLanswers(null);

  }


  public ParsedBallot parseBallotFile(String file) throws FactoryConfigurationError,
      ParserConfigurationException, IOException, SAXException {
    DocumentBuilder docB = DocumentBuilderFactory.newInstance().
        newDocumentBuilder();
    Document doc = docB.parse(file);
    return this.parseBallot(doc);
  }

  public ParsedBallot parseBallotString(String ballot) throws FactoryConfigurationError,
      ParserConfigurationException, IOException, SAXException {
    DocumentBuilder docB = DocumentBuilderFactory.newInstance().newDocumentBuilder();
    Document doc = docB.parse(new ByteArrayInputStream(ballot.getBytes()));
    return this.parseBallot(doc);
  }

  ParsedBallot parseBallot(Document doc){
    ParsedBallot parsed_ballot = new ParsedBallot();
    NodeList nl = doc.getElementsByTagName("ballot");
    Node ballot = nl.item(0);
    Node attribute;
    NamedNodeMap attribs;
    attribs = ballot.getAttributes();
    attribute = attribs.getNamedItem("electionCode");
    parsed_ballot.setElection(Integer.parseInt(attribute.getNodeValue()));
    for (Node child = ballot.getFirstChild(); child != null; child = child.getNextSibling()) {
      String name = child.getNodeName();
      String value = child.getNodeValue();
      name = child.getLocalName();
      if (child.getNodeName() == "ballotDescription") {
        parsed_ballot.setDescription(this.parseBallotDescription(child));
        continue;
      }
      if (child.getNodeName() == "group") {
        parsed_ballot.addGroup(this.parseGroup(child));
        continue;
      }
    }
    return parsed_ballot;
  }

  Vector parseBallotDescription(Node description){
    Vector description_lines = new Vector();
    Node line;
    for (Node child = description.getFirstChild(); child != null; child = child.getNextSibling()) {
      if (child.getNodeName() == "line") {
        line = child.getFirstChild();
        description_lines.add(line.getNodeValue());
        continue;
      }
    }
    return description_lines;
  }

  BallotGroup parseGroup(Node group){
    BallotGroup ballot_group = new BallotGroup();
    NamedNodeMap attribs;
    Node attribute;
    attribs = group.getAttributes();
    attribute = attribs.getNamedItem("code");
    ballot_group.setGroupId(Integer.parseInt(attribute.getNodeValue()));
    attribute = attribs.getNamedItem("description");
    ballot_group.setDescription(attribute.getNodeValue());

    for (Node child = group.getFirstChild(); child != null; child = child.getNextSibling()) {
      if (child.getNodeName() == "question") {
        BallotQuestion question = this.parseQuestion(child);
        attribs = child.getAttributes();
        attribute = attribs.getNamedItem("code");
        question.setCode(Integer.parseInt(attribute.getNodeValue()));
        attribute = attribs.getNamedItem("type");
        question.setType(attribute.getNodeValue());
        ballot_group.addQuestion(question);
        continue;
      }
      if (child.getNodeName() == "group") {
       ballot_group.addSubgroup(this.parseGroup(child));
       continue;
     }
    }
    return ballot_group;
  }

  BallotQuestion parseQuestion(Node question){
    NamedNodeMap attribs;
    Node attribute;
    Node aux;
    BallotQuestion ballot_question = new BallotQuestion();
    QuestionOption option;
    for (Node child = question.getFirstChild(); child != null; child = child.getNextSibling()) {
     if (child.getNodeName() == "questionDescription") {
       aux = child.getFirstChild();
       ballot_question.setDescription(aux.getNodeValue());
       continue;
     }
     if (child.getNodeName() == "answer") {
       option = new QuestionOption();
       attribs = child.getAttributes();
       attribute = attribs.getNamedItem("code");
       option.setCode(Integer.parseInt(attribute.getNodeValue()));
       aux = child.getFirstChild();
       option.setOption(aux.getNodeValue());
       ballot_question.addOption(option);
       continue;
     }
   }
   return ballot_question;
  }


  public String getXMLanswers(ParsedBallot ballot){
      String answers = "<ballotAnswer electionCode=\"" + ballot.election + "\">\n";
      Vector groups = ballot.getGroups();
      BallotGroup group;
      for(int g=0; g<groups.size(); g++){
        group = (BallotGroup) groups.get(g);
        answers += this.getXMLanswersGroup(group);
      }
      answers += "</ballotAnswer>\n";
    //  System.out.println(answers);
      return answers;
    }

    String getXMLanswersGroup(BallotGroup group){
      BallotQuestion question;
      Vector questions;
      String answers = "<group code=\"" + group.getGroupId() +"\">\n";
      questions = group.getQuestions();
      for(int q=0; q < questions.size(); q++){
        try {
          question = (BallotQuestion) questions.get(q);
          answers += this.getXMLanswersQuestion(question);
        }
        catch (Exception ex) {
          BallotGroup subgroup;
          subgroup = (BallotGroup) questions.get(q);
          answers += this.getXMLanswersGroup(subgroup);
        }
      }
      answers += "</group>\n";
      return answers;
    }

    String getXMLanswersQuestion(BallotQuestion question){
      QuestionAnswer question_answer;
      question_answer = question.getAnswer();
      String answers = "<question code=\"" + question.getCode() + "\" " +
          "answer=\"" + question_answer.getStringAnswers() + "\">";
      if (question_answer.getOpenAnswer() != null)
        answers += question_answer.getOpenAnswer();
      answers += "</question>\n";
      return answers;
    }



/*
 <!-- General ballot answer example -->
 <ballotAnswer electionCode="123">
     <group code="1">
         <!-- Single  type -->
         <question code="1" answer="1"></question>
         <!-- Multiple  type -->
         <question code="2" answer="1-3-7"></question>
         <! OpenS  type (selecting the open answer)-->
         <question code="3" answer="0">Open answer</question>
         <! OpenS  type (selecting another answer )-->
         <question code="4" answer="5"></question>
         <! OpenM  type -->
         <question code=5" answer=0-2-4">Open answer</question>
         <!-- more answers -->
     </group>
     <!-- more groups -->
 </ballotAnswer>


*/

}
